/*
 * Copyright (c) 2006-2007 Solexa
 * Author: Klaus Maisinger
 *
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */

#include <cstdlib>
#include <cstring>

#include "BaseCallParamsXMLReader.h"

BaseCallParamsXMLReader::BaseCallParamsXMLReader()
{
    inParameters = 0;
    inPhasing = inPrephasing = 0;
}

BaseCallParamsXMLReader::~BaseCallParamsXMLReader()
{
}

void BaseCallParamsXMLReader::startDocument()
{
    params.PhasingRate    = 0.;
    params.PrephasingRate = 0.;
}

void BaseCallParamsXMLReader::startElement(const char name[], 
                                           const Attributes *attr)
{
    if (!strcmp(name, "Parameters"))
        inParameters = 1;
    if (!strcmp(name, "Phasing"))
        inPhasing = 1;
    if (!strcmp(name, "Prephasing"))
        inPrephasing = 1;
}        

void BaseCallParamsXMLReader::endElement(const char name[])
{
    if (!strcmp(name, "Parameters"))
        inParameters = 0;
    if (!strcmp(name, "Phasing"))
        inPhasing = 0;
    if (!strcmp(name, "Prephasing"))
        inPrephasing = 0;
}        

void BaseCallParamsXMLReader::characters(const char ch[], int start, int length)
{
    if (!inParameters)
        return;
    if (inPhasing)
        params.PhasingRate    = atof(ch);
    if (inPrephasing)
        params.PrephasingRate = atof(ch);
}

void BaseCallParamsXMLReader::getParameters(struct BasecallParameters *params2)
{
    params2->PhasingRate    = params.PhasingRate;
    params2->PrephasingRate = params.PrephasingRate;
}
