/*
 * Copyright (c) Illumina 2008
 * Author:       Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
#ifndef CYCLEMATRIX_H_
#define CYCLEMATRIX_H_

#include <vector>
#include <cstdio>
#include "ParameterTypes.h"

class CycleMatrix : public std::vector< float * >
{
private:
    int m_numChannels;
public:
    CycleMatrix(const int numChannels = 0);
    ~CycleMatrix();
    void clear();
    int  estimate(const std::vector<float * > & cycleData,
                  const int                     numObjects,
                  const int                     numChannels);
    int print(const TileInfo    & tileInfo,
              const ImageRegion & imageRegion,
              FILE              * fout = stdout);
    int write(const char          filename[],
              const TileInfo    & tileInfo,
              const ImageRegion & imageRegion);
    int read(const char filename[]);
    int invert();
    int apply(float     ** cycleData,
              const int    numObjects,
              const int    numChannels,
              const int    numDataCycles = 1,
              const int    firstCycle = 0) const;
    int applyInverse(float     ** cycleData,
                     const int    numObjects,
                     const int    numChannels,
                     const int    numDataCycles = 1,
                     const int    firstCycle = 0) const;
};

#endif /*CYCLEMATRIX_H_*/
