/*
 * Copyright (c) Illumina 2007
 * Author:       Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */

#ifndef PHASINGESTIMATOR_H_
#define PHASINGESTIMATOR_H_

#include <vector>
#include <string>

class PhasingEstimator
{
private:
    std::vector<float *>   m_CycleIntensities;
    int                    m_NumClusters;
    int                    m_NumChannels;
    int                  * m_Sequences;
    int                    m_NumMinClusters;
    int                    m_NumMaxCycles;
    int                    m_NumMinCycles;
    std::string            m_ErrorMessage;
    void normalise();
public:
    PhasingEstimator(int numChannels = 4);
    virtual ~PhasingEstimator();
    // cycleIntensities is a vector of arrays of matrix-corrected
    // intensities
    void   setIntensities(const std::vector<float *> & cycleIntensities,
                          const int                    numClusters,
                          const int                    numCycles = -1);
    void setMinimumClusterNumber(const int num);
    void setMaximumFittedCycles(int num);
    int getClusterNumber() { return m_NumClusters; }
    // apply standard pipeline filtering to input data
    void filterChastity(const float CHASTITY   = 0.6,
                        const int   PURE_BASES = 12);
    float *estimateCycles(const int step = 1);
    int estimateCycles(float **phasingCycles,
                 float **prePhasingCycles);
    int estimate(float *phasing,
                 float *prePhasing,
                 const float *phasingCycles,
                 const float *prePhasingCycles);
    int estimate(float *phasing,
                 float *prePhasing);

    void printCycles(const float *phasingCycles,
                     const float *prePhasingCycles);

    enum ErrorCodes {
        errorNone,
        errorNotEnoughClusters,
        errorNotEnoughCycles
    };
    const char *getErrorMessage() { return m_ErrorMessage.c_str(); }
};

#endif /*PHASINGESTIMATOR_H_*/
