/*
 * Copyright (c) 2005-2007 Solexa
 * Author: Klaus Maisinger
 *
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */

#ifndef _PROCESSBASECALL_H_
#define _PROCESSBASECALL_H_

#include <vector>
#include "PhasingModel.h"
#include "CycleMatrix.h"

class ProcessBaseCall
{
    int             m_NumCycles;
    PhasingModel    m_PhasingModel;
    unsigned char * m_BaseOrder;
    int             m_NumChannels;
    int           * m_DeblockOrder;
public:
    ProcessBaseCall(const int numCycles = 1);
    virtual ~ProcessBaseCall();
    void setPhasingModel(const PhasingModel &phasingModel);
    void setCycleNumber(const int numCycles);
    void setBaseOrder(const unsigned char * bases,
                      const int             numChannels = 4);
    void setDeblockingOrder(const int * deblockOrder,
                            const int   numCycles = -1);
    void call(float                ** matrixSignal,
              float                ** noise,
              std::vector<float * > & inverseMatrices,
              const int               numObjects,
              const int               numChannels,
              const int               cycle,
              unsigned char         * sequence,
              float                 * qualityScores,
              float                 * signal,
              const int               numDataCycles,
              const int               firstDataCycle = 0);
    int getWindowSize(const int cycle = 0);
    void validateIntensities(float     ** intensities,
                             float     ** noise,
                             const int    numObjects,
                             const int    numChannels,
                             const int    numCycles = 1);
    void applyMatrixCorrection(float             ** intensities,
                               const CycleMatrix &  matrices,
                               float             ** matrix_signal,
                               const int            numClusters,
                               const int            numChannels,
                               const int            numCycles,
                               const int            firstCycle = 0);
private:
    void applyPhasingCorrection(float                ** matrixSignal,
                                float                ** noise,
                                std::vector<float * > & inverseMatrices,
                                const int               numObjects,
                                const int               numChannels,
                                const int               cycle,
                                float                 * dephasedSignal,
                                float                 * probabilities,
                                const int               numDataCycles,
                                const int               firstDataCycle);
};

#endif //_PROCESSBASECALL_H_
