#ifndef QHG_OPTIONS
#define QHG_OPTIONS

#include <string>

#include "CommandLine.hh"
#include "IntensityData.h"
#include "QhgFilter.hh"

class QhgOptions : public CommandLine::Options
{
public:
    typedef IntensityData::CompressionType Compression;
    /**
     ** Setting all the default command line options
     **/
    QhgOptions() 
        : CommandLine::Options()
        , filter           ("chastity")
        , relation         ("gt")
        , threshold        (0.6)
        , compressionName  ("none")
        , useBasesString   ("odd")
        , pureBases        (1)
        , chastityThreshold(0.6)
        , signalMeans      (false)
        , intensitySuffix  ("_sig2.txt")
        , sequenceSuffix   ("_seq.txt")
        , quahogSuffix     ("")
        , signalMeansSuffix("_all.txt")
        , intensityPrefix  ("")
        , sequencePrefix   ("")
        , quahogPrefix     ("")
        , signalMeansPrefix("")
    {
    }
private:
    virtual int getOptions(int argc, char * const argv[])
        throw (CommandLine::UnknownOptionException, 
               CommandLine::MissingOptionException, 
               CommandLine::InvalidArgumentException);
    virtual void processOptions() throw (CommandLine::InvalidArgumentException);
    virtual void getParameters(int argc, char * const argv[]) throw (CommandLine::InvalidArgumentException);
    virtual int usage(int argc, char * const argv[], const std::string &message = "");
    // parameters needing further processing
    std::string filter;
    std::string relation;
    float       threshold; 
    std::string compressionName;
    std::string useBasesString;
public:
    // parameters as they are used in the application
    int         pureBases;
    float       chastityThreshold; 
    int         signalMeans;
    std::string intensitySuffix;
    std::string sequenceSuffix;
    std::string quahogSuffix;
    std::string signalMeansSuffix;
    std::string intensityPrefix;
    std::string sequencePrefix;
    std::string quahogPrefix;
    std::string signalMeansPrefix;
    const QhgFilter  *qhgFilter;
    Compression compression;
    std::vector<bool> useBases;
    std::vector<std::string> inputList;
};


#endif // #ifndef QHG_OPTIONS

