#include <iostream>

using namespace std;

#include "testSequenceData.hh"

CPPUNIT_TEST_SUITE_REGISTRATION( TestSequenceData );

void TestSequenceData::setUp()
{
    const std::string intensityFile = "s_1_0007_sig2.txt";
    const std::string sequenceFile  = "s_1_0007_seq.txt";
    IntensityData  intensityData(intensityFile.c_str(), IntensityData::compressionNone);
    readInputErrorCode = intensityData.read(intensities, &numClusters, &numChannels, &clusterPos);
    if (readInputErrorCode == 0)
    {
        numCycles = intensities.size();
        sequence_array = new std::vector<unsigned char>(numClusters*numCycles);
        //unsigned char *tmp = new unsigned char[numClusters*numCycles];
        readSequenceErrorCode = read_sequence_file(sequenceFile.c_str(), (unsigned char *)sequence_array->data(), numClusters, numCycles);
        //readSequenceErrorCode = read_sequence_file(sequenceFile.c_str(), tmp, numClusters, numCycles);
        //sequence_array = new std::vector<unsigned char>(tmp, tmp+numClusters*numCycles);
    }
}

void TestSequenceData::tearDown()
{
    delete [] clusterPos;
    for (int i = 0; i < int(intensities.size()); i++)
        delete [] intensities[i];
}

void TestSequenceData::testConstructor()
{
    CPPUNIT_ASSERT_EQUAL(0, readInputErrorCode);
    CPPUNIT_ASSERT_EQUAL(0, readSequenceErrorCode);
    CPPUNIT_ASSERT_EQUAL(4, numChannels);
    CPPUNIT_ASSERT_EQUAL(36, numCycles);
    CPPUNIT_ASSERT_EQUAL(14232, numClusters);
    std::string s0 = "GGCAATAATGTTTATGTTGGTTTCATGGTTTGGTCT";
    std::string s1 = "GATGAGGGACATAAAAAGTAAAAATGTCTACAGTAG";
    std::string s2 = "GTCAACAAGAGAATCTCTACCATGAACAAAATGTGA";
    std::string c0(s0.length(), '.');
    for (unsigned int i = 0; i < c0.length(); ++i) c0[i] = (*sequence_array)[i*numClusters + 0];
    std::string c1(s1.length(), '.');
    for (unsigned int i = 0; i < c1.length(); ++i) c1[i] = (*sequence_array)[i*numClusters + 1];
    std::string c2(s2.length(), '.');
    for (unsigned int i = 0; i < c2.length(); ++i) c2[i] = (*sequence_array)[i*numClusters + 2];
    CPPUNIT_ASSERT_EQUAL((size_t)numCycles, s0.size());
    CPPUNIT_ASSERT_EQUAL(s0.compare(c0), 0);
    CPPUNIT_ASSERT_EQUAL(s1.compare(c1), 0);
    CPPUNIT_ASSERT_EQUAL(s2.compare(c2), 0);
}

