/*
 * Copyright (c) 2006 Solexa Ltd.
 * Author: Lorenzo Milazzo
 *
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */

//
// CVS - $Id: optimise_xtalk_mx.cpp,v 1.2 2008/07/03 11:36:01 craczy Exp $
//

#include "fitting.h"
#include "optimise_xtalk_mx.h"
#include "sampling.h"

#include "savinio_debugging.h"


void optimiseXTalkMatrix(const float * i_data,
                         const int     no_samples,
                         const int     no_bases,
                         Matrix      & mx,
                         const MatrixSamplingParameters params)
{
    int i, j;
    // selected data (after sampling) for the X and Y axes
    float *xaxis_data = new float [ params.numBins ];
    float **yaxis_data = new float * [ no_bases - 1 ];
    for (i = 0; i < no_bases - 1; i++)
        yaxis_data[i] =  new float [ params.numBins ];

    // off-diagonal elements of the cross-talk matrix
    float *mx_off_diagonals = new float[ no_bases * (no_bases-1) ];
    for (j = 0; j < no_bases * (no_bases-1); ++j)
        mx_off_diagonals[j] = 0.0;

    // first estimate of the cross-talk matrix:
    // . sampling
    // . fitting of the selected data and estimate of the slopes
    //   of the ndim * (ndim-1) archetypal directions
    for (i = 0; i < no_bases; ++i) {
        binAxes(i, no_samples, no_bases, i_data,
                xaxis_data, yaxis_data, params);
        for (j = 0; j < no_bases-1; j++)
            L1fit(xaxis_data, yaxis_data[j], params.numBins,
                  &mx_off_diagonals[j + i * (no_bases - 1)]);
    }

    j = 0;
    for (int c = 0; c < mx.cols; ++c) {
        for (int r = 0; r < mx.rows; ++r) {
            if (r == c) {
                mx.data[c + r * mx.cols] = 1.;
            }
            else {
                mx.data[c + r * mx.cols] = mx_off_diagonals[j];
                j++;
            }
        }
    }


    delete [] xaxis_data;
    for (i = 0; i < no_bases - 1; i++)
        delete [] yaxis_data[i];
    delete [] yaxis_data;
    delete [] mx_off_diagonals;


    //--
#ifdef DEBUG_XTALK_MATRIX_1
        std::cout << "\n>> debug - unscaled cross-talk matrix, first estimate" << std::endl;

        mx.display();

#endif // DEBUG_XTALK_MATRIX_1
    //--
}
