/*
 * $RCSfile: sampling.h,v $
 * Author: Lorenzo Milazzo
 * $Id: sampling.h,v 1.1 2008/07/02 08:38:24 craczy Exp $
 *
 * Copyright (c) 2006, Solexa Ltd.
 *               2008, Illumina
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */

#ifndef __SAMPLING_H__
#define __SAMPLING_H__

#include <vector>
#include "savinio_common.h"

struct MatrixSamplingParameters {
    int   numBins;
    float percentileXUpper;
    float percentileXLower;
    float percentageXUpper;
    float percentageXLower;
    float percentileY;
    MatrixSamplingParameters() : numBins(NO_BINS),
                                 percentileXUpper(PERCENTILE_UPPER_X_S1),
                                 percentileXLower(PERCENTILE_LOWER_X_S1),
                                 percentageXUpper(PERCENTAGE_UPPER_X_S1_ITER2),
                                 percentageXLower(PERCENTAGE_LOWER_X_S1_ITER2),
                                 percentileY(PERCENTILE_LOWER_Y_S1_ITER2)
                                 {};
};

void binAxes(const int      base_index,
             const int      no_points,
             const int      no_cols,
             const float *  i_data,
             float       *  xaxis_data,
             float       ** yaxis_data,
             const MatrixSamplingParameters params);
void baseCall(const float          * inputData,
              const int              numInputData,
              const int              numChannels,
              std::vector<float *> & data,
              std::vector<int>     & numData);
void filterLowIntensities(const float *data,
                          const int    numData,
                          float      * dataFiltered,
                          int        * numDataFiltered);


#endif /* __SAMPLING_H__ */
