#include <sstream>
#include <cassert>

#include "GzPack.hh"

namespace GzPack {

using namespace std;

DataLayout::DataLayout(const unsigned int numberOfChannels, const unsigned int numberOfClusters)
    : channels(numberOfChannels)
    , clusters(numberOfClusters)
{
}

DataLayout::DataLayout(const DataLayout &dataLayout)
    : channels(dataLayout.channels)
    , clusters(dataLayout.clusters)
{
}

DataLayout GzPackReader::initDataLayout() throw (std::logic_error)
{
    const char * layoutLine = static_cast<AutoGzFile *>(this)->gets();
    int channels = 0;
    int clusters = 0;
    const int success = sscanf(layoutLine, "#CH%d:OBJ%d", &channels, &clusters);
    if (2 != success) 
    {
        throw logic_error(string("GzPackReader: failed to read the data layout from: ") + layoutLine);
    }
    if (0 >= channels) 
    {
        throw logic_error(string("GzPackReader: the number of channels must be strictly positive: ") + layoutLine);
    }
    if (0 > clusters) 
    {
        throw logic_error(string("GzPackReader: the number of clusters must be positive: ") + layoutLine);
    }
    return DataLayout(channels, clusters);
}

GzPackReader::GzPackReader(const std::string fileName)  throw (std::logic_error)
    : AutoGzFile(fileName, string("r"))
    , dataLayout_(initDataLayout())
    , currentLine_(gets())
    , localStorage_(0)
{
    skipCommentLines();
}

const float *GzPackReader::getIntensityData() throw (std::logic_error)
{
    if (0 == localStorage_)
    {
        localStorage_ = new float[dataLayout_.clusters * dataLayout_.channels];
    }
    getIntensityData(localStorage_);
    return localStorage_;
}

void GzPackReader::skipCycle() throw (std::logic_error)
{
    skipCommentLines();
    for (unsigned int cluster = 0; dataLayout_.clusters > cluster; ++cluster)
    {
        if (Z_NULL == currentLine_)
        {
            throw logic_error(string("GzPackReader: Failed to read the intensity data"));
        }
        currentLine_ = gets();
    }        
    skipCommentLines();
}

void GzPackReader::skipCommentLines()
{
    while (Z_NULL != currentLine_ && '#' == *currentLine_)
    {
        currentLine_ = gets();
    }
}

} // namespace GzPack

