/*
 * Copyright (c) Illumina 2007-2008
 * Author:       Klaus Maisinger
 *
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
#include <stdarg.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <time.h>

#include "Message.h"

void info(const char * format, ...)
{
    char message[4024];
    va_list ap;
    va_start(ap, format);
    vsnprintf(message, sizeof(message), format, ap);
    va_end(ap);
    fprintf(stderr, "Info: %s\n", message);
}

static void logfile(const char *filename,
                    const char *message,
                    const int   level)
{
    FILE *fh = fopen(filename, "w");
    if (!fh)
        return;
    {
        time_t tp;
        struct tm *tps;
        char timebuffer[10];
        char datebuffer[12];
        char user      [20];
        char host      [25];
        char buffer   [256];

        time(&tp);
        tps = gmtime(&tp);
        strftime(timebuffer, 10, "%H:%M:%S", tps);
        strftime(datebuffer, 12, "%d/%m/%Y", tps);
        if (getenv("LOGNAME"))
            snprintf(user, 20, getenv("LOGNAME"));
        else if (getenv("USERNAME"))
            snprintf(user, 20, getenv("USERNAME"));
        else
            snprintf(user, 20, "unknown");

        if (getenv("HOSTNAME"))
            snprintf(host, 25, getenv("HOSTNAME"));
        else
            snprintf(host, 25, "unkown");
        snprintf(buffer, 256, "%i\t%s\t%s\t%s\t%s\t%s\t", 
                 level, datebuffer, timebuffer, "unknown", user, host);
        fprintf(fh, "%s%s\n", buffer, message);
    }
    fclose(fh);
}

void warning(const char * format, ...)
{
    char message[4024];
    va_list ap;
    va_start(ap, format);
    vsnprintf(message, sizeof(message), format, ap);
    va_end(ap);
    fprintf(stderr, "Warning: %s\n", message);
    logfile("warning.txt", message, 6);
}

void error(const char * format, ...)
{
    char message[4024];
    va_list ap;
    va_start(ap, format);
    vsnprintf(message, sizeof(message), format, ap);
    va_end(ap);
    fprintf(stderr, "Error: %s\n", message);
    logfile("error.txt", message, 7);
}
