// [in], [out] data types for analysis plugins
#ifndef _PARAMETER_TYPES_H_
#define _PARAMETER_TYPES_H_

extern "C"
{
	// Marshalled from AnalysisInterface\Direct\TileInfo.cs
	struct TileInfo
	{
		int Lane;
		int Column;
		int Row;
		int Location;// Linear, unique, 0-based identifier of the tile. Increments in the direction of the data acquisition.
		int TileIndex;
		int Cycle;
		int RunId;// Token allowing distinguishing between overlapping callbacks from adjacent runs.
		int IsDeblocked; // 1 for deblock images, 0 otherwise
	};

	// Marshalled from AnalysisInterface\Direct\ImageInfo.cs
	struct ImageInfo
	{
		int	 Channel;
		char Base;
		int	 Width;
		int	 Height;

		int NumBytes() const
		{
			return Width * Height * sizeof( unsigned short );
		}
	};

	struct TileImageInfo
	{
		struct TileInfo	 TileInfo;
		struct ImageInfo ImageInfo;
	};
    
	struct AnalysisParameters
	{
	  int   DetectionType;
	  int   FilterType;
	  float ClusterFwhm;
	  float ClusterC;     
	  float ClusterGamma;
	  float ClusterDistance;
 	  float GaussFactor;
	  float PearsonThreshold;
	  float MaxStageOffset; // default = 500
	  float MaxColourOffset;// default = 5
	  int   minObjectPixels; 
	    
          void SetImageDefaults()
          {
		DetectionType     = 0;
		FilterType        = 0;  // mexican hat
		ClusterFwhm       = 2.7f;
		ClusterC          = 1.0; // nearly a mexican hat
		ClusterGamma      = 2.0; // Gaussian noise
		ClusterDistance   = 1.5f;
		GaussFactor       = 4.f;
		PearsonThreshold  = 0.3f;
		MaxStageOffset    = 500.f;
		MaxColourOffset   = 5.f;
		minObjectPixels   = 3;
          } 

          AnalysisParameters() { SetImageDefaults(); }

          // Copy Constructor
          AnalysisParameters(const AnalysisParameters & source) { *this = source; }

	};

	struct RunInfo
	{
		int			 ImageWidth; // Pixels
		int			 ImageHeight;// Pixels
		int			 NumChannels;
		int			 NumTiles;// number of tile positions in a cycle.
		int			 NumCycles;
		const char * RunFolder;
		const char * TiffPathTemplate;
	};

	// Marshalled from Stargazer\Config\Alignment.cs
	struct Alignment
	{
		float XOffset;
		float YOffset;
		float XScale;
		float YScale;
	};

	struct Position
	{
		float X;
		float Y;
	};

    struct PixelValue : public Position {
        float Intensity;
        float Noise;
    };

    struct ImageRegion {
        int XMin;
        int YMin;
        int XSize;
        int YSize;
    };
    
    struct ImageStats {
        float Minimum;
        float Maximum;
        float Average;
        float StandardDeviation;
        float AverageBackground;
        float FractionSaturation; // >= 0., << 0.0001
        float DispersionBackground;
    };

    struct ClusterStats {
        int Num;
        float FwhmMean;
        float FwhmMedian;
        float FwhmStandardDeviation;

	    void operator=(const ClusterStats& cluster_stats) {
		    Num = cluster_stats.Num;
		    FwhmMean = cluster_stats.FwhmMean;
		    FwhmMedian = cluster_stats.FwhmMedian;
			FwhmStandardDeviation = cluster_stats.FwhmStandardDeviation;
		}
    };

    struct AlignmentStats : public Alignment {
        struct ImageRegion Region;
        float CorrelationMinimum;
        float CorrelationMaximum;
        float CorrelationWidth;   // > 0., < 2.
    };

    const int NumAlignmentRegions = 4;

    struct TileQCValues
    {
        float  EstimatedFWHM; // should be between 2. and 8. pixels
        int    OffsetsValidated; // should be 1
        struct ImageStats     IntensityStats;
        struct Alignment      UsedAlignment;
        struct AlignmentStats RawAlignment;
        struct AlignmentStats RegionAlignments[NumAlignmentRegions];
        struct ClusterStats   Fwhm;
	  int	   NumClusters;
    };
}

#endif//_PARAMETER_TYPES_H_
