/*
 * Copyright (c) Illumina 2007
 * Author:
 *               Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <string>
#include "PositionIndex.h"
#include "Stats.h"
#include "Message.h"

PositionIndex::PositionIndex() : lane(0), tile(0), X(0), Y(0)
{
}

PositionIndex::PositionIndex(const PositionIndex & rhs) 
                                      : lane(rhs.lane), tile(rhs.tile),
                                        X(rhs.X), Y(rhs.Y)
{
}

PositionIndex::PositionIndex(const int               iLane,
                             const int               iTile,
                             const struct Position & position)
{
    X = 0;
    Y = 0;
    lane = iLane;
    tile = iTile;
    X = Stats::round(int(Stats::round(position.X * 100.)) / 100.);
    Y = Stats::round(int(Stats::round(position.Y * 100.)) / 100.);
 }

PositionIndex::PositionIndex(const int iLane,
                             const int iTile,
                             const int iX,
                             const int iY) : lane(iLane), tile(iTile),
                                             X(iX), Y(iY)
{
}

PositionIndex::~PositionIndex()
{
}

int PositionIndex::snprint(char      buffer[],
                           const int buffersize)
{
    return snprintf(buffer, buffersize, "%i\t%i\t%i\t%i",
                    lane + 1, tile + 1, X, Y);
}

int PositionIndex::write(const char       filename[],
                         const int        lane,
                         const int        tile,
                         const Position * positions,
                         const int        numClusters)
{
    int          errorno  = 0;
    std::string  errormsg = "";
    FILE        *fp       = fopen(filename, "w");
    if (!fp) {
        errormsg = "Failed to open file " + std::string(filename) + ".";
        errorno  = 1; goto terminate;
    }
    
    int  i, idx;
    char buffer[256];
    for (i = 0; i < numClusters; i++) {
        PositionIndex position(lane, tile, positions[i]);
        idx = position.snprint(buffer, 20);
        sprintf(buffer+idx, "\n");
        if (fprintf(fp, buffer) < (int)strlen(buffer)) {
            errormsg = "Failed to write to " + std::string(filename) + ".";
            errorno  = 2; goto terminate;
        }
    }
    if (fclose(fp)) {
        errormsg = "Failed to close file " + std::string(filename) + ".";
        errorno  = 3; goto terminate;
    }
    terminate:
    if (errorno != 0)
        error(errormsg.c_str());
    return errorno;
}
