//  Time-stamp: <$Id: StringUtil.cpp,v 1.3 2008/07/17 15:44:49 rshaw Exp $>
/*
 * Functions useful in string manipulation. 
 *
 * Copyright (c) Illumina 2007
 * Author: Lukasz Szajkowski
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
#include <stdlib.h> 
#include <stdio.h>
#include <cstring>
#include <vector>
#include <string>

using namespace std;

/* 
 * Convert input string into vector of string tokens
 * strin string to be parsed
 * delims list of delimiters.
 */
std::vector<std::string> tokenize_str(const std::string & strin,
		const std::string & delims=", \t") {
	if (strin.length() == 0) {
		fprintf(stderr, "tokenize_str: Error - empty string\n");
		exit(-1);
	}
	if (delims.length() == 0) {
		fprintf(stderr, "tokenize_str: Error - empty delims\n");
		exit(-1);
	}
	string::size_type lastPos = strin.find_first_not_of(delims, 0);
	string::size_type pos = strin.find_first_of(delims, lastPos);

	// output vector
	vector<string> tokens;

	while (string::npos != pos || string::npos != lastPos) {
		std::string token = strin.substr(lastPos, pos - lastPos);
		tokens.push_back(token);
		lastPos = strin.find_first_not_of(delims, pos);
		pos = strin.find_first_of(delims, lastPos);
	}
	return tokens;
}

void fastTokenizer(const char* str, char delim, vector<char*>& tokenList) {
	tokenList.clear();
	int start = 0;
	int i;
	for (i = 0; str[i]; i++) {
		if (str[i] == delim) {
			if (i > 0 && str[i - 1] != delim) {
				char* token = new char[i-start+1];
				memcpy(token, &str[start], i-start);
				token[i-start] = '\0';
				tokenList.push_back(token);
			}
			start = i + 1;
		}
	}
	if (start < i) {
		char* token = new char[i-start+1];
		memcpy(token, &str[start], i-start);
		token[i-start] = '\0';
		tokenList.push_back(token);
	}
}

void fastTokenizerInPlace(char* str, char delim, vector<char*>& tokenList) {
	tokenList.clear();
	int start = -1;
	int i;
	int lastEnd = -1;
	for (i = 0; str[i]; i++) {
		if (str[i] == delim) {
			if (i > 0 && str[i - 1] != delim && i - 1 != lastEnd) {
				str[i] = '\0';
				lastEnd = i;
				tokenList.push_back(&str[start+1]);
			}
			start = i;
		}
	}
	if (start + 1 < i) {
		str[i] = '\0';
		tokenList.push_back(&str[start + 1]);
	}	
}

/*
 * Replaces maching characters in the string with given new characters.
 * return number of characters replaced.
 */
int replaceTR(std::string & strin, const std::string & findChars,
		const std::string & replChars) {
	int length = strin.length();
	int i, j;
	if (findChars.length() != replChars.length()) {
		return -1;
	}
	int lengthRepl = replChars.length();
	int replCount = 0;
	for (i = 0; i < length; i++) {
		for (j = 0; j < lengthRepl; j++) {
			if (strin[i] == findChars[j]) {
				strin[i] = replChars[j];
				replCount++;
			}
		}
	}
	return replCount;
}

/*****************************************************************************/
// Returns a ptr to a newly allocated C-string that is a copy of the supplied
// string.

char* copy_to_new_cstr(const std::string& str)
{
    char* buf_ptr = new char[str.size() + 1];

    if (!buf_ptr) {
        return buf_ptr;
    }

    strncpy(buf_ptr, str.c_str(), str.size());
    *(buf_ptr + str.size()) = '\0';

    return buf_ptr;
}

/*****************************************************************************/
