/*****************************************************************************/
// Copyright (c) Illumina 2008
// Author: Richard Shaw
// This source file is covered by the Illumina Genome Analyzer Software
// License Agreement and bound by the terms therein.
/*****************************************************************************/

#include <sstream>

#if defined(__BORLANDC__)
#include <stdio.h>
#else
#include <cstdio>
#endif

#include "Message.h"

#include "Tile_List.h"

/*****************************************************************************/

Tile_List::Tile_List(const std::string& tile_list_file_path_str)
    : my_tile_list_file_path_str(tile_list_file_path_str),
      my_list_is_loaded(false)
{
    ;
}

/*****************************************************************************/

Tile_List::~Tile_List()
{
    ;
}

/*****************************************************************************/

bool Tile_List::find_tiles(const std::string& search_prefix_str,
                           Tile_Name_List& tile_name_list,
                           const std::string& suffix_to_append_str)
{
    if (!my_list_is_loaded) {
        if (!load()) {
            error("Failed to read tile list from `%s'.",
                  my_tile_list_file_path_str.c_str());
            return false;
        }
    }

    const unsigned int search_prefix_str_len(search_prefix_str.length());

    tile_name_list.clear();

    for (Tile_Name_List_CIter tile_name_citer(my_full_tile_name_list.begin());
         tile_name_citer != my_full_tile_name_list.end(); ++tile_name_citer) {
        const std::string tile_name(*tile_name_citer);

        if (tile_name.compare(0, search_prefix_str_len, search_prefix_str)
            == 0) {
            tile_name_list.push_back(tile_name + suffix_to_append_str);
        }
    }

    return true;
}

/*****************************************************************************/

bool Tile_List::load()
{
    FILE* file_hndl = fopen(my_tile_list_file_path_str.c_str(), "r");

    if (!file_hndl) {
        error("Failed to open `%s'.",
              my_tile_list_file_path_str.c_str());
        return false;
    }

    const unsigned int max_tile_name_len(16); // 8 chars + null + wiggle room
    char cstr[max_tile_name_len];

    std::ostringstream format_ostrm;
    format_ostrm << "%" << max_tile_name_len << "s";

    int scan_ret(0);

    while ((scan_ret = fscanf(file_hndl, format_ostrm.str().c_str(), cstr))
           == 1) {
        my_full_tile_name_list.push_back(cstr);
    }

    if (scan_ret == EOF) {
        my_list_is_loaded = true;
        info("Read tile list from `%s'.",
             my_tile_list_file_path_str.c_str());
    } else {
        error("Failure while reading tile list from `%s'.",
              my_tile_list_file_path_str.c_str());
    }

    fclose(file_hndl);

    return (scan_ret == EOF);
}

/*****************************************************************************/
