/*  Time-stamp: <$Id: Use_Bases.h,v 1.1 2008/07/02 10:25:19 craczy Exp $> */
/*
 * Copyright (c) Illumina 2007  
 * Author: Lukasz Szajkowski
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 * 
 *  @version $Id: Use_Bases.h,v 1.1 2008/07/02 10:25:19 craczy Exp $
 */

#ifndef USE_BASES_H
#define USE_BASES_H

#include <vector>
#include <string>

using namespace std;
const std::string CONST_ALL = "all";
const std::string CONST_ODD = "odd";
const std::string CONST_LEGAL_MAST = "YNyn";
/*
 * The function expands bases the the useBases string.
 */
void expandLegacyUseBasesString(const std::string & useBases,
        const int unsigned readLength, std::string & useBasesOut);
/*
 * The function parseUseBases parses a long form USE_BASES parameter 
 * into an array of indices
 * Inputs:
 * 1. useBases string - either:
 * 'all' - use all bases (probably an anachronism)
 * 'odd' - use odd numbered bases (definitely an anachronism)
 * Some combination of 'Y', 'y', 'N' and 'n'
 * 2. readLength - used in 'all' or 'odd' modes to generate a string
 * of the right length, also used to check there are enough 'Y/y's in
 * useBases - again probably an anachronism, set to 0 to ignore
 * 3. readPairMode (optional) Which read of a paired read - either
 * '0' - default 
 * '1' - first read: treat all 'y's as 'n's
 * '2' - second read: treat all 'Y's as 'N's
 */
void parseUseBases(const std::string & useBases, const int unsigned readLength,
        vector<unsigned int> & baseIndex, const int unsigned readPairMode = 0);

#endif /* USE_BASES_H */
