/*
 * Copyright (c) Solexa 2004, 2005, 2006
 * Author: 
 * Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
//---------------------------------------------------------------------------

#ifndef ImageProcessH
#define ImageProcessH
//---------------------------------------------------------------------------

#include <bitset>
#include "ImageUtil.h"
#include "ClusterObject.h"
#include "ProcessDetectedObjectStats.h"
#include "ProcessAlign.h"
#include "ParameterTypes.h"

struct ImageStatsRecord : public ImageStats {
    ImageRegion Region;
};

struct ImageAnalysisRecord {
    float average_noise;
    float average_threshold;
    float average_threshold_signal;
    float average_threshold_background;
};

struct SequencingRecord {
    unsigned char base;
    int is_deblocked;
};


class ImageData {
  public:
    float               * image;
    int                   nx, ny;
    AnalysisParameters    params;
    unsigned short        bitsperpixel;
    float               * fourier_image;
    int                   fnx, fny;
    float               * filter_image;
    float               * noise_image;
    float               * background_image;
    ClusterObject       * objects;
    ObjectStatsRecord   * object_stats;
    ObjectStatsRecord   * filtered_object_stats;
    int                   nobj;
    ClusterStats          cluster_stats;
    st_pixel            * positions;
    PixelValue          * positiondata;
    ImageStatsRecord    * image_stats;
    ImageStatsRecord    * local_image_stats;
    int                   nlocal_stats;
    ImageAnalysisRecord * analysis_stats;
    SequencingRecord    * sequencing;
    AlignmentTransform  * transform;
    AlignmentTransform  * transform2;
    AlignmentTransform  * usedTransform;
    float                 estimated_fwhm;
    bool                  useInterpolatedPositionFlag;
  private:
    static const int      maxImageBuffers = 10;
    enum States { haveImageStats, haveFilter, haveBackgroundNoise,
                  haveAnalysis, haveObjectStats, haveFilteredObjectStats,
                  numStates };
    std::bitset<numStates> state;
    int                    nmax;
  public:
    ImageData(unsigned short * data,
              const int              x,
              const int              y,
              const unsigned short   bpp);
    ImageData(const float          * data,
              const int              x,
              const int              y,
              const unsigned short   bpp);
    virtual ~ImageData();
    void Init();
    void setSequencingInfo(const unsigned char base,
                           const int is_deblocked = 0);
    void setAlignmentTransform(AlignmentTransform *t) { transform = t; }
    void analyseImageStats();
    void analyseLocalImageStats();
    void analyseFilteredObjectStats();
    void analyseObjectStats();
    void analyseFilter();
    void analyseBackgroundNoise();
    void analyseEstimateFWHM();
    void analyse();
    void clear();
    void clearImages();
    int writeObjectStats(const char         filename[],
                         const PixelValue * lists = 0,
                         const int        * indices = 0,
                         const int          count = 0);
    int writePositions(FILE      * filehandle,
                       const int   indexflag = 0);
};
    
#endif /* ImageProcessH */
