/*
 * Copyright (c) Illumina 2007
 * Author:       Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
#include "ImageRegionList.h"

ImageRegionList::ImageRegionList(const int nx,
                                 const int ny)
{
    m_XDim = nx;
    m_YDim = ny;
}


void ImageRegionList::setRegionSizes(const int sizeX,
                                     const int sizeY)
{
    int numXRegions = m_XDim / sizeX;
    int numYRegions = m_YDim / sizeY;

    m_Regions.clear();

    /* split the image into a set of tiles */
    int i, j;
    for (i = 0; i < numXRegions; i++)
        for (j = 0; j < numYRegions; j++) {
            ImageRegion region;
            region.XMin = i * sizeX;
            region.YMin = j * sizeY;
            region.XSize = sizeX;
            region.YSize = sizeY;
            /* add extra pixels if not all pixels have been covered
             * by the last tile */
            if ((i == numXRegions - 1) && (m_XDim - region.XMin > region.XSize))
                region.XSize = m_XDim - region.XMin;
            if ((j == numYRegions - 1) && (m_YDim - region.YMin > region.YSize))
                region.YSize = m_YDim - region.YMin;
            m_Regions.push_back(region);
        }
}

void ImageRegionList::setNumRegions(const int numXRegions,
                                    const int numYRegions)
{
    setRegionSizes(m_XDim / numXRegions, m_YDim / numYRegions);
}

void ImageRegionList::getRegions(std::vector<ImageRegion> & regions)
{
    regions = m_Regions;
}

int  ImageRegionList::getNumRegions()
{
    return int(m_Regions.size());
}

int *ImageRegionList::getRegionIndex(Position * position, int numObjects)
{
    int *result = new int [ numObjects ];
    // TODO implement
    return result;
}

ImageRegionList::~ImageRegionList()
{
//    m_Regions.clear();
}
