/*
 * Copyright (c) Solexa 2004, 2005, 2006
 * Author:
 * Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
//---------------------------------------------------------------------------

#ifndef ImagesH
#define ImagesH

#include <vector>
#include <string>
#include <bitset>
#include "Positions.h"
#include "ParameterTypes.h"
#include "tifflib.h"

//---------------------------------------------------------------------------

class ImageProperties {
public:
    unsigned int   MinScale, MaxScale;
    float          StageXPosition, StageYPosition;
    char           StageLabel [80];
    char           ImageDescription[80];
    unsigned short BitsPerPixel;
    float          ExposureTime;
public:
    ImageProperties();
};

class Image {
private:
    int         file_index;
    int         nframes;
    char        id[300];
    std::string filename;
    enum States { haveFilename, haveSize, haveProperties, haveImage,
                  haveData, haveObjects, numStates };
    std::bitset<numStates> state;
    int read_image_properties(TIFF_FILE *fp);
protected:
    unsigned short *image;
public:
    AnalysisParameters params;
    bool               useInterpolatedPositionFlag;
    int nx, ny;
    int active;
    ImageProperties *properties;
    ImageData *data;

    Image();
    ~Image();
    void setFilename(const char fn[], int pos = 0, int npos = 0);
    void setSize(int x, int y, int totalpos = 0);
    int loadSize(TIFF_FILE *hd = 0);
    int loadProperties(TIFF_FILE *hd = 0);
    int load(TIFF_FILE *hd = 0);
    void setImage(unsigned short *pixels, int pnx, int pny);
    void clearImages();
    void clearData();
    void clear();
    int analyse();
    std::string get_id() const;
    std::string getFilename() const;
    int get_frame_index();
    int get_frame_total();
    PixelValue * getObjects() const;
    int getNumObjects() const;
    unsigned short *get_image();
    ImageData *getData();
};

class ImageList : public std::vector<Image *>
{
    std::vector <char *> filename;
    void clearImages();
public:
    int loadDimensions(const char filename[]);
    void addFile(const char *name) 
        {   char *fn = new char [FILENAME_MAX];
            snprintf(fn, FILENAME_MAX, name);
            filename.push_back(fn);
        }
    int      load();
    void     clearList();
    virtual ~ImageList();
    int      createEmptyImages(int nx, int ny, int ni = 1);
    void     write_info();
    void     read_info();
};

#endif /* ImagesH */
