/*
 * Copyright (c) Illumina 2008
 * Author:
 *               Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */

#include <vector>
#include "Message.h"
#include "PositionData.h"
#include "ImageAnalysisConfig.h"


/**
 * Reads positions from a text file.
 */
int read_positions(const char    filename[],
                   Position   ** positions,
                   int         * npos)
{
    *npos      = 0;
    *positions = 0;

    int nread  = 0;
    std::vector<Position> values;
    values.reserve(ImageAnalysisConfig::maxObjects);
    FILE *posfile = fopen(filename, "r");
    if (posfile) {
        char buffer[200+1];
        while (fgets(buffer, 200, posfile)) {
            Position position;
            if (buffer[0] == '#') // discard lines marked as comments
                continue;
            if (sscanf(buffer, "%f %f", &(position.X), &(position.Y)) == 2) {
                nread++;
                values.push_back(position);
            }
            else {
                error("Bad line in %s.", filename);
                return 2;
            }
        }
        fclose(posfile);
        *positions = new Position [ nread ];
        *npos = nread;
        memcpy(*positions, &(values[0]), nread * sizeof(Position));
        return 0;
    }
    else {
        error("Failed to open %s.", filename);
        return 1;
    }
    return 0;
}

/**
 * Writes positions to a text file.
 */
int write_positions(const char       filename[],
                    const Position * positions,
                    const int        n)
{
    FILE *posfile = fopen(filename, "wt");
    if (!posfile) {
        error("Failed to open position file %s.", filename);
        return 1;
    }
    int code = 0;
    int i;
    for (i = 0; i < n; i++)
        if (fprintf(posfile, "%7.2f %7.2f\n", positions[i].X, positions[i].Y)
            < 15) {
            error("Failed to write to %s.", filename);
            code = 2;
        }
            
    if (fclose(posfile)) {
        error("Failed to close position file %s.", filename);
        return 3;
    }
    return code;
}

PositionData::PositionData(const std::string &filename) : m_Filename(filename)
{
}

PositionData::~PositionData()
{
}

int PositionData::read(Position ** positions,
                       int       * numPositions)
{
    return read_positions(m_Filename.c_str(), positions, numPositions);
}

int PositionData::write(const Position * positions,
                        const int        numPositions)
{
    return write_positions(m_Filename.c_str(), positions, numPositions);
}
