/*
 * Copyright (c) Solexa 2004, 2006
 * Author:
 * Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */

#ifndef _POSITIONS_H_
#define _POSITIONS_H_
//---------------------------------------------------------------------------
#include <vector>
#include "ParameterTypes.h"
#include "ImageProcess.h"



class Positions : public std::vector<Position>
{
public:
    float shiftX, shiftY;
    int   nx, ny;
private:
    void initialise();
public:
    Positions();
    Positions(const Positions& temp);
    Positions(const int n,
              const int x,
              const int y);
    ~Positions();
    int write(const char filename[]) const;
    int read(const char filename[]);
    void shift(const float x,
               const float y);
    Position *getPositions() { return &at(0); }
    void align(Positions & reference,
               const int         maxlag,
               const float       fwhm);
    Positions *bin_or      (const Positions &s2, const float dist);
    Positions *bin_subtract(const Positions &s2, const float dist);
    Positions *bin_and     (const Positions &s2, const float dist);
    // binary operations
    static void bin_or(Position       * positions,  int       * count,
                       const Position * positions1, const int   count1,
                       const Position * positions2, const int   count2,
                       const int        nx,
                       const int        ny,
                       const float      min_dist);
    static void bin_subtract(Position       * positions,  int       * count,
                             const Position * positions1, const int   count1,
                             const Position * positions2, const int   count2,
                             const int        nx,
                             const int        ny,
                             const float      min_dist);
    static void bin_and(Position       * positions,  int       * count,
                        const Position * positions1, const int   count1,
                        const Position * positions2, const int   count2,
                        const int        nx,
                        const int        ny,
                        const float      min_dist);
    static int calculate_object_overlap(const Position * positions1, const int n1,
                                        const Position * positions2, const int n2,
                                        const int nx,   const int ny,
                                        const int xoff, const int yoff,
                                        const float min_dist);
    static int calculate_object_overlap(const PixelValue * positions1, const int n1,
                                        const PixelValue * positions2, const int n2,
                                        const int nx, const int ny,
                                        const int xoff, const int yoff,
                                        const float min_dist);
};

#endif /* _POSITIONS_H_ */
