/*
 * Copyright (c) Solexa 2006
 * Author:
 * Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
#ifndef PROCESSDETECTEDOBJECTSTATS_H_
#define PROCESSDETECTEDOBJECTSTATS_H_

const int OBJECT_PROPERTY_IS_SPLIT = 1;

struct ObjectStatsRecord {
    int   area;
    int   boundary;
    int   xdim, ydim;
    int   properties;
    float maximum;
    float integral;
    float noise;
    float background;
    float xcentre, ycentre;
    float xinter, yinter;
    float width_sigma;
    float gaussian_xcentre;
    float gaussian_ycentre;
    float gaussian_xsigma;
    float gaussian_ysigma;
    float gaussian_maximum;
    float gaussian_volume;
    float gaussian_background;
    float gaussian_chi2;
};

void create_lookup_array(int *array, int nx, int ny,
                         ClusterObject *objects, int nobj);
void expand_lookup_array(int *array, int nx, int ny,
                         ClusterObject *objects, int nobj);
void print_lookup_array(int *array, int nx, int ny,
                        int firstx = 0, int firsty = 0,
                        int lastx = -1, int lasty = -1);
int fit_gaussian_object(ClusterObject *object, float *image, int nx, int ny,
                        float *x, float *y, float *max,
                        float *xfwhm, float *yfwhm, 
                        float *background);
int fit_gaussian_area(int *lookup, float *array, int nx, int ny,
                      ClusterObject *object, int idx, ObjectStatsRecord *stats,
                      float *array2);
void get_background_dispersion(float *array, int *lookup, int nx, int ny,
                               ClusterObject *objects, int nobj,
                               ObjectStatsRecord *stats, float *array2);
void integrate_objects(int *lookup, float *array, int nx, int ny,
                       ClusterObject *objects, int nobj, ObjectStatsRecord *stats);
void printObjectStats(FILE                    * fp,
                      const PixelValue        * positions,
                      const ObjectStatsRecord * stats,
                      const int                 nobj);
void printObjectStats(FILE                    * fp,
                      const float             * image,
                      const float             * noise,
                      const int                 nx,
                      const int                 ny,
                      st_pixel                * positions,
                      const ObjectStatsRecord * stats,
                      const int                 nobj);

#endif /*PROCESSDETECTEDOBJECTSTATS_H_*/
