/*
 * Copyright (c) Solexa 2006
 * Author:
 * Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */
#ifndef PROCESSREMAPPING_H_
#define PROCESSREMAPPING_H_

#include "ProcessAlign.h"
#include "ParameterTypes.h"

class RemappingLookup {
    int        * lookup;
    int          nx, ny;
    Position   * reference;
    Position   * newpositions;
    PixelValue * candidate_newpositions;
    int        * refindex;
    int          nref;
    int          nnew, nmax, ncandnew;
    void remap(const PixelValue * positions,
               int              * new_index,
               const int          npos,
               int              * ref_index,
               const float        min_dist);
    void remapPosition(const PixelValue & position,
                       int              * new_index,
                       float            * new_nearest_dist, 
                       const float        min_dist,
                       const Position   * reference,
                       const int          nref,
                       const int          pos,
                       int              * ref_index,
                       float            * ref_nearest_distance);
public:
    RemappingLookup(int x, int y);
    virtual ~RemappingLookup();
    void setReference(const Position * positions,
                      const int        numPositions);
    void clearReference();
    void clearNewPositions();
    void setMaxNewPositions(int n);
    void getNewPositions(Position  * positions,
                         const int   numMax);
    void addNewPositions(const PixelValue * positions,
                         const int          numPositions,
                         const float        minDistance);
    void findNewPositions(float min_dist);
    void addNewPositionsToReference();
    void remap(const PixelValue * positions,
               const int          npos,
               const float        min_dist);
    int getNumberNewPositions() { return nnew; }
    Position * getNewPositions() { return newpositions; }
    int *getIndex() { return refindex;}
    
};


void remap_colour(const Position   * refpositions,
                  const int        * refindices,
                  const int          nref,
                  const PixelValue * data,
                  const int          ndata,
                  float            * intensities,
                  float            * noise,
                  const int          ncol,
                  const int          icol);
void complete_intensities(const Position           * refpositions,
                          const int                * refindices, 
                          const int                  nref,
                          const int                  ncol,
                          const int                  icol, 
                          const float              * image,
                          const float              * noise_image,
                          const int                  nx,
                          const int                  ny,
                          float                    * intensities,
                          float                    * noise_dispersions,
                          const float                fwhm,
                          const float                min_dist,
                          AlignmentTransform       * transform);

#endif /*PROCESSREMAPPING_H_*/
