/*
 * Copyright (c) 2007 Illumina
 * Author: Klaus Maisinger
 * This source file is covered by the "Illumina Genome Analyzer
 * Software License Agreement" and bound by the terms therein.
 */

#include "XMLImageConfigHandler.h"
#include "ImageProcess.h"

XMLImageConfigHandler::XMLImageConfigHandler(AnalysisParameters *_params)
	: inParameters(0)
	, inFwhm(0)
	, inThreshold(0)
	, inDistance(0)
	, inMinMatch(0)
	, params(_params)    
{
}

XMLImageConfigHandler::~XMLImageConfigHandler()
{
}

void XMLImageConfigHandler::startElement(const char name[], const Attributes *attr)
{
    std::string tag(name);
    if (tag == "ImageParameters")
        inParameters = 1;
    if (tag == "Fwhm")
        inFwhm = 1;
    if (tag == "Threshold")
        inThreshold = 1;
    if (tag == "RemappingDistance")
        inDistance = 1;
    if (tag == "MinMatch")
    	inMinMatch = 1;
}

void XMLImageConfigHandler::characters(const char ch[], int start, int length)
{
    if (!inParameters)
        return;
    if (inFwhm)
        params->ClusterFwhm = atof(ch);
    if (inThreshold)
        params->GaussFactor = atof(ch);
    if (inDistance)
        params->ClusterDistance = atof(ch);
    if (inMinMatch)
        params->PearsonThreshold = atof(ch);
}

void XMLImageConfigHandler::endElement(const char name[])
{
    std::string tag(name);
    if (tag == "ImageParameters")
        inParameters = 0;
    if (tag == "Fwhm")
        inFwhm = 0;
    if (tag == "Threshold")
        inThreshold = 0;
     if (tag == "RemappingDistance")
        inDistance = 0;
    if (tag == "MinMatch")
    	inMinMatch = 0;
}
