/*
 * Time-stamp: <$Id: convertQuality.cpp,v 1.2 2008/07/17 15:44:49 rshaw Exp $>
 * 
 * Main program for data/file/format conversion module in the Illumina
 * data analysis pipeline.
 * Copyright (c) 2007 Illumina
 * Author: Lukasz Szajkowski
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

#include <cstdio>
#include <cassert>
#include <vector>
#include <string>
#include "QualityConverter.h"



const std::string usage =
        string("Usage: $0 [--useBases useBases]  [--read[1|2]] ")
                + string("[--numeric|--symbolic] [--qrawFile qrawFile] ")
                + string("readLength prbFile[s] \n");

int parseArguments(int argc, char * argv[], vector<std::string> &prbFileVector,
        std::string &qrawFile, std::string &useBases, int &read, int &toFile,
        int &formatNumeric, int &readLength) {
    int status = 0;
    int i = 1;
    int pairReadValue = 0;
    readLength = -1;
    int prbFileId = 0;
    if (argc < 2) {
        return -1;
    }

    while (i < argc && status == 0) {
        if (!strcmp(argv[i], "--useBases")) {
            if (i + 2 < argc) {
                useBases.append(argv[i + 1]);
                i++;
            } else {
                status = -i;
            }
        } else if (sscanf(argv[i], "--read%d", &pairReadValue)) {
            if (i + 2 < argc) {
                read = pairReadValue;
                if (read < 0 || read > 2) {
                    status = -i;
                }
                i++;
            } else {
                status = -i;
            }
        } else if (!strcmp(argv[i], "--numeric")) {
            if (formatNumeric > 0) {
                status = -i;
            }
            formatNumeric = 1;
        } else if (!strcmp(argv[i], "--symbolic")) {
            if (formatNumeric > 0) {
                status = -i;
            }
            formatNumeric = 0;
        } else if (!strcmp(argv[i], "--qrawFile")) {
            if (i + 2 < argc) {
                qrawFile.append(argv[i + 1]);
                i++;
            } else {
                status = -i;
            }
        } else {
		if (readLength == -1) {
                    readLength = atoi(argv[i]);
            	    if (readLength < 2) { // Minimum readLenth = 2
                        status = -i;
                    }
		} else {
            	    prbFileVector.push_back(std::string(argv[i]));
		}
		prbFileId++;
            //}
        }
        i++;
    }
    if (qrawFile.length() > 0) {
        toFile = 1;
    } else {
        toFile = 0;
    }
    read = pairReadValue;
    return status;
}

int main(int argc, char *argv[]) {
    vector<std::string> prbFileVector;
    std::string qrawFile;
    std::string useBases;
    int read;
    int formatNumeric = 0;
    int readLength;
    int toFile = 0;

    int status = parseArguments(argc, argv, prbFileVector, qrawFile, useBases, read,
            toFile, formatNumeric, readLength);

    if (status < 0) {
        if (status < -1) {
            int argIndex = status * (-1);
            fprintf(stderr, "Error parsing argument %d - [%s].", argIndex,
            argv[argIndex]);
        }
        fprintf(stderr, usage.c_str());
        exit(-1);
    }
    QualityConverter *converter = new QualityConverter(prbFileVector);
    converter->convert(qrawFile, useBases, read, toFile, formatNumeric,
            readLength);
    delete converter;
    return 0;
}
