// -*- mode: c++; indent-tabs-mode: nil; c-basic-offset: 4; -*-
/*
****************************************************************************/
// Copyright (c) Illumina 2008
// Author: Richard Shaw
// This source file is covered by the Illumina Genome Analyzer Software 
// License Agreement and bound by the terms therein.
/*****************************************************************************/

#ifndef EQP_OPTIONS_H
#define EQP_OPTIONS_H

/*****************************************************************************/

#include <string>
#include <list>
#include <map>

#include "CommandLine.hh"
#include "IntensityData.h"
#include "QualityData.h"

/*****************************************************************************/

class EQP_Options : public CommandLine::Options
{
 public:
    EQP_Options();

    std::string my_orig_read_lengths_str;

    typedef std::list<unsigned int> Orig_Read_Lens_List;
    typedef Orig_Read_Lens_List::const_iterator Orig_Read_Lens_List_CIter;
    Orig_Read_Lens_List my_orig_read_lens_list;

    unsigned int my_num_early_cycles;
    bool my_multi_read_flag;
    std::string my_sig2_compr_name;
    IntensityData::CompressionType my_sig2_compr;
    std::string my_qval_compr_name;
    File_Buffer::Compression_Type my_qval_compr;

    std::string my_qptypes_str;
    Quality_Data::QPTypes my_qptypes;

    std::string my_sig2_path_str;
    std::string my_prb_path_str;
    std::string my_seq_path_str;

    typedef std::list<std::string> File_Path_Str_List;
    typedef File_Path_Str_List::const_iterator File_Path_Str_List_CIter;
    File_Path_Str_List my_qval_path_str_list;

    typedef std::vector<unsigned int> Read_Num_Vec;
    typedef Read_Num_Vec::const_iterator Read_Num_Vec_CIter;
    Read_Num_Vec my_read_num_vec;

    static const unsigned int my_default_num_early_cycles = 12;

    bool get_read_cycle_range(unsigned int read_num,
                              unsigned int& read_lo_cycle_num,
                              unsigned int& read_hi_cycle_num);
    unsigned int total_num_cycles();

 private:
    virtual int getOptions(int argc, char* const argv[])
        throw (CommandLine::UnknownOptionException, 
               CommandLine::MissingOptionException, 
               CommandLine::InvalidArgumentException);
    bool parse_qual_fields(std::string& msg_str);
    virtual void processOptions()
        throw (CommandLine::InvalidArgumentException);
    virtual void getParameters(int argc, char* const argv[])
        throw (CommandLine::InvalidArgumentException);
    virtual int usage(int argc, char* const argv[],
                      const std::string &message = "");

    Quality_Data::QPTypes my_default_qptypes;
};

/*****************************************************************************/

#endif // ! EQP_OPTIONS_H

/*****************************************************************************/
