// -*- mode: c++; indent-tabs-mode: nil; c-basic-offset: 4; -*-
//
// Copyright 2008 Illumina, Inc
// Author: Chris Saunders
//
// This source file is covered by the Illumina Genome Analyzer Software
// License Agreement and bound by the terms therein.
//

#ifndef QPREDICTORFUNCTIONS_H
#define QPREDICTORFUNCTIONS_H

#include <vector>
#include <memory>

#include "PositionIndex.h"
#include "QPredictorTypes.h"
#include "QualityData.h"


namespace QPREDICTOR {

    struct FuncInitData {
        FuncInitData(const unsigned l,
                     const unsigned h,
                     const unsigned cl,
                     const unsigned ch,
                     const unsigned nec,
                     const std::vector<float*>& si,
                     const std::vector<float*>& pr,
                     const char* se,
                     const PositionIndex* pi)
            : low_cycle(l), high_cycle(h),
              n_clusters(cl), n_channels(ch), n_early_cycles(nec),
              sig(si), prob(pr), seq(se),pos(pi) {}

        unsigned low_cycle;
        unsigned high_cycle;
        unsigned n_clusters;
        unsigned n_channels;
        unsigned n_early_cycles;
        const std::vector<float*>& sig;
        const std::vector<float*>& prob;
        const char* seq;
        const PositionIndex* pos;
    };


    struct FuncBase {
        FuncBase(const FuncInitData& d) 
            : _data(d), _cycle_range(_data.high_cycle-(_data.low_cycle-1)) {}

        virtual ~FuncBase() {}

        virtual
        void get_cluster_values(const unsigned cluster_id,
                                std::vector<float>::iterator fval) = 0;

    protected:
        const FuncInitData& _data;
        const unsigned _cycle_range;
    };


    struct FuncSet {
        FuncSet(const FuncInitData& d);

        FuncBase* get_func_ptr(const index_t i) { return _qpms[i].get(); }
        FuncBase* get_func_ptr(const int i) { return get_func_ptr(static_cast<index_t>(i)); }
        
    private:
        const FuncInitData _data;
        std::auto_ptr<FuncBase> _qpms[SIZE];
    };
}


#endif
