// -*- mode: c++; indent-tabs-mode: nil; c-basic-offset: 4; -*-
//
// Copyright 2008 Illumina, Inc
// Author: Chris Saunders
//
// This source file is covered by the Illumina Genome Analyzer Software
// License Agreement and bound by the terms therein.
//

#include <iostream>

#include <cstdlib>

#include "QPredictorMotif5p13p1.h"


struct sf {
  sf(const char* i_s,const float i_f) : s(i_s), f(i_f) {}
  const char* s;
  float f;
};

extern const sf map_array_5p13p1[];
extern const unsigned map_array_5p13p1_size;
  


Motif5P13P1Data::Motif5P13P1Data() {
    
    static const unsigned nb(4);
    static const unsigned nb2(nb*nb);
    static const unsigned nb3(nb2*nb);
    static const unsigned expected_size(nb3+2*(nb2)+nb);

    for(unsigned i(0);i<map_array_5p13p1_size;++i){
        motif_scores[map_array_5p13p1[i].s] = map_array_5p13p1[i].f;
    }

    // check that at least the motif count is correct
    if(motif_scores.size() != expected_size){
        /// \todo standarize to pipeline error protocol
        std::cerr << "Invalid motif table\n";
        abort();
    }
    /// \todo add full motif coverage check
}

  

const sf map_array_5p13p1[] = {
sf("AAA",-36.624001),
sf("AAC",-35.917950),
sf("AAG",-35.956744),
sf("AAN",-36.931604),
sf("AAT",-35.989630),
sf("ACA",-31.653978),
sf("ACC",-32.244186),
sf("ACG",-32.040900),
sf("ACN",-31.829841),
sf("ACT",-31.335928),
sf("AGA",-34.733991),
sf("AGC",-35.536683),
sf("AGG",-34.492053),
sf("AGN",-35.069247),
sf("AGT",-34.926655),
sf("ATA",-34.363472),
sf("ATC",-34.700255),
sf("ATG",-35.207684),
sf("ATN",-35.490915),
sf("ATT",-37.714816),
sf("CAA",-36.427436),
sf("CAC",-33.094035),
sf("CAG",-35.099530),
sf("CAN",-35.217175),
sf("CAT",-35.169419),
sf("CCA",-34.455674),
sf("CCC",-27.987096),
sf("CCG",-29.587194),
sf("CCN",-31.140559),
sf("CCT",-30.895178),
sf("CGA",-34.566332),
sf("CGC",-35.369023),
sf("CGG",-34.324393),
sf("CGN",-34.733928),
sf("CGT",-34.758995),
sf("CTA",-31.367862),
sf("CTC",-35.385724),
sf("CTG",-35.341634),
sf("CTN",-34.862261),
sf("CTT",-37.048237),
sf("GAA",-35.546335),
sf("GAC",-34.024538),
sf("GAG",-34.879078),
sf("GAN",-34.776272),
sf("GAT",-33.836536),
sf("GCA",-34.774107),
sf("GCC",-32.024890),
sf("GCG",-34.575437),
sf("GCN",-33.593915),
sf("GCT",-33.250638),
sf("GGA",-32.013242),
sf("GGC",-34.263872),
sf("GGG",-33.219241),
sf("GGN",-32.523624),
sf("GGT",-33.653844),
sf("GTA",-32.781900),
sf("GTC",-33.787125),
sf("GTG",-31.659921),
sf("GTN",-33.747536),
sf("GTT",-35.745297),
sf("NAA",-36.316397),
sf("NAC",-34.229420),
sf("NAG",-34.981884),
sf("NAN",-35.205980),
sf("NAT",-35.047656),
sf("NCA",-33.967339),
sf("NCC",-31.142193),
sf("NCG",-31.724787),
sf("NCN",-32.173575),
sf("NCT",-31.684814),
sf("NGA",-34.398735),
sf("NGC",-36.004119),
sf("NGG",-33.914859),
sf("NGN",-34.780004),
sf("NGT",-34.784063),
sf("NTA",-33.056270),
sf("NTC",-35.139822),
sf("NTG",-34.372890),
sf("NTN",-34.949630),
sf("NTT",-37.229780),
sf("TAA",-34.609226),
sf("TAC",-33.154637),
sf("TAG",-31.947712),
sf("TAN",-33.709300),
sf("TAT",-34.481311),
sf("TCA",-34.579441),
sf("TCC",-30.980437),
sf("TCG",-31.153340),
sf("TCN",-32.129902),
sf("TCT",-31.269246),
sf("TGA",-35.706508),
sf("TGC",-36.509201),
sf("TGG",-35.464571),
sf("TGN",-37.014282),
sf("TGT",-35.899173),
sf("TTA",-33.985053),
sf("TTC",-36.430454),
sf("TTG",-34.920838),
sf("TTN",-35.780288),
sf("TTT",-38.123712)};

const unsigned map_array_5p13p1_size = sizeof(map_array_5p13p1)/sizeof(sf);
