// -*- mode: c++; indent-tabs-mode: nil; c-basic-offset: 4; -*-
//
// Copyright 2008 Illumina, Inc
// Author: Chris Saunders
//
// This source file is covered by the Illumina Genome Analyzer Software
// License Agreement and bound by the terms therein.
//

  
#include <iostream>

#include <cstdlib>

#include "QPredictorMotif5p23p2.h"


struct sf {
  sf(const char* i_s,const float i_f) : s(i_s), f(i_f) {}
  const char* s;
  float f;
};

extern const sf map_array_5p23p2[];
extern const unsigned map_array_5p23p2_size;
  

Motif5P23P2Data::Motif5P23P2Data() {

    static const unsigned nb(4);
    static const unsigned nb2(nb*nb);
    static const unsigned nb3(nb2*nb);
    static const unsigned nb4(nb3*nb);
    static const unsigned nb5(nb4*nb);
    static const unsigned expected_size(nb5+2*(nb4)+3*(nb3)+2*(nb2)+nb);

    for(unsigned i(0);i<map_array_5p23p2_size;++i){
        motif_scores[map_array_5p23p2[i].s] = map_array_5p23p2[i].f;
    }

    // check that at least the motif count is correct
    if(motif_scores.size() != expected_size){
        /// \todo standarize to pipeline error protocol
        std::cerr << "Invalid motif table\n";
        abort();
    }
    /// \todo add full motif coverage check
}



const sf map_array_5p23p2[] = {
sf("AAAAA",-36.220599),
sf("AAAAC",-36.447054),
sf("AAAAG",-36.447054),
sf("AAAAN",-36.447054),
sf("AAAAT",-36.289025),
sf("AAACA",-35.917950),
sf("AAACC",-35.917950),
sf("AAACG",-35.917950),
sf("AAACN",-35.917950),
sf("AAACT",-35.917950),
sf("AAAGA",-36.652611),
sf("AAAGC",-36.352929),
sf("AAAGG",-37.217271),
sf("AAAGN",-37.217271),
sf("AAAGT",-35.458784),
sf("AAANN",-37.217271),
sf("AAATA",-36.652611),
sf("AAATC",-37.217271),
sf("AAATG",-36.273638),
sf("AAATN",-37.217271),
sf("AAATT",-35.885362),
sf("AACAA",-32.119515),
sf("AACAC",-33.357402),
sf("AACAG",-33.357402),
sf("AACAN",-33.357402),
sf("AACAT",-33.357402),
sf("AACCA",-32.385272),
sf("AACCC",-31.767050),
sf("AACCG",-31.767050),
sf("AACCN",-31.767050),
sf("AACCT",-31.767050),
sf("AACGA",-31.262227),
sf("AACGC",-32.292605),
sf("AACGG",-32.040900),
sf("AACGN",-32.040900),
sf("AACGT",-32.505420),
sf("AACNN",-32.577722),
sf("AACTA",-31.699813),
sf("AACTC",-31.335928),
sf("AACTG",-31.335928),
sf("AACTN",-31.335928),
sf("AACTT",-31.263745),
sf("AAGAA",-34.600476),
sf("AAGAC",-34.828142),
sf("AAGAG",-34.828142),
sf("AAGAN",-34.828142),
sf("AAGAT",-34.828142),
sf("AAGCA",-34.919266),
sf("AAGCC",-34.919266),
sf("AAGCG",-34.919266),
sf("AAGCN",-34.919266),
sf("AAGCT",-34.919266),
sf("AAGGA",-34.919266),
sf("AAGGC",-34.919266),
sf("AAGGG",-34.919266),
sf("AAGGN",-34.919266),
sf("AAGGT",-34.919266),
sf("AAGNN",-34.919266),
sf("AAGTA",-34.919266),
sf("AAGTC",-34.919266),
sf("AAGTG",-34.919266),
sf("AAGTN",-34.919266),
sf("AAGTT",-34.919266),
sf("AATAA",-34.363472),
sf("AATAC",-34.363472),
sf("AATAG",-34.363472),
sf("AATAN",-34.363472),
sf("AATAT",-34.363472),
sf("AATCA",-34.700255),
sf("AATCC",-34.700255),
sf("AATCG",-34.700255),
sf("AATCN",-34.700255),
sf("AATCT",-34.700255),
sf("AATGA",-35.207684),
sf("AATGC",-35.207684),
sf("AATGG",-35.207684),
sf("AATGN",-35.207684),
sf("AATGT",-35.207684),
sf("AATNN",-35.209901),
sf("AATTA",-37.714816),
sf("AATTC",-37.714816),
sf("AATTG",-37.714816),
sf("AATTN",-37.714816),
sf("AATTT",-37.714816),
sf("ACAAA",-36.427436),
sf("ACAAC",-36.427436),
sf("ACAAG",-36.427436),
sf("ACAAN",-36.427436),
sf("ACAAT",-36.427436),
sf("ACACA",-33.094035),
sf("ACACC",-33.094035),
sf("ACACG",-33.094035),
sf("ACACN",-33.094035),
sf("ACACT",-33.094035),
sf("ACAGA",-34.081174),
sf("ACAGC",-34.644303),
sf("ACAGG",-33.218269),
sf("ACAGN",-33.218269),
sf("ACAGT",-33.987645),
sf("ACANN",-33.218269),
sf("ACATA",-35.169419),
sf("ACATC",-35.169419),
sf("ACATG",-35.169419),
sf("ACATN",-35.169419),
sf("ACATT",-35.169419),
sf("ACCAA",-34.455674),
sf("ACCAC",-34.455674),
sf("ACCAG",-34.455674),
sf("ACCAN",-34.455674),
sf("ACCAT",-34.455674),
sf("ACCCA",-29.819562),
sf("ACCCC",-27.987096),
sf("ACCCG",-27.987096),
sf("ACCCN",-27.987096),
sf("ACCCT",-27.987096),
sf("ACCGA",-28.858765),
sf("ACCGC",-29.587194),
sf("ACCGG",-29.587194),
sf("ACCGN",-29.587194),
sf("ACCGT",-29.587194),
sf("ACCNN",-33.379341),
sf("ACCTA",-30.895178),
sf("ACCTC",-30.794654),
sf("ACCTG",-30.895178),
sf("ACCTN",-30.895178),
sf("ACCTT",-30.813740),
sf("ACGAA",-34.441080),
sf("ACGAC",-34.737018),
sf("ACGAG",-34.737018),
sf("ACGAN",-34.737018),
sf("ACGAT",-34.737018),
sf("ACGCA",-34.249952),
sf("ACGCC",-34.051764),
sf("ACGCG",-34.196831),
sf("ACGCN",-34.617363),
sf("ACGCT",-34.196450),
sf("ACGGA",-33.920865),
sf("ACGGC",-33.897299),
sf("ACGGG",-33.893695),
sf("ACGGN",-34.013586),
sf("ACGGT",-33.666177),
sf("ACGNN",-34.733928),
sf("ACGTA",-34.483282),
sf("ACGTC",-34.500322),
sf("ACGTG",-34.562053),
sf("ACGTN",-34.640055),
sf("ACGTT",-34.441710),
sf("ACTAA",-31.367862),
sf("ACTAC",-31.367862),
sf("ACTAG",-31.367862),
sf("ACTAN",-31.367862),
sf("ACTAT",-31.367862),
sf("ACTCA",-35.385724),
sf("ACTCC",-35.385724),
sf("ACTCG",-35.385724),
sf("ACTCN",-35.385724),
sf("ACTCT",-35.385724),
sf("ACTGA",-35.341634),
sf("ACTGC",-35.341634),
sf("ACTGG",-35.341634),
sf("ACTGN",-35.341634),
sf("ACTGT",-35.341634),
sf("ACTNN",-34.170695),
sf("ACTTA",-37.048237),
sf("ACTTC",-37.048237),
sf("ACTTG",-37.048237),
sf("ACTTN",-37.048237),
sf("ACTTT",-37.048237),
sf("AGAAA",-35.368265),
sf("AGAAC",-35.676838),
sf("AGAAG",-35.676838),
sf("AGAAN",-35.676838),
sf("AGAAT",-35.290401),
sf("AGACA",-34.024538),
sf("AGACC",-34.024538),
sf("AGACG",-34.024538),
sf("AGACN",-34.024538),
sf("AGACT",-34.024538),
sf("AGAGA",-34.527620),
sf("AGAGC",-35.488587),
sf("AGAGG",-34.207827),
sf("AGAGN",-34.481126),
sf("AGAGT",-33.960849),
sf("AGANN",-34.776272),
sf("AGATA",-33.836536),
sf("AGATC",-33.836536),
sf("AGATG",-33.836536),
sf("AGATN",-33.836536),
sf("AGATT",-33.836536),
sf("AGCAA",-34.774107),
sf("AGCAC",-34.774107),
sf("AGCAG",-34.774107),
sf("AGCAN",-34.774107),
sf("AGCAT",-34.774107),
sf("AGCCA",-32.123018),
sf("AGCCC",-32.024890),
sf("AGCCG",-32.024890),
sf("AGCCN",-32.024890),
sf("AGCCT",-32.024890),
sf("AGCGA",-34.575437),
sf("AGCGC",-34.575437),
sf("AGCGG",-34.575437),
sf("AGCGN",-34.575437),
sf("AGCGT",-34.575437),
sf("AGCNN",-34.726515),
sf("AGCTA",-33.250638),
sf("AGCTC",-33.250638),
sf("AGCTG",-33.250638),
sf("AGCTN",-33.250638),
sf("AGCTT",-33.535746),
sf("AGGAA",-32.013242),
sf("AGGAC",-32.013242),
sf("AGGAG",-32.013242),
sf("AGGAN",-32.013242),
sf("AGGAT",-32.013242),
sf("AGGCA",-32.514251),
sf("AGGCC",-32.694257),
sf("AGGCG",-32.694257),
sf("AGGCN",-32.694257),
sf("AGGCT",-32.694257),
sf("AGGGA",-32.514251),
sf("AGGGC",-32.694257),
sf("AGGGG",-32.694257),
sf("AGGGN",-32.694257),
sf("AGGGT",-32.694257),
sf("AGGNN",-32.694257),
sf("AGGTA",-32.514251),
sf("AGGTC",-32.694257),
sf("AGGTG",-32.694257),
sf("AGGTN",-32.694257),
sf("AGGTT",-32.694257),
sf("AGTAA",-32.781900),
sf("AGTAC",-32.781900),
sf("AGTAG",-32.781900),
sf("AGTAN",-32.781900),
sf("AGTAT",-32.781900),
sf("AGTCA",-33.787125),
sf("AGTCC",-33.787125),
sf("AGTCG",-33.787125),
sf("AGTCN",-33.787125),
sf("AGTCT",-33.787125),
sf("AGTGA",-31.659921),
sf("AGTGC",-31.659921),
sf("AGTGG",-31.659921),
sf("AGTGN",-31.659921),
sf("AGTGT",-31.659921),
sf("AGTNN",-33.336829),
sf("AGTTA",-35.745297),
sf("AGTTC",-35.745297),
sf("AGTTG",-35.745297),
sf("AGTTN",-35.745297),
sf("AGTTT",-35.745297),
sf("ATAAA",-34.609226),
sf("ATAAC",-34.609226),
sf("ATAAG",-34.609226),
sf("ATAAN",-34.609226),
sf("ATAAT",-34.609226),
sf("ATACA",-33.154637),
sf("ATACC",-33.154637),
sf("ATACG",-33.154637),
sf("ATACN",-33.154637),
sf("ATACT",-33.154637),
sf("ATAGA",-31.947712),
sf("ATAGC",-31.947712),
sf("ATAGG",-31.947712),
sf("ATAGN",-31.947712),
sf("ATAGT",-31.947712),
sf("ATANN",-34.035641),
sf("ATATA",-34.481311),
sf("ATATC",-34.481311),
sf("ATATG",-34.481311),
sf("ATATN",-34.481311),
sf("ATATT",-34.481311),
sf("ATCAA",-34.579441),
sf("ATCAC",-34.579441),
sf("ATCAG",-34.579441),
sf("ATCAN",-34.579441),
sf("ATCAT",-34.579441),
sf("ATCCA",-31.316233),
sf("ATCCC",-30.980437),
sf("ATCCG",-30.980437),
sf("ATCCN",-30.980437),
sf("ATCCT",-30.757394),
sf("ATCGA",-28.542832),
sf("ATCGC",-31.153340),
sf("ATCGG",-31.153340),
sf("ATCGN",-31.153340),
sf("ATCGT",-31.153340),
sf("ATCNN",-32.523129),
sf("ATCTA",-31.076491),
sf("ATCTC",-31.052196),
sf("ATCTG",-31.643350),
sf("ATCTN",-31.076491),
sf("ATCTT",-31.154215),
sf("ATGAA",-34.441080),
sf("ATGAC",-34.737018),
sf("ATGAG",-34.737018),
sf("ATGAN",-34.737018),
sf("ATGAT",-34.737018),
sf("ATGCA",-35.163925),
sf("ATGCC",-35.260359),
sf("ATGCG",-35.280795),
sf("ATGCN",-35.760221),
sf("ATGCT",-35.276048),
sf("ATGGA",-34.513096),
sf("ATGGC",-34.412630),
sf("ATGGG",-34.728214),
sf("ATGGN",-34.806208),
sf("ATGGT",-34.295185),
sf("ATGNN",-37.014282),
sf("ATGTA",-35.165589),
sf("ATGTC",-35.393231),
sf("ATGTG",-35.370859),
sf("ATGTN",-35.597155),
sf("ATGTT",-35.318653),
sf("ATTAA",-33.985053),
sf("ATTAC",-33.985053),
sf("ATTAG",-33.985053),
sf("ATTAN",-33.985053),
sf("ATTAT",-33.730407),
sf("ATTCA",-36.430454),
sf("ATTCC",-36.430454),
sf("ATTCG",-36.430454),
sf("ATTCN",-36.430454),
sf("ATTCT",-36.430454),
sf("ATTGA",-34.920838),
sf("ATTGC",-34.920838),
sf("ATTGG",-34.920838),
sf("ATTGN",-34.920838),
sf("ATTGT",-31.333563),
sf("ATTNN",-35.917229),
sf("ATTTA",-38.123712),
sf("ATTTC",-38.123712),
sf("ATTTG",-38.123712),
sf("ATTTN",-38.123712),
sf("ATTTT",-38.123712),
sf("CAAAA",-35.994145),
sf("CAAAC",-35.306355),
sf("CAAAG",-35.378560),
sf("CAAAN",-35.698553),
sf("CAAAT",-35.972966),
sf("CAACA",-35.917950),
sf("CAACC",-35.917950),
sf("CAACG",-35.917950),
sf("CAACN",-35.917950),
sf("CAACT",-35.917950),
sf("CAAGA",-36.087952),
sf("CAAGC",-35.488587),
sf("CAAGG",-34.945812),
sf("CAAGN",-35.093838),
sf("CAAGT",-33.700298),
sf("CAANN",-36.931604),
sf("CAATA",-36.087952),
sf("CAATC",-35.258586),
sf("CAATG",-35.330006),
sf("CAATN",-35.509658),
sf("CAATT",-34.553453),
sf("CACAA",-30.881628),
sf("CACAC",-31.653978),
sf("CACAG",-31.653978),
sf("CACAN",-31.653978),
sf("CACAT",-31.653978),
sf("CACCA",-31.691076),
sf("CACCC",-30.378657),
sf("CACCG",-30.378657),
sf("CACCN",-30.378657),
sf("CACCT",-30.378657),
sf("CACGA",-31.262227),
sf("CACGC",-32.292605),
sf("CACGG",-32.040900),
sf("CACGN",-32.040900),
sf("CACGT",-32.040900),
sf("CACNN",-30.013478),
sf("CACTA",-30.457184),
sf("CACTC",-31.335928),
sf("CACTG",-31.335928),
sf("CACTN",-31.335928),
sf("CACTT",-31.263745),
sf("CAGAA",-34.145143),
sf("CAGAC",-34.361350),
sf("CAGAG",-34.433556),
sf("CAGAN",-34.438547),
sf("CAGAT",-34.367796),
sf("CAGCA",-34.739867),
sf("CAGCC",-34.755072),
sf("CAGCG",-34.751327),
sf("CAGCN",-34.928807),
sf("CAGCT",-34.768023),
sf("CAGGA",-34.107310),
sf("CAGGC",-34.100235),
sf("CAGGG",-34.213467),
sf("CAGGN",-34.117377),
sf("CAGGT",-34.114897),
sf("CAGNN",-35.069247),
sf("CAGTA",-34.729794),
sf("CAGTC",-34.727099),
sf("CAGTG",-34.841244),
sf("CAGTN",-34.801008),
sf("CAGTT",-34.818965),
sf("CATAA",-34.363472),
sf("CATAC",-34.363472),
sf("CATAG",-34.363472),
sf("CATAN",-34.363472),
sf("CATAT",-34.363472),
sf("CATCA",-34.700255),
sf("CATCC",-34.700255),
sf("CATCG",-34.700255),
sf("CATCN",-34.700255),
sf("CATCT",-34.700255),
sf("CATGA",-35.207684),
sf("CATGC",-35.207684),
sf("CATGG",-35.207684),
sf("CATGN",-35.207684),
sf("CATGT",-35.207684),
sf("CATNN",-35.865385),
sf("CATTA",-37.714816),
sf("CATTC",-37.714816),
sf("CATTG",-37.714816),
sf("CATTN",-37.714816),
sf("CATTT",-37.714816),
sf("CCAAA",-36.427436),
sf("CCAAC",-36.427436),
sf("CCAAG",-36.427436),
sf("CCAAN",-36.427436),
sf("CCAAT",-36.427436),
sf("CCACA",-33.094035),
sf("CCACC",-33.094035),
sf("CCACG",-33.094035),
sf("CCACN",-33.094035),
sf("CCACT",-33.094035),
sf("CCAGA",-34.944079),
sf("CCAGC",-35.357321),
sf("CCAGG",-34.080937),
sf("CCAGN",-34.263242),
sf("CCAGT",-34.372333),
sf("CCANN",-35.217175),
sf("CCATA",-35.169419),
sf("CCATC",-35.169419),
sf("CCATG",-35.169419),
sf("CCATN",-35.169419),
sf("CCATT",-35.169419),
sf("CCCAA",-34.455674),
sf("CCCAC",-34.455674),
sf("CCCAG",-34.455674),
sf("CCCAN",-34.455674),
sf("CCCAT",-34.455674),
sf("CCCCA",-29.562988),
sf("CCCCC",-27.987096),
sf("CCCCG",-27.987096),
sf("CCCCN",-27.987096),
sf("CCCCT",-29.134926),
sf("CCCGA",-28.858765),
sf("CCCGC",-29.587194),
sf("CCCGG",-29.587194),
sf("CCCGN",-29.587194),
sf("CCCGT",-29.587194),
sf("CCCNN",-28.018361),
sf("CCCTA",-30.895178),
sf("CCCTC",-30.794654),
sf("CCCTG",-30.895178),
sf("CCCTN",-30.895178),
sf("CCCTT",-30.813740),
sf("CCGAA",-34.145143),
sf("CCGAC",-33.796764),
sf("CCGAG",-33.805587),
sf("CCGAN",-33.847839),
sf("CCGAT",-33.852339),
sf("CCGCA",-33.978588),
sf("CCGCC",-33.827191),
sf("CCGCG",-33.972258),
sf("CCGCN",-34.317932),
sf("CCGCT",-33.971877),
sf("CCGGA",-33.591476),
sf("CCGGC",-33.696973),
sf("CCGGG",-33.693369),
sf("CCGGN",-33.746485),
sf("CCGGT",-33.465851),
sf("CCGNN",-34.733928),
sf("CCGTA",-34.212527),
sf("CCGTC",-34.218024),
sf("CCGTG",-34.279756),
sf("CCGTN",-34.263658),
sf("CCGTT",-34.159412),
sf("CCTAA",-31.367862),
sf("CCTAC",-31.367862),
sf("CCTAG",-31.367862),
sf("CCTAN",-31.367862),
sf("CCTAT",-31.367862),
sf("CCTCA",-35.385724),
sf("CCTCC",-35.385724),
sf("CCTCG",-35.385724),
sf("CCTCN",-35.385724),
sf("CCTCT",-35.385724),
sf("CCTGA",-35.341634),
sf("CCTGC",-35.341634),
sf("CCTGG",-35.341634),
sf("CCTGN",-35.341634),
sf("CCTGT",-35.341634),
sf("CCTNN",-34.922822),
sf("CCTTA",-37.048237),
sf("CCTTC",-37.048237),
sf("CCTTG",-37.048237),
sf("CCTTN",-37.048237),
sf("CCTTT",-37.048237),
sf("CGAAA",-35.213978),
sf("CGAAC",-34.666676),
sf("CGAAG",-34.854726),
sf("CGAAN",-35.019109),
sf("CGAAT",-35.097183),
sf("CGACA",-34.024538),
sf("CGACC",-34.024538),
sf("CGACG",-34.024538),
sf("CGACN",-34.024538),
sf("CGACT",-34.024538),
sf("CGAGA",-34.527620),
sf("CGAGC",-35.488587),
sf("CGAGG",-34.382028),
sf("CGAGN",-34.879078),
sf("CGAGT",-33.960849),
sf("CGANN",-34.776272),
sf("CGATA",-33.836536),
sf("CGATC",-33.836536),
sf("CGATG",-33.836536),
sf("CGATN",-33.836536),
sf("CGATT",-33.836536),
sf("CGCAA",-34.774107),
sf("CGCAC",-34.774107),
sf("CGCAG",-34.774107),
sf("CGCAN",-34.774107),
sf("CGCAT",-34.774107),
sf("CGCCA",-32.123018),
sf("CGCCC",-32.024890),
sf("CGCCG",-32.024890),
sf("CGCCN",-32.024890),
sf("CGCCT",-31.153824),
sf("CGCGA",-31.995911),
sf("CGCGC",-34.575437),
sf("CGCGG",-34.575437),
sf("CGCGN",-34.575437),
sf("CGCGT",-34.575437),
sf("CGCNN",-32.901635),
sf("CGCTA",-33.250638),
sf("CGCTC",-33.250638),
sf("CGCTG",-33.250638),
sf("CGCTN",-33.250638),
sf("CGCTT",-33.535746),
sf("CGGAA",-32.013242),
sf("CGGAC",-32.013242),
sf("CGGAG",-32.013242),
sf("CGGAN",-32.013242),
sf("CGGAT",-32.013242),
sf("CGGCA",-32.334244),
sf("CGGCC",-33.455282),
sf("CGGCG",-33.704428),
sf("CGGCN",-33.846675),
sf("CGGCT",-33.733407),
sf("CGGGA",-32.334244),
sf("CGGGC",-33.291258),
sf("CGGGG",-33.275471),
sf("CGGGN",-33.219241),
sf("CGGGT",-33.301062),
sf("CGGNN",-32.523624),
sf("CGGTA",-32.334244),
sf("CGGTC",-33.922144),
sf("CGGTG",-33.953677),
sf("CGGTN",-33.803919),
sf("CGGTT",-33.867027),
sf("CGTAA",-32.781900),
sf("CGTAC",-32.781900),
sf("CGTAG",-32.781900),
sf("CGTAN",-32.781900),
sf("CGTAT",-32.781900),
sf("CGTCA",-33.787125),
sf("CGTCC",-33.787125),
sf("CGTCG",-33.787125),
sf("CGTCN",-33.787125),
sf("CGTCT",-33.787125),
sf("CGTGA",-31.659921),
sf("CGTGC",-31.659921),
sf("CGTGG",-31.659921),
sf("CGTGN",-31.659921),
sf("CGTGT",-31.659921),
sf("CGTNN",-35.092092),
sf("CGTTA",-35.745297),
sf("CGTTC",-35.745297),
sf("CGTTG",-35.745297),
sf("CGTTN",-35.745297),
sf("CGTTT",-35.745297),
sf("CTAAA",-34.609226),
sf("CTAAC",-34.609226),
sf("CTAAG",-34.609226),
sf("CTAAN",-34.609226),
sf("CTAAT",-34.609226),
sf("CTACA",-33.154637),
sf("CTACC",-33.154637),
sf("CTACG",-33.154637),
sf("CTACN",-33.154637),
sf("CTACT",-33.154637),
sf("CTAGA",-31.947712),
sf("CTAGC",-31.947712),
sf("CTAGG",-31.947712),
sf("CTAGN",-31.947712),
sf("CTAGT",-31.947712),
sf("CTANN",-31.866186),
sf("CTATA",-34.481311),
sf("CTATC",-34.481311),
sf("CTATG",-34.481311),
sf("CTATN",-34.481311),
sf("CTATT",-34.481311),
sf("CTCAA",-34.579441),
sf("CTCAC",-34.579441),
sf("CTCAG",-34.579441),
sf("CTCAN",-34.579441),
sf("CTCAT",-33.660350),
sf("CTCCA",-30.255356),
sf("CTCCC",-30.255356),
sf("CTCCG",-30.255356),
sf("CTCCN",-30.255356),
sf("CTCCT",-30.506375),
sf("CTCGA",-28.542832),
sf("CTCGC",-31.153340),
sf("CTCGG",-31.153340),
sf("CTCGN",-31.153340),
sf("CTCGT",-31.947299),
sf("CTCNN",-31.693019),
sf("CTCTA",-30.494757),
sf("CTCTC",-30.761329),
sf("CTCTG",-31.352483),
sf("CTCTN",-30.494757),
sf("CTCTT",-30.863347),
sf("CTGAA",-34.145143),
sf("CTGAC",-35.026153),
sf("CTGAG",-35.083535),
sf("CTGAN",-35.260459),
sf("CTGAT",-35.029664),
sf("CTGCA",-35.148645),
sf("CTGCC",-35.063376),
sf("CTGCG",-35.083813),
sf("CTGCN",-35.497578),
sf("CTGCT",-35.079065),
sf("CTGGA",-34.557808),
sf("CTGGC",-34.438345),
sf("CTGGG",-34.753930),
sf("CTGGN",-34.840496),
sf("CTGGT",-34.320901),
sf("CTGNN",-37.014282),
sf("CTGTA",-35.078592),
sf("CTGTC",-35.124115),
sf("CTGTG",-35.101743),
sf("CTGTN",-35.238334),
sf("CTGTT",-35.049537),
sf("CTTAA",-33.985053),
sf("CTTAC",-33.985053),
sf("CTTAG",-33.985053),
sf("CTTAN",-33.985053),
sf("CTTAT",-33.730407),
sf("CTTCA",-36.430454),
sf("CTTCC",-36.430454),
sf("CTTCG",-36.430454),
sf("CTTCN",-36.430454),
sf("CTTCT",-36.430454),
sf("CTTGA",-34.920838),
sf("CTTGC",-34.920838),
sf("CTTGG",-34.920838),
sf("CTTGN",-34.920838),
sf("CTTGT",-31.333563),
sf("CTTNN",-36.003859),
sf("CTTTA",-38.123712),
sf("CTTTC",-38.123712),
sf("CTTTG",-38.123712),
sf("CTTTN",-38.123712),
sf("CTTTT",-38.123712),
sf("GAAAA",-35.994145),
sf("GAAAC",-35.559806),
sf("GAAAG",-35.632011),
sf("GAAAN",-36.036487),
sf("GAAAT",-35.972966),
sf("GAACA",-35.917950),
sf("GAACC",-35.917950),
sf("GAACG",-35.917950),
sf("GAACN",-35.917950),
sf("GAACT",-35.917950),
sf("GAAGA",-36.087952),
sf("GAAGC",-35.488587),
sf("GAAGG",-35.201971),
sf("GAAGN",-35.435382),
sf("GAAGT",-33.700298),
sf("GAANN",-36.931604),
sf("GAATA",-34.996220),
sf("GAATC",-33.904487),
sf("GAATG",-34.617247),
sf("GAATN",-33.904487),
sf("GAATT",-34.228970),
sf("GACAA",-30.881628),
sf("GACAC",-31.653978),
sf("GACAG",-31.653978),
sf("GACAN",-31.653978),
sf("GACAT",-31.653978),
sf("GACCA",-33.003494),
sf("GACCC",-32.244186),
sf("GACCG",-32.244186),
sf("GACCN",-32.244186),
sf("GACCT",-32.244186),
sf("GACGA",-31.262227),
sf("GACGC",-32.292605),
sf("GACGG",-32.040900),
sf("GACGN",-32.040900),
sf("GACGT",-32.040900),
sf("GACNN",-32.922100),
sf("GACTA",-31.335928),
sf("GACTC",-31.335928),
sf("GACTG",-31.335928),
sf("GACTN",-31.335928),
sf("GACTT",-31.263745),
sf("GAGAA",-34.145143),
sf("GAGAC",-34.614801),
sf("GAGAG",-34.687006),
sf("GAGAN",-34.776481),
sf("GAGAT",-34.621247),
sf("GAGCA",-35.082137),
sf("GAGCC",-35.102410),
sf("GAGCG",-35.097417),
sf("GAGCN",-35.536683),
sf("GAGCT",-35.119678),
sf("GAGGA",-34.363468),
sf("GAGGC",-34.356394),
sf("GAGGG",-34.469625),
sf("GAGGN",-34.458922),
sf("GAGGT",-34.371056),
sf("GAGNN",-35.069247),
sf("GAGTA",-34.794037),
sf("GAGTC",-34.791341),
sf("GAGTG",-34.905486),
sf("GAGTN",-34.886665),
sf("GAGTT",-34.883207),
sf("GATAA",-34.363472),
sf("GATAC",-34.363472),
sf("GATAG",-34.363472),
sf("GATAN",-34.363472),
sf("GATAT",-34.363472),
sf("GATCA",-34.700255),
sf("GATCC",-34.700255),
sf("GATCG",-34.700255),
sf("GATCN",-34.700255),
sf("GATCT",-34.700255),
sf("GATGA",-35.207684),
sf("GATGC",-35.207684),
sf("GATGG",-35.207684),
sf("GATGN",-35.207684),
sf("GATGT",-35.207684),
sf("GATNN",-35.228846),
sf("GATTA",-37.714816),
sf("GATTC",-37.714816),
sf("GATTG",-37.714816),
sf("GATTN",-37.714816),
sf("GATTT",-37.714816),
sf("GCAAA",-36.427436),
sf("GCAAC",-36.427436),
sf("GCAAG",-36.427436),
sf("GCAAN",-36.427436),
sf("GCAAT",-36.427436),
sf("GCACA",-33.094035),
sf("GCACC",-33.094035),
sf("GCACG",-33.094035),
sf("GCACN",-33.094035),
sf("GCACT",-33.094035),
sf("GCAGA",-34.688885),
sf("GCAGC",-35.049444),
sf("GCAGG",-34.433690),
sf("GCAGN",-34.433690),
sf("GCAGT",-34.392786),
sf("GCANN",-34.433690),
sf("GCATA",-35.169419),
sf("GCATC",-35.169419),
sf("GCATG",-35.169419),
sf("GCATN",-35.169419),
sf("GCATT",-35.169419),
sf("GCCAA",-32.484846),
sf("GCCAC",-34.455674),
sf("GCCAG",-34.455674),
sf("GCCAN",-34.455674),
sf("GCCAT",-34.455674),
sf("GCCCA",-29.250557),
sf("GCCCC",-27.987096),
sf("GCCCG",-27.987096),
sf("GCCCN",-27.987096),
sf("GCCCT",-27.987096),
sf("GCCGA",-29.934450),
sf("GCCGC",-31.010135),
sf("GCCGG",-31.010135),
sf("GCCGN",-31.010135),
sf("GCCGT",-31.010135),
sf("GCCNN",-31.233576),
sf("GCCTA",-30.183149),
sf("GCCTC",-30.488902),
sf("GCCTG",-30.183149),
sf("GCCTN",-30.183149),
sf("GCCTT",-30.498444),
sf("GCGAA",-34.145143),
sf("GCGAC",-34.187065),
sf("GCGAG",-34.195887),
sf("GCGAN",-34.368239),
sf("GCGAT",-34.242639),
sf("GCGCA",-34.113101),
sf("GCGCC",-33.961703),
sf("GCGCG",-34.106770),
sf("GCGCN",-34.497282),
sf("GCGCT",-34.106390),
sf("GCGGA",-33.807529),
sf("GCGGC",-33.913025),
sf("GCGGG",-33.909421),
sf("GCGGN",-34.034554),
sf("GCGGT",-33.681903),
sf("GCGNN",-34.733928),
sf("GCGTA",-34.426473),
sf("GCGTC",-34.431971),
sf("GCGTG",-34.493702),
sf("GCGTN",-34.548919),
sf("GCGTT",-34.373358),
sf("GCTAA",-31.367862),
sf("GCTAC",-31.367862),
sf("GCTAG",-31.367862),
sf("GCTAN",-31.367862),
sf("GCTAT",-31.367862),
sf("GCTCA",-35.385724),
sf("GCTCC",-35.385724),
sf("GCTCG",-35.385724),
sf("GCTCN",-35.385724),
sf("GCTCT",-35.385724),
sf("GCTGA",-35.341634),
sf("GCTGC",-35.341634),
sf("GCTGG",-35.341634),
sf("GCTGN",-35.341634),
sf("GCTGT",-35.341634),
sf("GCTNN",-35.504949),
sf("GCTTA",-37.048237),
sf("GCTTC",-37.048237),
sf("GCTTG",-37.048237),
sf("GCTTN",-37.048237),
sf("GCTTT",-37.048237),
sf("GGAAA",-35.213978),
sf("GGAAC",-34.614572),
sf("GGAAG",-34.802623),
sf("GGAAN",-34.949638),
sf("GGAAT",-35.097183),
sf("GGACA",-34.024538),
sf("GGACC",-34.024538),
sf("GGACG",-34.024538),
sf("GGACN",-34.024538),
sf("GGACT",-34.024538),
sf("GGAGA",-34.527620),
sf("GGAGC",-35.488587),
sf("GGAGG",-34.043704),
sf("GGAGN",-34.262296),
sf("GGAGT",-33.960849),
sf("GGANN",-34.776272),
sf("GGATA",-33.836536),
sf("GGATC",-33.836536),
sf("GGATG",-33.836536),
sf("GGATN",-33.836536),
sf("GGATT",-33.836536),
sf("GGCAA",-34.774107),
sf("GGCAC",-34.774107),
sf("GGCAG",-34.774107),
sf("GGCAN",-34.774107),
sf("GGCAT",-34.774107),
sf("GGCCA",-32.123018),
sf("GGCCC",-32.024890),
sf("GGCCG",-32.024890),
sf("GGCCN",-32.024890),
sf("GGCCT",-32.024890),
sf("GGCGA",-34.575437),
sf("GGCGC",-34.575437),
sf("GGCGG",-34.575437),
sf("GGCGN",-34.575437),
sf("GGCGT",-34.575437),
sf("GGCNN",-32.976991),
sf("GGCTA",-33.250638),
sf("GGCTC",-33.250638),
sf("GGCTG",-33.250638),
sf("GGCTN",-33.250638),
sf("GGCTT",-33.535746),
sf("GGGAA",-32.013242),
sf("GGGAC",-32.013242),
sf("GGGAG",-32.013242),
sf("GGGAN",-32.013242),
sf("GGGAT",-32.013242),
sf("GGGCA",-32.334244),
sf("GGGCC",-33.402231),
sf("GGGCG",-33.651377),
sf("GGGCN",-33.775940),
sf("GGGCT",-33.680356),
sf("GGGGA",-32.334244),
sf("GGGGC",-33.225626),
sf("GGGGG",-33.213786),
sf("GGGGN",-33.155738),
sf("GGGGT",-33.232979),
sf("GGGNN",-32.523624),
sf("GGGTA",-32.334244),
sf("GGGTC",-33.860818),
sf("GGGTG",-33.892350),
sf("GGGTN",-33.722151),
sf("GGGTT",-33.805701),
sf("GGTAA",-32.781900),
sf("GGTAC",-32.781900),
sf("GGTAG",-32.781900),
sf("GGTAN",-32.781900),
sf("GGTAT",-32.781900),
sf("GGTCA",-33.787125),
sf("GGTCC",-33.787125),
sf("GGTCG",-33.787125),
sf("GGTCN",-33.787125),
sf("GGTCT",-33.787125),
sf("GGTGA",-31.659921),
sf("GGTGC",-31.659921),
sf("GGTGG",-31.659921),
sf("GGTGN",-31.659921),
sf("GGTGT",-31.659921),
sf("GGTNN",-33.747536),
sf("GGTTA",-35.745297),
sf("GGTTC",-35.745297),
sf("GGTTG",-35.745297),
sf("GGTTN",-35.745297),
sf("GGTTT",-35.745297),
sf("GTAAA",-34.609226),
sf("GTAAC",-34.609226),
sf("GTAAG",-34.609226),
sf("GTAAN",-34.609226),
sf("GTAAT",-34.609226),
sf("GTACA",-33.154637),
sf("GTACC",-33.154637),
sf("GTACG",-33.154637),
sf("GTACN",-33.154637),
sf("GTACT",-33.154637),
sf("GTAGA",-31.947712),
sf("GTAGC",-31.947712),
sf("GTAGG",-31.947712),
sf("GTAGN",-31.947712),
sf("GTAGT",-31.947712),
sf("GTANN",-33.461656),
sf("GTATA",-34.481311),
sf("GTATC",-34.481311),
sf("GTATG",-34.481311),
sf("GTATN",-34.481311),
sf("GTATT",-34.481311),
sf("GTCAA",-34.579441),
sf("GTCAC",-34.579441),
sf("GTCAG",-34.579441),
sf("GTCAN",-34.579441),
sf("GTCAT",-34.579441),
sf("GTCCA",-30.980437),
sf("GTCCC",-30.980437),
sf("GTCCG",-30.980437),
sf("GTCCN",-30.980437),
sf("GTCCT",-30.757394),
sf("GTCGA",-28.542832),
sf("GTCGC",-31.153340),
sf("GTCGG",-31.153340),
sf("GTCGN",-31.153340),
sf("GTCGT",-31.153340),
sf("GTCNN",-32.395446),
sf("GTCTA",-29.714822),
sf("GTCTC",-30.371361),
sf("GTCTG",-30.962516),
sf("GTCTN",-29.714822),
sf("GTCTT",-30.473380),
sf("GTGAA",-34.145143),
sf("GTGAC",-35.244120),
sf("GTGAG",-35.301502),
sf("GTGAN",-35.551082),
sf("GTGAT",-35.247631),
sf("GTGCA",-35.293253),
sf("GTGCC",-35.207984),
sf("GTGCG",-35.228420),
sf("GTGCN",-35.690389),
sf("GTGCT",-35.223673),
sf("GTGGA",-34.693129),
sf("GTGGC",-34.573666),
sf("GTGGG",-34.889251),
sf("GTGGN",-35.020924),
sf("GTGGT",-34.456222),
sf("GTGNN",-37.014282),
sf("GTGTA",-35.170262),
sf("GTGTC",-35.215785),
sf("GTGTG",-35.193412),
sf("GTGTN",-35.360560),
sf("GTGTT",-35.141207),
sf("GTTAA",-33.985053),
sf("GTTAC",-33.985053),
sf("GTTAG",-33.985053),
sf("GTTAN",-33.985053),
sf("GTTAT",-33.730407),
sf("GTTCA",-36.430454),
sf("GTTCC",-36.430454),
sf("GTTCG",-36.430454),
sf("GTTCN",-36.430454),
sf("GTTCT",-36.430454),
sf("GTTGA",-34.920838),
sf("GTTGC",-34.920838),
sf("GTTGG",-34.920838),
sf("GTTGN",-34.920838),
sf("GTTGT",-31.333563),
sf("GTTNN",-36.870124),
sf("GTTTA",-38.123712),
sf("GTTTC",-38.123712),
sf("GTTTG",-38.123712),
sf("GTTTN",-38.123712),
sf("GTTTT",-38.123712),
sf("NAAAA",-35.994145),
sf("NAAAC",-35.792587),
sf("NAAAG",-35.888861),
sf("NAAAN",-36.624001),
sf("NAAAT",-35.972966),
sf("NAACA",-35.917950),
sf("NAACC",-35.917950),
sf("NAACG",-35.917950),
sf("NAACN",-35.917950),
sf("NAACT",-35.917950),
sf("NAAGA",-36.087952),
sf("NAAGC",-35.488587),
sf("NAAGG",-35.471741),
sf("NAAGN",-35.956744),
sf("NAAGT",-33.700298),
sf("NAANN",-36.931604),
sf("NAATA",-36.087952),
sf("NAATC",-35.494877),
sf("NAATG",-35.330006),
sf("NAATN",-35.989630),
sf("NAATT",-34.553453),
sf("NACAA",-30.881628),
sf("NACAC",-31.653978),
sf("NACAG",-31.653978),
sf("NACAN",-31.653978),
sf("NACAT",-31.653978),
sf("NACCA",-33.003494),
sf("NACCC",-32.244186),
sf("NACCG",-32.244186),
sf("NACCN",-32.244186),
sf("NACCT",-32.244186),
sf("NACGA",-31.262227),
sf("NACGC",-32.292605),
sf("NACGG",-32.040900),
sf("NACGN",-32.040900),
sf("NACGT",-32.040900),
sf("NACNN",-31.829841),
sf("NACTA",-31.335928),
sf("NACTC",-31.335928),
sf("NACTG",-31.335928),
sf("NACTN",-31.335928),
sf("NACTT",-31.263745),
sf("NAGAA",-34.145143),
sf("NAGAC",-34.532581),
sf("NAGAG",-34.628855),
sf("NAGAN",-34.733991),
sf("NAGAT",-34.541175),
sf("NAGCA",-35.082137),
sf("NAGCC",-35.102410),
sf("NAGCG",-35.097417),
sf("NAGCN",-35.536683),
sf("NAGCT",-35.119678),
sf("NAGGA",-34.353738),
sf("NAGGC",-34.344305),
sf("NAGGG",-34.495280),
sf("NAGGN",-34.492053),
sf("NAGGT",-34.363854),
sf("NAGNN",-35.069247),
sf("NAGTA",-34.789821),
sf("NAGTC",-34.786227),
sf("NAGTG",-34.938421),
sf("NAGTN",-34.926655),
sf("NAGTT",-34.908715),
sf("NATAA",-34.363472),
sf("NATAC",-34.363472),
sf("NATAG",-34.363472),
sf("NATAN",-34.363472),
sf("NATAT",-34.363472),
sf("NATCA",-34.700255),
sf("NATCC",-34.700255),
sf("NATCG",-34.700255),
sf("NATCN",-34.700255),
sf("NATCT",-34.700255),
sf("NATGA",-35.207684),
sf("NATGC",-35.207684),
sf("NATGG",-35.207684),
sf("NATGN",-35.207684),
sf("NATGT",-35.207684),
sf("NATNN",-35.490915),
sf("NATTA",-37.714816),
sf("NATTC",-37.714816),
sf("NATTG",-37.714816),
sf("NATTN",-37.714816),
sf("NATTT",-37.714816),
sf("NCAAA",-36.427436),
sf("NCAAC",-36.427436),
sf("NCAAG",-36.427436),
sf("NCAAN",-36.427436),
sf("NCAAT",-36.427436),
sf("NCACA",-33.094035),
sf("NCACC",-33.094035),
sf("NCACG",-33.094035),
sf("NCACN",-33.094035),
sf("NCACT",-33.094035),
sf("NCAGA",-34.944079),
sf("NCAGC",-35.357321),
sf("NCAGG",-34.577694),
sf("NCAGN",-35.099530),
sf("NCAGT",-34.372333),
sf("NCANN",-35.217175),
sf("NCATA",-35.169419),
sf("NCATC",-35.169419),
sf("NCATG",-35.169419),
sf("NCATN",-35.169419),
sf("NCATT",-35.169419),
sf("NCCAA",-34.455674),
sf("NCCAC",-34.455674),
sf("NCCAG",-34.455674),
sf("NCCAN",-34.455674),
sf("NCCAT",-34.455674),
sf("NCCCA",-27.987096),
sf("NCCCC",-27.987096),
sf("NCCCG",-27.987096),
sf("NCCCN",-27.987096),
sf("NCCCT",-27.987096),
sf("NCCGA",-28.858765),
sf("NCCGC",-29.587194),
sf("NCCGG",-29.587194),
sf("NCCGN",-29.587194),
sf("NCCGT",-29.587194),
sf("NCCNN",-31.140559),
sf("NCCTA",-30.895178),
sf("NCCTC",-30.794654),
sf("NCCTG",-30.895178),
sf("NCCTN",-30.895178),
sf("NCCTT",-30.813740),
sf("NCGAA",-34.145143),
sf("NCGAC",-34.258735),
sf("NCGAG",-34.270498),
sf("NCGAN",-34.566332),
sf("NCGAT",-34.332833),
sf("NCGCA",-34.566201),
sf("NCGCC",-34.364338),
sf("NCGCG",-34.557760),
sf("NCGCN",-35.369023),
sf("NCGCT",-34.557253),
sf("NCGGA",-33.925080),
sf("NCGGC",-34.065741),
sf("NCGGG",-34.060936),
sf("NCGGN",-34.324393),
sf("NCGGT",-33.757579),
sf("NCGNN",-34.733928),
sf("NCGTA",-34.525708),
sf("NCGTC",-34.533038),
sf("NCGTG",-34.615347),
sf("NCGTN",-34.758995),
sf("NCGTT",-34.454889),
sf("NCTAA",-31.367862),
sf("NCTAC",-31.367862),
sf("NCTAG",-31.367862),
sf("NCTAN",-31.367862),
sf("NCTAT",-31.367862),
sf("NCTCA",-35.385724),
sf("NCTCC",-35.385724),
sf("NCTCG",-35.385724),
sf("NCTCN",-35.385724),
sf("NCTCT",-35.385724),
sf("NCTGA",-35.341634),
sf("NCTGC",-35.341634),
sf("NCTGG",-35.341634),
sf("NCTGN",-35.341634),
sf("NCTGT",-35.341634),
sf("NCTNN",-34.862261),
sf("NCTTA",-37.048237),
sf("NCTTC",-37.048237),
sf("NCTTG",-37.048237),
sf("NCTTN",-37.048237),
sf("NCTTT",-37.048237),
sf("NGAAA",-35.213978),
sf("NGAAC",-34.900681),
sf("NGAAG",-35.151415),
sf("NGAAN",-35.546335),
sf("NGAAT",-35.097183),
sf("NGACA",-34.024538),
sf("NGACC",-34.024538),
sf("NGACG",-34.024538),
sf("NGACN",-34.024538),
sf("NGACT",-34.024538),
sf("NGAGA",-34.527620),
sf("NGAGC",-35.488587),
sf("NGAGG",-34.382028),
sf("NGAGN",-34.879078),
sf("NGAGT",-33.960849),
sf("NGANN",-34.776272),
sf("NGATA",-33.836536),
sf("NGATC",-33.836536),
sf("NGATG",-33.836536),
sf("NGATN",-33.836536),
sf("NGATT",-33.836536),
sf("NGCAA",-34.774107),
sf("NGCAC",-34.774107),
sf("NGCAG",-34.774107),
sf("NGCAN",-34.774107),
sf("NGCAT",-34.774107),
sf("NGCCA",-32.123018),
sf("NGCCC",-32.024890),
sf("NGCCG",-32.024890),
sf("NGCCN",-32.024890),
sf("NGCCT",-32.024890),
sf("NGCGA",-34.575437),
sf("NGCGC",-34.575437),
sf("NGCGG",-34.575437),
sf("NGCGN",-34.575437),
sf("NGCGT",-34.575437),
sf("NGCNN",-33.593915),
sf("NGCTA",-33.250638),
sf("NGCTC",-33.250638),
sf("NGCTG",-33.250638),
sf("NGCTN",-33.250638),
sf("NGCTT",-33.535746),
sf("NGGAA",-32.013242),
sf("NGGAC",-32.013242),
sf("NGGAG",-32.013242),
sf("NGGAN",-32.013242),
sf("NGGAT",-32.013242),
sf("NGGCA",-32.334244),
sf("NGGCC",-33.602948),
sf("NGGCG",-33.935144),
sf("NGGCN",-34.263872),
sf("NGGCT",-33.973782),
sf("NGGGA",-32.334244),
sf("NGGGC",-33.291258),
sf("NGGGG",-33.275471),
sf("NGGGN",-33.219241),
sf("NGGGT",-33.301062),
sf("NGGNN",-32.523624),
sf("NGGTA",-32.334244),
sf("NGGTC",-33.861503),
sf("NGGTG",-33.903546),
sf("NGGTN",-33.653844),
sf("NGGTT",-33.788013),
sf("NGTAA",-32.781900),
sf("NGTAC",-32.781900),
sf("NGTAG",-32.781900),
sf("NGTAN",-32.781900),
sf("NGTAT",-32.781900),
sf("NGTCA",-33.787125),
sf("NGTCC",-33.787125),
sf("NGTCG",-33.787125),
sf("NGTCN",-33.787125),
sf("NGTCT",-33.787125),
sf("NGTGA",-31.659921),
sf("NGTGC",-31.659921),
sf("NGTGG",-31.659921),
sf("NGTGN",-31.659921),
sf("NGTGT",-31.659921),
sf("NGTNN",-33.747536),
sf("NGTTA",-35.745297),
sf("NGTTC",-35.745297),
sf("NGTTG",-35.745297),
sf("NGTTN",-35.745297),
sf("NGTTT",-35.745297),
sf("NNAAA",-35.900337),
sf("NNAAC",-36.316397),
sf("NNAAG",-36.316397),
sf("NNAAN",-36.316397),
sf("NNAAT",-35.972966),
sf("NNACA",-31.951958),
sf("NNACC",-34.229420),
sf("NNACG",-34.229420),
sf("NNACN",-34.229420),
sf("NNACT",-33.839586),
sf("NNAGA",-34.981884),
sf("NNAGC",-35.488587),
sf("NNAGG",-34.981884),
sf("NNAGN",-34.981884),
sf("NNAGT",-33.700298),
sf("NNANN",-35.205980),
sf("NNATA",-35.047656),
sf("NNATC",-35.047656),
sf("NNATG",-35.330006),
sf("NNATN",-35.047656),
sf("NNATT",-34.553453),
sf("NNCAA",-34.106836),
sf("NNCAC",-32.022723),
sf("NNCAG",-34.904658),
sf("NNCAN",-33.967339),
sf("NNCAT",-34.302495),
sf("NNCCA",-31.951704),
sf("NNCCC",-28.452435),
sf("NNCCG",-31.258530),
sf("NNCCN",-31.142193),
sf("NNCCT",-31.480441),
sf("NNCGA",-30.337442),
sf("NNCGC",-31.797389),
sf("NNCGG",-31.131417),
sf("NNCGN",-31.724787),
sf("NNCGT",-33.707086),
sf("NNCNN",-32.173575),
sf("NNCTA",-30.398887),
sf("NNCTC",-31.502711),
sf("NNCTG",-32.603698),
sf("NNCTN",-31.684814),
sf("NNCTT",-31.755942),
sf("NNGAA",-34.145143),
sf("NNGAC",-34.398735),
sf("NNGAG",-34.398735),
sf("NNGAN",-34.398735),
sf("NNGAT",-34.398735),
sf("NNGCA",-36.004119),
sf("NNGCC",-36.004119),
sf("NNGCG",-36.004119),
sf("NNGCN",-36.004119),
sf("NNGCT",-36.004119),
sf("NNGGA",-33.914859),
sf("NNGGC",-33.914859),
sf("NNGGG",-33.914859),
sf("NNGGN",-33.914859),
sf("NNGGT",-33.914859),
sf("NNGNN",-34.780004),
sf("NNGTA",-34.784063),
sf("NNGTC",-34.784063),
sf("NNGTG",-34.784063),
sf("NNGTN",-34.784063),
sf("NNGTT",-34.784063),
sf("NNTAA",-33.828586),
sf("NNTAC",-33.642692),
sf("NNTAG",-31.755391),
sf("NNTAN",-33.056270),
sf("NNTAT",-32.643738),
sf("NNTCA",-36.097292),
sf("NNTCC",-35.421177),
sf("NNTCG",-34.208568),
sf("NNTCN",-35.139822),
sf("NNTCT",-34.615187),
sf("NNTGA",-35.564481),
sf("NNTGC",-34.260192),
sf("NNTGG",-36.396630),
sf("NNTGN",-34.372890),
sf("NNTGT",-31.543938),
sf("NNTNN",-34.949630),
sf("NNTTA",-37.305943),
sf("NNTTC",-37.581370),
sf("NNTTG",-37.348369),
sf("NNTTN",-37.229780),
sf("NNTTT",-36.879835),
sf("NTAAA",-34.609226),
sf("NTAAC",-34.609226),
sf("NTAAG",-34.609226),
sf("NTAAN",-34.609226),
sf("NTAAT",-34.609226),
sf("NTACA",-33.154637),
sf("NTACC",-33.154637),
sf("NTACG",-33.154637),
sf("NTACN",-33.154637),
sf("NTACT",-33.154637),
sf("NTAGA",-31.947712),
sf("NTAGC",-31.947712),
sf("NTAGG",-31.947712),
sf("NTAGN",-31.947712),
sf("NTAGT",-31.947712),
sf("NTANN",-33.709300),
sf("NTATA",-34.481311),
sf("NTATC",-34.481311),
sf("NTATG",-34.481311),
sf("NTATN",-34.481311),
sf("NTATT",-34.481311),
sf("NTCAA",-34.579441),
sf("NTCAC",-34.579441),
sf("NTCAG",-34.579441),
sf("NTCAN",-34.579441),
sf("NTCAT",-34.579441),
sf("NTCCA",-30.980437),
sf("NTCCC",-30.980437),
sf("NTCCG",-30.980437),
sf("NTCCN",-30.980437),
sf("NTCCT",-30.757394),
sf("NTCGA",-28.542832),
sf("NTCGC",-31.153340),
sf("NTCGG",-31.153340),
sf("NTCGN",-31.153340),
sf("NTCGT",-31.153340),
sf("NTCNN",-32.129902),
sf("NTCTA",-31.269246),
sf("NTCTC",-31.027901),
sf("NTCTG",-32.210210),
sf("NTCTN",-31.269246),
sf("NTCTT",-31.231938),
sf("NTGAA",-34.145143),
sf("NTGAC",-35.245417),
sf("NTGAG",-35.321927),
sf("NTGAN",-35.706508),
sf("NTGAT",-35.250099),
sf("NTGCA",-35.706749),
sf("NTGCC",-35.593057),
sf("NTGCG",-35.620306),
sf("NTGCN",-36.509201),
sf("NTGCT",-35.613976),
sf("NTGGA",-34.879628),
sf("NTGGC",-34.720345),
sf("NTGGG",-35.141125),
sf("NTGGN",-35.464571),
sf("NTGGT",-34.563752),
sf("NTGNN",-37.014282),
sf("NTGTA",-35.465904),
sf("NTGTC",-35.526601),
sf("NTGTG",-35.496771),
sf("NTGTN",-35.899173),
sf("NTGTT",-35.427164),
sf("NTTAA",-33.985053),
sf("NTTAC",-33.985053),
sf("NTTAG",-33.985053),
sf("NTTAN",-33.985053),
sf("NTTAT",-33.730407),
sf("NTTCA",-36.430454),
sf("NTTCC",-36.430454),
sf("NTTCG",-36.430454),
sf("NTTCN",-36.430454),
sf("NTTCT",-36.430454),
sf("NTTGA",-34.920838),
sf("NTTGC",-34.920838),
sf("NTTGG",-34.920838),
sf("NTTGN",-34.920838),
sf("NTTGT",-31.333563),
sf("NTTNN",-35.780288),
sf("NTTTA",-38.123712),
sf("NTTTC",-38.123712),
sf("NTTTG",-38.123712),
sf("NTTTN",-38.123712),
sf("NTTTT",-38.123712),
sf("TAAAA",-35.994145),
sf("TAAAC",-35.139162),
sf("TAAAG",-35.301896),
sf("TAAAN",-35.475630),
sf("TAAAT",-35.972966),
sf("TAACA",-35.917950),
sf("TAACC",-35.917950),
sf("TAACG",-35.917950),
sf("TAACN",-35.917950),
sf("TAACT",-35.917950),
sf("TAAGA",-36.087952),
sf("TAAGC",-35.488587),
sf("TAAGG",-35.260887),
sf("TAAGN",-35.379564),
sf("TAAGT",-33.700298),
sf("TAANN",-36.931604),
sf("TAATA",-35.818154),
sf("TAATC",-35.548355),
sf("TAATG",-35.439180),
sf("TAATN",-35.548355),
sf("TAATT",-35.050904),
sf("TACAA",-30.881628),
sf("TACAC",-31.653978),
sf("TACAG",-31.653978),
sf("TACAN",-31.653978),
sf("TACAT",-31.653978),
sf("TACCA",-33.003494),
sf("TACCC",-32.244186),
sf("TACCG",-32.244186),
sf("TACCN",-32.244186),
sf("TACCT",-32.244186),
sf("TACGA",-31.262227),
sf("TACGC",-32.292605),
sf("TACGG",-32.040900),
sf("TACGN",-32.040900),
sf("TACGT",-32.040900),
sf("TACNN",-31.346438),
sf("TACTA",-31.335928),
sf("TACTC",-31.335928),
sf("TACTG",-32.523880),
sf("TACTN",-31.335928),
sf("TACTT",-31.263745),
sf("TAGAA",-34.145143),
sf("TAGAC",-34.194158),
sf("TAGAG",-34.266363),
sf("TAGAN",-34.215623),
sf("TAGAT",-34.200603),
sf("TAGCA",-34.900188),
sf("TAGCC",-34.915394),
sf("TAGCG",-34.911648),
sf("TAGCN",-35.142570),
sf("TAGCT",-34.928344),
sf("TAGGA",-34.321605),
sf("TAGGC",-34.314530),
sf("TAGGG",-34.427761),
sf("TAGGN",-34.403104),
sf("TAGGT",-34.329192),
sf("TAGNN",-35.069247),
sf("TAGTA",-34.787733),
sf("TAGTC",-34.785038),
sf("TAGTG",-34.899183),
sf("TAGTN",-34.878260),
sf("TAGTT",-34.876904),
sf("TATAA",-34.363472),
sf("TATAC",-34.363472),
sf("TATAG",-34.363472),
sf("TATAN",-34.363472),
sf("TATAT",-34.363472),
sf("TATCA",-34.700255),
sf("TATCC",-34.700255),
sf("TATCG",-34.700255),
sf("TATCN",-34.700255),
sf("TATCT",-34.700255),
sf("TATGA",-35.207684),
sf("TATGC",-35.207684),
sf("TATGG",-35.207684),
sf("TATGN",-35.207684),
sf("TATGT",-33.572598),
sf("TATNN",-35.947821),
sf("TATTA",-37.714816),
sf("TATTC",-37.714816),
sf("TATTG",-37.714816),
sf("TATTN",-37.714816),
sf("TATTT",-37.714816),
sf("TCAAA",-36.427436),
sf("TCAAC",-36.427436),
sf("TCAAG",-36.427436),
sf("TCAAN",-36.427436),
sf("TCAAT",-36.427436),
sf("TCACA",-33.094035),
sf("TCACC",-33.094035),
sf("TCACG",-33.094035),
sf("TCACN",-33.094035),
sf("TCACT",-33.094035),
sf("TCAGA",-35.530033),
sf("TCAGC",-35.610209),
sf("TCAGG",-36.115986),
sf("TCAGN",-36.115986),
sf("TCAGT",-34.953551),
sf("TCANN",-36.115986),
sf("TCATA",-35.169419),
sf("TCATC",-35.169419),
sf("TCATG",-35.169419),
sf("TCATN",-35.169419),
sf("TCATT",-35.169419),
sf("TCCAA",-34.455674),
sf("TCCAC",-34.455674),
sf("TCCAG",-34.455674),
sf("TCCAN",-34.455674),
sf("TCCAT",-34.455674),
sf("TCCCA",-27.987096),
sf("TCCCC",-27.987096),
sf("TCCCG",-27.987096),
sf("TCCCN",-27.987096),
sf("TCCCT",-27.987096),
sf("TCCGA",-28.514913),
sf("TCCGC",-28.171061),
sf("TCCGG",-28.171061),
sf("TCCGN",-28.171061),
sf("TCCGT",-28.171061),
sf("TCCNN",-30.703442),
sf("TCCTA",-31.539339),
sf("TCCTC",-31.166996),
sf("TCCTG",-31.539339),
sf("TCCTN",-31.539339),
sf("TCCTT",-31.176539),
sf("TCGAA",-34.145143),
sf("TCGAC",-34.073861),
sf("TCGAG",-34.082684),
sf("TCGAN",-34.217302),
sf("TCGAT",-34.129435),
sf("TCGCA",-34.166989),
sf("TCGCC",-34.015591),
sf("TCGCG",-34.160658),
sf("TCGCN",-34.569133),
sf("TCGCT",-34.160278),
sf("TCGGA",-33.705699),
sf("TCGGC",-33.811195),
sf("TCGGG",-33.807592),
sf("TCGGN",-33.898781),
sf("TCGGT",-33.580074),
sf("TCGNN",-34.733928),
sf("TCGTA",-34.449917),
sf("TCGTC",-34.455415),
sf("TCGTG",-34.517146),
sf("TCGTN",-34.580178),
sf("TCGTT",-34.396802),
sf("TCTAA",-31.367862),
sf("TCTAC",-31.367862),
sf("TCTAG",-31.367862),
sf("TCTAN",-31.367862),
sf("TCTAT",-31.367862),
sf("TCTCA",-35.385724),
sf("TCTCC",-35.385724),
sf("TCTCG",-35.385724),
sf("TCTCN",-35.385724),
sf("TCTCT",-35.385724),
sf("TCTGA",-35.341634),
sf("TCTGC",-35.341634),
sf("TCTGG",-35.341634),
sf("TCTGN",-35.341634),
sf("TCTGT",-33.639573),
sf("TCTNN",-34.973637),
sf("TCTTA",-37.048237),
sf("TCTTC",-37.048237),
sf("TCTTG",-37.048237),
sf("TCTTN",-37.048237),
sf("TCTTT",-37.048237),
sf("TGAAA",-35.213978),
sf("TGAAC",-34.549126),
sf("TGAAG",-34.922543),
sf("TGAAN",-34.862376),
sf("TGAAT",-35.097183),
sf("TGACA",-34.024538),
sf("TGACC",-34.024538),
sf("TGACG",-34.024538),
sf("TGACN",-34.024538),
sf("TGACT",-34.024538),
sf("TGAGA",-34.527620),
sf("TGAGC",-35.488587),
sf("TGAGG",-34.386117),
sf("TGAGN",-34.292882),
sf("TGAGT",-33.960849),
sf("TGANN",-34.776272),
sf("TGATA",-33.836536),
sf("TGATC",-33.836536),
sf("TGATG",-33.836536),
sf("TGATN",-33.836536),
sf("TGATT",-33.836536),
sf("TGCAA",-34.774107),
sf("TGCAC",-34.774107),
sf("TGCAG",-34.774107),
sf("TGCAN",-34.774107),
sf("TGCAT",-34.774107),
sf("TGCCA",-32.123018),
sf("TGCCC",-32.024890),
sf("TGCCG",-32.024890),
sf("TGCCN",-32.024890),
sf("TGCCT",-32.024890),
sf("TGCGA",-34.575437),
sf("TGCGC",-34.575437),
sf("TGCGG",-34.575437),
sf("TGCGN",-34.575437),
sf("TGCGT",-34.575437),
sf("TGCNN",-33.486210),
sf("TGCTA",-33.290527),
sf("TGCTC",-33.290527),
sf("TGCTG",-33.290527),
sf("TGCTN",-33.290527),
sf("TGCTT",-33.413137),
sf("TGGAA",-32.013242),
sf("TGGAC",-32.013242),
sf("TGGAG",-32.013242),
sf("TGGAN",-32.013242),
sf("TGGAT",-32.013242),
sf("TGGCA",-32.334244),
sf("TGGCC",-33.434161),
sf("TGGCG",-33.683308),
sf("TGGCN",-33.818514),
sf("TGGCT",-33.712287),
sf("TGGGA",-32.334244),
sf("TGGGC",-33.248566),
sf("TGGGG",-33.236725),
sf("TGGGN",-33.186324),
sf("TGGGT",-33.255919),
sf("TGGNN",-32.523624),
sf("TGGTA",-32.334244),
sf("TGGTC",-33.866746),
sf("TGGTG",-33.898279),
sf("TGGTN",-33.730055),
sf("TGGTT",-33.811629),
sf("TGTAA",-32.781900),
sf("TGTAC",-32.781900),
sf("TGTAG",-32.781900),
sf("TGTAN",-32.781900),
sf("TGTAT",-32.781900),
sf("TGTCA",-33.787125),
sf("TGTCC",-33.787125),
sf("TGTCG",-33.787125),
sf("TGTCN",-33.787125),
sf("TGTCT",-33.787125),
sf("TGTGA",-31.659921),
sf("TGTGC",-31.659921),
sf("TGTGG",-31.659921),
sf("TGTGN",-31.659921),
sf("TGTGT",-31.798717),
sf("TGTNN",-31.801218),
sf("TGTTA",-35.745297),
sf("TGTTC",-35.745297),
sf("TGTTG",-35.745297),
sf("TGTTN",-35.745297),
sf("TGTTT",-35.745297),
sf("TTAAA",-34.609226),
sf("TTAAC",-34.609226),
sf("TTAAG",-34.609226),
sf("TTAAN",-34.609226),
sf("TTAAT",-34.609226),
sf("TTACA",-33.154637),
sf("TTACC",-33.154637),
sf("TTACG",-33.154637),
sf("TTACN",-33.154637),
sf("TTACT",-33.154637),
sf("TTAGA",-31.947712),
sf("TTAGC",-31.947712),
sf("TTAGG",-31.947712),
sf("TTAGN",-31.947712),
sf("TTAGT",-31.947712),
sf("TTANN",-34.512627),
sf("TTATA",-34.481311),
sf("TTATC",-34.481311),
sf("TTATG",-34.481311),
sf("TTATN",-34.481311),
sf("TTATT",-34.481311),
sf("TTCAA",-34.579441),
sf("TTCAC",-32.376528),
sf("TTCAG",-33.881548),
sf("TTCAN",-34.579441),
sf("TTCAT",-33.394367),
sf("TTCCA",-30.713626),
sf("TTCCC",-30.713626),
sf("TTCCG",-30.713626),
sf("TTCCN",-30.713626),
sf("TTCCT",-30.735510),
sf("TTCGA",-29.591488),
sf("TTCGC",-30.640144),
sf("TTCGG",-30.640144),
sf("TTCGN",-30.640144),
sf("TTCGT",-30.640144),
sf("TTCNN",-31.989993),
sf("TTCTA",-32.507280),
sf("TTCTC",-31.767591),
sf("TTCTG",-32.358745),
sf("TTCTN",-32.507280),
sf("TTCTT",-31.869609),
sf("TTGAA",-34.145143),
sf("TTGAC",-34.928057),
sf("TTGAG",-34.985439),
sf("TTGAN",-35.129664),
sf("TTGAT",-34.931568),
sf("TTGCA",-35.217693),
sf("TTGCC",-35.132424),
sf("TTGCG",-35.152861),
sf("TTGCN",-35.589642),
sf("TTGCT",-35.148113),
sf("TTGGA",-34.413213),
sf("TTGGC",-34.293750),
sf("TTGGG",-34.609335),
sf("TTGGN",-34.647702),
sf("TTGGT",-34.176306),
sf("TTGNN",-37.014282),
sf("TTGTA",-35.317993),
sf("TTGTC",-35.363516),
sf("TTGTG",-35.341144),
sf("TTGTN",-35.557535),
sf("TTGTT",-35.288938),
sf("TTTAA",-33.488064),
sf("TTTAC",-33.488064),
sf("TTTAG",-33.488064),
sf("TTTAN",-33.488064),
sf("TTTAT",-33.609235),
sf("TTTCA",-36.430454),
sf("TTTCC",-36.430454),
sf("TTTCG",-36.430454),
sf("TTTCN",-36.430454),
sf("TTTCT",-35.208799),
sf("TTTGA",-33.886064),
sf("TTTGC",-33.886064),
sf("TTTGG",-33.886064),
sf("TTTGN",-33.886064),
sf("TTTGT",-32.609814),
sf("TTTNN",-35.012608),
sf("TTTTA",-38.123712),
sf("TTTTC",-38.123712),
sf("TTTTG",-38.123712),
sf("TTTTN",-38.123712),
sf("TTTTT",-36.055428)};

const unsigned map_array_5p23p2_size = sizeof(map_array_5p23p2)/sizeof(sf);
