// -*- mode: c++; indent-tabs-mode: nil; c-basic-offset: 4; -*-
//
// Copyright 2008 Illumina, Inc
// Author: Chris Saunders
//
// This source file is covered by the Illumina Genome Analyzer Software
// License Agreement and bound by the terms therein.
//

#include <iostream>
#include <map>

#include <cstdlib>

#include "CompUtil.h"
#include "QPredictorTypes.h"


#if 0
/// \tmp change to follow pipeline error convention
///
#include <exception>
#include <string>
struct simple_exception : public std::exception {
    simple_exception(const char* s) : message(s) {}
    ~simple_exception() throw() {}
    const char* what() const throw() { return message.c_str(); }
    std::string message;
};
#endif



namespace QPREDICTOR {

    const char*
    get_label(const index_t i) {
        switch(i){
        case IN_READ_CYCLE_NUM:            return "in_read_cycle";
        case SIGNAL_DECAY:                 return "signal_decay";
        case UNBOUNDED_SIGNAL_DECAY:       return "unbounded_signal_decay";
        case HOMOPOL_LEN:                  return "homopol_len";
        case RAW_UNQUALITY:                return "raw_unquality";
        case UNCHASTITY:                   return "unchastity";
        case UNPURITY:                     return "unpurity";
        case SHIFTED_UNCHASTITY:           return "shifted_unchastity";
        case SHIFTED_UNPURITY:             return "shifted_unpurity";
        case SHIFTED_SUM_DIFF_UNCHASTITY:  return "shifted_sum_diff_unchastity";
        case GC_CONTENT:                   return "gc_content";
        case UNENTROPY:                    return "unentropy";
        case EDGINESS:                     return "edginess";
        case MAX_EARLY_UNCHASTITY:         return "max_early_unchastity";
        case MAX_EARLY_UNPURITY:           return "max_early_unpurity";
        case MAX_EARLY_SHIFTED_UNCHASTITY: return "max_early_shifted_unchastity";
        case MAX_EARLY_SHIFTED_UNPURITY:   return "max_early_shifted_unpurity";
        case MAX_EARLY_SHIFTED_SUM_DIFF_UNCHASTITY: 
                                           return "max_early_shifted_sum_diff_unchastity";
        case MAX_LOCAL_UNCHASTITY:         return "max_local_unchastity";
        case UNCHASTITY_NHOOD1:            return "unchastity_nhood1";
        case UNPURITY_NHOOD1:              return "unpurity_nhood1";
        case SHIFTED_UNCHASTITY_NHOOD1:    return "shifted_unchastity_nhood1";
        case SHIFTED_UNPURITY_NHOOD1:      return "shifted_unpurity_nhood1";
        case SHIFTED_SUM_DIFF_UNCHASTITY_NHOOD1: 
                                           return "shifted_sum_diff_unchastity_nhood1";
        case CONTEXT_5P1_3P1:              return "context_5p1_3p1";
        case CONTEXT_5P2_3P2:              return "context_5p2_3p2";
        default:
            /// \tmp change to follow pipeline error convention
            std::cerr << "FATAL:: unknown label for QPREDICTOR index";
            abort();
        }
    }


    index_t
    get_index(const char* label){
        typedef std::map<const char*,index_t,ltstr> labelmap;

        static bool is_init(false);
        static labelmap label_to_id;
        if(! is_init){
            for(int i(0);i<SIZE;++i) label_to_id[get_label(i)] = static_cast<index_t>(i);
            is_init=true;
        }

        labelmap::const_iterator li(label_to_id.find(label));
        if( li == label_to_id.end() ) return SIZE;
        else                          return li->second;
    }
}
