// -*- mode: c++; indent-tabs-mode: nil; c-basic-offset: 4; -*-
/*****************************************************************************/
//
// Copyright (c) Illumina 2008
// Author: R. J. Shaw
// This source file is covered by the "Solexa Public Source License"
// agreement and bound by the terms therein.
//
/*****************************************************************************/

#ifndef QUALITY_DATA_H
#define QUALITY_DATA_H

/*****************************************************************************/

#include <string>
#include <vector>
#include <list>
#include <map>

#include "File_Buffer.h"
#include "QPredictorTypes.h"


/*****************************************************************************/

// For the Phred-like algorithm, the predictor values have to increase
// in the direction of decreasing expected quality; hence `unchastity', etc.
// It is possible that called_base might in future be incorporated into the 
// prediction but currently it is included only for alignment scoring.

struct QPBufferType {
    void
    init(const unsigned n_predictors,
         const unsigned n_lines) {
        if(value.size() == n_predictors && called_base.size() == n_lines) return;

        value.resize(n_predictors);
        for(unsigned i(0);i<n_predictors;++i){
            value[i].resize(n_lines);
        }
        called_base.resize(n_lines);
    }

    /// \todo a simple 2d array type would be nice to have here:
    std::vector<std::vector<float> > value;
    std::vector<char> called_base;
};


/*****************************************************************************/

struct Quality_Data {
    typedef QPREDICTOR::index_t QPType;
    typedef std::vector<QPType> QPTypes;

    Quality_Data(const std::string& data_file_name,
                 const QPTypes& init_qptypes,
                 const File_Buffer::Compression_Type compression 
                 = File_Buffer::compressionNone)
        : my_file_buffer(data_file_name, compression), my_qptypes(init_qptypes) {}

    int write(const QPBufferType& qp_buffer);
//    int read_chunk(Quality_Predictors_Vec& quality_predictors_vec);

    const std::string file_path_str() { return my_file_buffer.file_path_str(); }

 private:
    File_Buffer my_file_buffer;
    QPTypes my_qptypes;
};

/*****************************************************************************/

//int read_quality_file_chunk(File_Buffer& file_buffer,
//                            Quality_Predictors_Vec& quality_predictors_vec);

/*****************************************************************************/

#endif // ! QUALITY_DATA_H

/*****************************************************************************/
