/*****************************************************************************/
// Copyright (c) Illumina 2008
// Author: Richard Shaw
// This source file is covered by the Illumina Genome Analyzer Software
// License Agreement and bound by the terms therein.
/*****************************************************************************/

#include "Align_Data.h"

/*****************************************************************************/

Align_Data::Align_Data(const std::string& data_file_name,
                       const File_Buffer::Compression_Type compression)
    : my_file_buffer(data_file_name, compression),
      my_cached_read_was_eof(false)
{
    ;
}

/*****************************************************************************/

Align_Data::~Align_Data()
{
    ;
}

/*****************************************************************************/

bool Align_Data::get_next_read_align_info(Read_Align_Info& read_align_info,
                                          bool& at_end_of_tile, bool& at_eof)
{
    if (my_cached_read_align_info.my_tile_num == 0) {
        // Cache was empty - need to fill it first.
        if (!get_next_read_align_info(my_cached_read_align_info,
                                      my_cached_read_was_eof)) {

            at_end_of_tile = false; // There were no clusters at all.
            at_eof = my_cached_read_was_eof;
            return false; // nothing read
        }
    }

    // Extract a read from the cache.
    read_align_info = my_cached_read_align_info;

    if ((at_eof = my_cached_read_was_eof)) {
        // Cached read was already known to be the last one available.
        at_end_of_tile = true; // no more data so end of tile
    } else {
        // Try to reload the cache.
        if (get_next_read_align_info(my_cached_read_align_info,
                                     my_cached_read_was_eof)) {
            at_end_of_tile = (my_cached_read_align_info.my_tile_num
                              != read_align_info.my_tile_num);
        } else {
            // Could not refill the cache, so the current read will be the
            // last one (end of tile, end of file) after all.
            at_end_of_tile = true;
            at_eof = true;
            return true;
        }
    }

    return true;
}

/*****************************************************************************/
