// -*- c++ -*-
/*****************************************************************************/
// Copyright (c) Illumina 2008
// Author: Richard Shaw
// This source file is covered by the Illumina Genome Analyzer Software
// License Agreement and bound by the terms therein.
/*****************************************************************************/

#ifndef ALIGN_DATA_H
#define ALIGN_DATA_H

/*****************************************************************************/

#include "File_Buffer.h"
#include "Read_Align_Info.h"

/*****************************************************************************/

class Align_Data
{
public:
    Align_Data(const std::string& data_file_name,
               const File_Buffer::Compression_Type compression 
               = File_Buffer::compressionNone);
    virtual ~Align_Data()=0;

    /** The return value is true if data was read.
        If at_eof is true the returned data is the last available.
        If at_end_of_tile is true the returned data is the last for the
        tile to which it belongs. (Either the next data read will be for a
        different tile or there is no more data.)
    */
    bool get_next_read_align_info(Read_Align_Info& read_align_info,
                                  bool& at_end_of_tile, bool& at_eof);

    virtual char blank_base()=0;

protected:
    virtual bool get_next_read_align_info(Read_Align_Info& read_align_info,
                                          bool& at_eof)=0;

    

    File_Buffer my_file_buffer;
    Read_Align_Info my_cached_read_align_info;
    bool my_cached_read_was_eof;
};

/*****************************************************************************/

#endif // ! ALIGN_DATA_H

/*****************************************************************************/
