// -*- c++ -*-
/*****************************************************************************/
// Copyright (c) Illumina 2008
// Author: Richard Shaw
// This source file is covered by the Illumina Genome Analyzer Software
// License Agreement and bound by the terms therein.
/*****************************************************************************/

#ifndef INFO_CONTENTS
#define INFO_CONTENTS

/*****************************************************************************/

#include <valarray>
#include <vector>
#include <iostream>

#include "Base_Calls.h"

/*****************************************************************************/

class Info_Contents
{
    friend std::ostream& operator<<(std::ostream& ostrm, const Info_Contents&);

 public:
    Info_Contents();
    ~Info_Contents();
    bool derive(Base_Calls<unsigned int>& base_calls,
                std::valarray<unsigned int>& cycle_non_blank_freqs);

    void set_cycle_prefix_ch(char cycle_prefix_ch) {
        my_cycle_prefix_ch = cycle_prefix_ch;
    }

 private:
    void derive_cumulative_stats();

    typedef std::vector<float> Cycle_Float_Vec;
    typedef std::vector<unsigned int> Cycle_Uint_Vec;
    typedef Cycle_Float_Vec::iterator Cycle_Float_Vec_Iter;
    typedef Cycle_Uint_Vec::iterator Cycle_Uint_Vec_Iter;

    Cycle_Float_Vec my_info_vec;
    Cycle_Uint_Vec my_aligned_vec;
    Cycle_Uint_Vec my_total_vec;

    Cycle_Float_Vec my_cum_info_vec;
    Cycle_Uint_Vec my_cum_aligned_vec;
    Cycle_Uint_Vec my_cum_total_vec;

    char my_cycle_prefix_ch;
};

/*****************************************************************************/

std::ostream& operator<<(std::ostream& ostrm,
                         const Info_Contents& info_content);

/*****************************************************************************/

#endif // ! INFO_CONTENTS

/*****************************************************************************/
