/*
 * Time-stamp: <$Id: QrawData.h,v 1.1 2008/07/02 11:05:38 craczy Exp $>
 * 
 * Copyright (c) 2007      Illumina
 * Author:       Lukasz Szajkowski
 *
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 */

#ifndef _QRAW_DATA_H_
#define _QRAW_DATA_H_

#include <string>
#include <vector>

class QrawData
{
    std::string _Filename;
public:
	QrawData(const char filename []);
	
    /**
     * The method is using serialized string to write the probabilities to the
     * _Filename. 
     */ 
    int write(const std::string & convertedString);
    
	/**
	 * The method is using serialize method to write the probabilities to the
	 * _Filename. 
	 */ 
    int write(const std::vector<int *> & probabilities,
              const int                  numClusters,
              const int                  numChannels,
              const int                  isNumeric);	
    /**
     * The method serializes qraw data to qraw string.
     * 
     * Takes a prb data and converts each record in the vector to a quality string
     * Smallest possible QV of -5 gets called ASCII 59=';' 
     * QV = 0  gets ASCII 64 = '@'
     * QV = 30 gets ASCII 94 = '^'
     *      
     * probabilities - vector of quartets of probabilities (already in ASCII(64+QV)) 
     * numClusters - number of clasters in the strin string
     * 
     * convertedString - output string
     */
    void serialize(
            const std::vector<int *>   & probabilities,
            const unsigned int           numClusters, 
            const int                    isNumeric,
            std::string                & convertedString);
    
    
	virtual ~QrawData();
};

#endif /*_QRAW_DATA_H_*/
