/*  Time-stamp: <$Id: QualityConverter.h,v 1.1 2008/07/02 11:05:38 craczy Exp $> */
/*
 * Copyright (c) Illumina 2007  
 * Author: Lukasz Szajkowski
 * This source file is covered by the "Solexa Public Source License"
 * agreement and bound by the terms therein.
 * 
 * @version $Revision: 1.1 $
 * $Log: QualityConverter.h,v $
 * Revision 1.1  2008/07/02 11:05:38  craczy
 *
 * Jira Key: DAP-205
 *
 * Revision 1.5  2008/02/27 17:05:14  lszajkowski
 * Add support for reading many prb files.
 *
 * Revision 1.4  2008/01/14 13:48:57  lszajkowski
 * Cosmetic changes
 *
 * Revision 1.3  2008/01/14 11:26:24  lszajkowski
 * Add run main procedures.
 *
 * Revision 1.2  2008/01/11 17:13:55  lszajkowski
 * Refactor the code:
 * - separate the read, parse and process procedures
 * - improve performance
 * - C version of tokenizer
 *
 * Revision 1.1  2008/01/08 18:31:09  lszajkowski
 * Add first version of convertQuality rewrite to C++.
 *
 */

#ifndef _QUALITY_CONVERTER_H_
#define _QUALITY_CONVERTER_H_

#include <string>
#include <vector>

#include "Converter.h"
using namespace std;

class QualityConverter : public Converter {

private:
    int maxValue(const vector<int> & values);

    vector<std::string> _dataSourceNames; // File or other
public:

    /** Default constructor
     */
    QualityConverter();

    /** Constructor
     */
    QualityConverter(const vector<std::string> &filenames);

    virtual ~QualityConverter();
    
    /**
     * The method converts prb data (quality scores) to 
     * qraw.
     * 
     * Takes a prb data and converts each record in the vector to a quality vector
     * Conversion factor is chr=ASCII(64+QV) (NB that's 64 decimal)
     * so smallest possible QV of -5 gets called ASCII 59=';' 
     * QV = 0  gets ASCII 64 = '@'
     * QV = 30 gets ASCII 94 = '^'
     *      
     * probabilities - vector of quartets of probabilities
     * numClusters - number of clasters in the strin string
     * numChannels - number of chanels (at the moment default 4)
     * 
     * convertedProbabilities - output vector
     */
    void toQrawData(
            const std::vector<float *> & probabilities,
            const unsigned int           numClusters, 	    
            const unsigned int           numChannels,
            const vector<unsigned int> & baseIndexTable, 
            std::vector<int *>         & convertedProbabilities);
    
    
    /**
     * The method converts prb file to qraw file. 
     * 
     */
    int convert(const std::string & qrawFile,
                const std::string & useBases, 
                const int           read, 
                const int           toFile,
                const int           formatNumeric, 
                const int           readLength);
};

#endif /*_QUALITY_CONVERTER_H_*/
