// -*- c++ -*-
/*****************************************************************************/
// Copyright (c) Illumina 2008
// Author: Richard Shaw
// This source file is covered by the Illumina Genome Analyzer Software
// License Agreement and bound by the terms therein.
/*****************************************************************************/

#ifndef STATS_SET_H
#define STATS_SET_H

/*****************************************************************************/

#include <valarray>

#include "Base_Calls.h"
#include "Cum_Errs.h"
#include "File_Buffer.h"
#include "Basic_Stats.h"
#include "Info_Contents.h"

/*****************************************************************************/

class Stats_Set
{
 public:
    Stats_Set();
    ~Stats_Set();

    /// This is intended only for initialisation.
    void set_num_cycles(const unsigned int num_cycles);
    unsigned int num_cycles() const;

    void add_align_score(unsigned int align_score);
    void add_align_score(const Stats_Set& stats_set);
    void apply_non_blank_mask(const std::valarray<unsigned int>&
                              non_blank_mask);
    void derive_stats();
    bool write(File_Buffer& file_buffer) const;

    typedef Base_Calls<unsigned int> Uint_Base_Calls;
    Uint_Base_Calls& base_calls() { return my_base_calls; }
    Cum_Errs& cum_errs() { return my_cum_errs; }

    void mark_as_all_reads() { my_all_reads_flag = true; }
    bool is_all_reads() const { return my_all_reads_flag; }

 private:
    typedef Base_Calls<int> Int_Base_Calls;
    typedef Base_Calls<float> Float_Base_Calls;

    void derive_proportions(const Uint_Base_Calls& base_calls,
                            Float_Base_Calls& pc_base_calls,
                            Uint_Base_Calls::Dimension sum_dimension,
                            bool as_percents = true,
                            bool exclude_blanks = false);
    void derive_likelihoods();
    void set_formats();

    bool my_all_reads_flag;

    unsigned int my_num_bases;
    unsigned int my_num_errs;
    unsigned int my_num_blanks;
    unsigned int my_num_unique_aligns;
    unsigned int my_align_score_sum;

    std::valarray<unsigned int> my_cycle_non_blank_freqs;

    Uint_Base_Calls my_base_calls;
    Uint_Base_Calls my_sum_over_cycles_base_calls;

    Basic_Stats my_basic_stats;

    Float_Base_Calls my_pc_reads_by_ref;
    Float_Base_Calls my_pc_reads_excl_blanks_by_ref;
    Float_Base_Calls my_pc_refs_by_read;
    Float_Base_Calls my_prop_reads_by_ref_cycle;

    Info_Contents my_info_contents;
    Int_Base_Calls my_likelihoods;
    Cum_Errs my_cum_errs;
};

/*****************************************************************************/

#endif // ! STATS_SET_H

/*****************************************************************************/
