// -*- c++ -*-
/*****************************************************************************/
// Copyright (c) Illumina 2008
// Author: Richard Shaw
// This source file is covered by the Illumina Genome Analyzer Software
// License Agreement and bound by the terms therein.
/*****************************************************************************/

#ifndef TILE_STATS_H
#define TILE_STATS_H

/*****************************************************************************/

#include <string>
#include <list>
#include <map>
#include <stdexcept>
#include <valarray>

#include "File_Buffer.h"
#include "Non_Blank_Filter.h"
#include "Read_Align_Info.h"
#include "Stats_Set.h"

/*****************************************************************************/

class Tile_Stats
{
 public:
    typedef enum { QUALITY_RAW, QUALITY_FILTERED } Quality_Type;
    typedef std::list<Quality_Type> Quality_Type_List;
    typedef Quality_Type_List::const_iterator Quality_Type_List_CIter;

    /// Code assumes 0 is valid for neither lane num nor tile num.
    enum { MIN_LANE_NUM=1, MAX_LANE_NUM=8, MIN_TILE_NUM=1 };

    Tile_Stats(const unsigned int num_reads);
    ~Tile_Stats();

    void add_read_align_info(const Read_Align_Info& read_align_info);
    void derive_stats();
    bool write_score_file(Quality_Type quality_type,
                          File_Buffer& file_buffer) const;

    unsigned int lane_num() { return my_lane_num; }
    unsigned int tile_num() { return my_tile_num; }

    static const std::string hdr_str(unsigned int lane_num,
                                     unsigned int tile_num,
                                     Quality_Type quality_type,
                                     unsigned int read_num);

 private:
    typedef std::map<std::string, Stats_Set*> Key_Stats_Set_Ptr_Map;
    typedef Key_Stats_Set_Ptr_Map::iterator Key_Stats_Set_Ptr_Map_Iter;
    typedef Key_Stats_Set_Ptr_Map::const_iterator Key_Stats_Set_Ptr_Map_CIter;

    static const std::string qual_str(Quality_Type quality_type);
    void make_key_str(Quality_Type quality_type,
                      const unsigned int read_num,
                      std::string& key_str) const;
    void handle_new_read(const unsigned int read_num,
                         const unsigned int num_cycles);
    void derive_all_reads_stats();

    unsigned int my_lane_num;
    unsigned int my_tile_num;
    unsigned int my_num_reads;
    unsigned int my_curr_read_num;
    unsigned int my_curr_read_num_cycles;

    // Actually for 1s and 0s not bools.
    std::valarray<unsigned int> my_non_blank_mask;
    Non_Blank_Filter my_non_blank_filter;

    // Actually for 1s and 0s not bools.
    std::valarray<unsigned int> my_one_read_cum_errs_mask;

    Key_Stats_Set_Ptr_Map my_key_stats_set_ptr_map;

    // Cached Stats_Set ptrs.
    Stats_Set* my_all_stats_set_ptr;
    Stats_Set* my_passed_filter_stats_set_ptr;

    Quality_Type_List my_quality_type_list;
    static const unsigned int our_all_reads_read_num;
};

/*****************************************************************************/

#endif // ! TILE_STATS_H

/*****************************************************************************/
