/*****************************************************************************/
//
// Copyright (c) Illumina 2008
// Author: R. J. Shaw
// This source file is covered by the "Solexa Public Source License"
// agreement and bound by the terms therein.
//
/*****************************************************************************/

#include <string>
#include <iostream>

#include "qcalibration/QualityData.h"

/*****************************************************************************/

enum {ERR_OK = 0, ERR_USAGE = 1, ERR_FILE_READ = 2, ERR_FILE_WRITE = 3};

/*****************************************************************************/

int usage(const std::string& app_name, int ret_val)
{
    std::cerr << "Usage: " << app_name
              << " <in qval path> <out qval path>" << std::endl;
}

/*****************************************************************************/

int main(int num_args, char* cstr_arg_arr[])
{
    // Parse args.

    const std::string app_name(cstr_arg_arr[0]);

    if (num_args != 3) {
        return usage(app_name, ERR_USAGE);
    }

    const std::string in_qval_path_str(cstr_arg_arr[1]);
    const std::string out_qval_path_str(cstr_arg_arr[2]);

    Quality_Data::Compression_Type in_compr(Quality_Data::compressionNone);
    Quality_Data::Compression_Type out_compr(Quality_Data::compressionNone);

    Quality_Predictors_Vec quality_predictors_vec;


    // Read the file.


    Quality_Data in_quality_data(in_qval_path_str, in_compr);

    const int read_ret(in_quality_data.read(quality_predictors_vec));

    if (read_ret != 0) {
        std::cerr << "Failed to read `" << in_qval_path_str
                  << "'" << std::endl;
        return ERR_FILE_READ;
    }


    // Write out a copy.

    Quality_Data out_quality_data(out_qval_path_str, out_compr);

    const int write_ret(out_quality_data.write(quality_predictors_vec));

    if (write_ret != 0) {
        std::cerr << "Failed to write `" << out_qval_path_str
                  << "'" << std::endl;
        return ERR_FILE_WRITE;
    }

    return ERR_OK;
}

/*****************************************************************************/
