##
## the location of the cppunit shared libraries will be needed for
## LD_LIBRARY_PATH
##

if (HAVE_CPPUNIT)
    get_filename_component(CPPUNIT_LOCATION ${HAVE_CPPUNIT} PATH)
else (HAVE_CPPUNIT)
    message(FATAL_ERROR "cppunit not found")
endif (HAVE_CPPUNIT)

##
## find all the source files
##

file (GLOB GA_TEST_SOURCE_LIST "*.cpp")

##
## create the targets to build the tests
##

set(GA_CPPUNIT_TEST_NAME cppunitTest)
add_executable(${GA_UNIQUE_PREFIX}${GA_CPPUNIT_TEST_NAME} ${GA_TEST_SOURCE_LIST})
set_target_properties(${GA_UNIQUE_PREFIX}${GA_CPPUNIT_TEST_NAME} PROPERTIES OUTPUT_NAME ${GA_CPPUNIT_TEST_NAME})
add_test(ga_test_common ${GA_CPPUNIT_TEST_NAME})

include_directories   (${GA_ALL_INCLUDES} ${GA_OPT_INC} "${CMAKE_SOURCE_DIR}/c++/unittest")
target_link_libraries (${GA_UNIQUE_PREFIX}${GA_CPPUNIT_TEST_NAME}
                       ${GA}_${GA_LIB_DIR} ${GA_AVAILABLE_LIBRARIES} 
                       ga_cppunit ${Boost_LIBRARIES} cppunit dl z bz2)

##
## Run some sanity check on the source file
##
foreach(GA_CPPUNIT_SOURCE_FILE ${GA_TEST_SOURCE_LIST})
    add_custom_command(TARGET ${GA_UNIQUE_PREFIX}${GA_CPPUNIT_TEST_NAME} 
                       PRE_BUILD
                       COMMAND ${CMAKE_SOURCE_DIR}/opt/bin/check-source.sh ARGS ${GA_CPPUNIT_SOURCE_FILE}.checked ${GA_CPPUNIT_SOURCE_FILE}
                       COMMENT "Sanity check on ${GA_CPPUNIT_SOURCE_FILE}")
endforeach(GA_CPPUNIT_SOURCE_FILE)

add_custom_command(OUTPUT ${CMAKE_CURRENT_BINARY_DIR}/RegistryNames.txt 
                   COMMAND ${CMAKE_COMMAND} -E copy ${CMAKE_CURRENT_SOURCE_DIR}/RegistryNames.txt ${CMAKE_CURRENT_BINARY_DIR}
		   DEPENDS ${CMAKE_CURRENT_SOURCE_DIR}/RegistryNames.txt
                   COMMENT "Copying RegistryNames.txt for ${GA_UNIQUE_PREFIX}${GA_CPPUNIT_TEST_NAME}")

##
## create the targets to run the tests
##
add_custom_command(OUTPUT  ${CMAKE_CURRENT_BINARY_DIR}/${GA_CPPUNIT_TEST_NAME}.passed 
		   COMMAND export LD_LIBRARY_PATH=${CPPUNIT_LOCATION} && ./${GA_CPPUNIT_TEST_NAME}
	           COMMAND touch ${GA_CPPUNIT_TEST_NAME}.passed  
		   DEPENDS ${GA_UNIQUE_PREFIX}${GA_CPPUNIT_TEST_NAME} ${CMAKE_CURRENT_BINARY_DIR}/RegistryNames.txt
		   COMMENT "Running unit tests ${GA_UNIQUE_PREFIX}${GA_CPPUNIT_TEST_NAME}")
add_custom_target(${GA_UNIQUE_PREFIX}passed ALL DEPENDS ${CMAKE_CURRENT_BINARY_DIR}/${GA_CPPUNIT_TEST_NAME}.passed)
