#!/bin/sh
# PROJECT: RAMM
# MODULE: RAMMSetup.sh
# AUTHOR: Gery Vessere
# Copyright (c) 2008 Illumina

# this script configures folders locations for RAMM and the type of installation
# desired (Standalone or distributed). It also prompts the user for how often 
# they want the sync script to ftp into Illumina's server, the default value 
# will be 120 minutes.

RAMMSCRIPTBASE='gatherDiagnosticData'
RAMMSCRIPT="$RAMMSCRIPTBASE.sh"
SYNCSCRIPTBASE="sendDiagnosticData"
SYNCSCRIPT="$SYNCSCRIPTBASE.sh"

umask a+w

MACHINENAME=`uname -n`


if [[ -n $1 && $1 = "default" ]]
then
	QSTANDALONE=1
fi

# find out which kind of pipeline version we are on
# and set the folder name that will receive the conf files
THISSCRIPTLOC=$(cd `dirname "$0"` && pwd)
LOC=`echo $THISSCRIPTLOC | grep "opt/RAMM" | wc -l`

if [ $LOC -eq 1 ]
then
	# we're in a pipeline 1.1 folder hierarchy
	PIPELINEGE1=1
else
	# we're in a post 1.1 hierarchy
	PIPELINEGE1=0
fi

if [ $PIPELINEGE1 -eq 1 ]
then
	# we're in a pipeline 1.1 folder hierarchy
	PIPELINE_CONFIG_FOLDER_BINREL="../share/RAMM"
	PIPELINE_CONFIG_FOLDER="$THISSCRIPTLOC/../../share/RAMM"
else
	# we're in a post 1.1 hierarchy
	PIPELINE_CONFIG_FOLDER=$THISSCRIPTLOC
fi

if [ ! -d $PIPELINE_CONFIG_FOLDER ]
then
	mkdir -p $PIPELINE_CONFIG_FOLDER 2> /dev/null
fi

# check whether the crontab is already installed and prompt for deletion
INSTALLED=`crontab -l | grep "$SYNCSCRIPT" | wc -l`

if [[ $INSTALLED -ge 1 && $QSTANDALONE -ne 1 ]]
then
	read -p "the $SYNCSCRIPT script is already installed in your crontab file would you like to uninstall it (y/n) ? " YN
	
	if [[ $YN = 'y' ]]
	then
		crontab -l | grep -v "$SYNCSCRIPT" > cron.tmp
		crontab -r
		crontab cron.tmp
		rm -f cron.tmp
		exit
	fi
fi

if [[ ! $QSTANDALONE -eq 1 ]]
then
	echo "This script will guide you through the configuration of the RAMM (Remote Access Mining and Monitoring) script. 
RAMM's function is to zip the diagnostic results of your runs and send them to Illumina's FTP server for analysis and/or troubleshooting."
	echo "-------------------------------------------------------------------"
	echo ""
fi

function getFolderLoc
{
	DIR=/notadir
	ISDIR=0
	WRITABLE=0
	while [[ $ISDIR -eq 0 || $WRITABLE -eq 0 ]]
	do
		read -p "$1" DIR
		
                DIR=${DIR%/}
		if [ -z $DIR ]
		then
			echo "you did not specify any folder, please enter a folder location"
		else
			if [[ ! -d $DIR || ! -w $DIR ]]
			then
				echo "the folder you specified does not exist or is not writeable, please try again"
			else
				ISDIR=1
				if [ ! -w $DIR ]
				then 
					echo "
The folder is not writable. please allow read/write access with the following command \"chmod a+rw $DIR\" or verify the mounted directory is read/writable"
				else
					WRITABLE=1
				fi
			fi
		fi
	done

}

function getEmailSettings
{
	echo "in order to set up email reporting once a file has been sent to Illumina, please let us know your SMTP Servers, Domain and Emails needed to configure outgoing emails: "
	read -p "Please enter your SMTP Server (eg. smtp.illumina.com), enter nothing for run defaults (standalone installs) or no email (distributed installs): " SMTP
	read -p "Please enter your Domain (which domain does this machine sit on, this is needed for your smtp server to trust you as a source of email and figure if it can relay emails from you (eg. hwus.illumina.com), enter nothing for run defaults (standalone installs) or no email (distributed installs): " DOMAIN
	read -p "Please enter a space separated list of emails that will receive notifications, enter nothing for run defaults (standalone installs) or no email (distributed installs): " EMAILS
}


FTPFOLDER_TEXT="
Please specify a folder that will be a recipient to the zip files to be uploaded to Illumina's FTP Server, typically this folder should be regularly cleaned up since RAMM will leave the zip files sent for your review. Every zip file takes about 50Mb of disk space."


OPMODEDONE=0
while [ ! $OPMODEDONE -eq 1 ]
do

if [[ ! $QSTANDALONE -eq 1 ]]
then

read -p "RAMM can function in 2 different modes, standalone or distributed.
1-In a standalone installation, one machine is responsible for zipping the results of its runs and sending those results over to the Illumina ftp server.
2-In a distributed installation, one or more Pipeline analysis servers are isolated from the internet and need to deposit their run diagnostic information into a shared network folder. This network folder is accessible by one machine that is connected to the internet and will take responsibility for uploading the zip files to Illumina's ftp server.
----> " OPERATIONMODE
else
	# quiet standalone mode for makefile install
	OPERATIONMODE=1
fi

	if [[ -n $OPERATIONMODE && $OPERATIONMODE -eq 1 ]]
	then
		if [[ ! $QSTANDALONE -eq 1 ]]
		then
			echo "OK -- Standalone Installation"
		fi
		DIR=/tmp/RammFtpCache
		rm -f $THISSCRIPTLOC/$SYNCSCRIPTBASE.conf 2>&1 > /dev/null

		echo "FTP Folder=$DIR" > $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf
		echo "System Configs Folder=[pipelineBin]/$PIPELINE_CONFIG_FOLDER_BINREL/Configs/SystemConfigs" >> $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf
		cat $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf > $PIPELINE_CONFIG_FOLDER/$RAMMSCRIPTBASE.conf
		echo "Transferred Files Folder=./Transferred" >> $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf
		echo "FTP Configs Folder=[pipelineBin]/$PIPELINE_CONFIG_FOLDER_BINREL/Configs/FtpConfigs" >> $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf
		echo "Report Configs Folder=[pipelineBin]/$PIPELINE_CONFIG_FOLDER_BINREL/Configs/ReportConfigs" >> $PIPELINE_CONFIG_FOLDER/$RAMMSCRIPTBASE.conf
		echo "FTP Synchronization Script=[pipelineBin]/$SYNCSCRIPT" >> $PIPELINE_CONFIG_FOLDER/$RAMMSCRIPTBASE.conf
		
		if [[ ! $QSTANDALONE -eq 1 ]]
		then
			getEmailSettings
		fi
		
		if [ -z $SMTP ]
		then
			SMTP="[runEmailServer]"
		fi
		
		if [ -z $DOMAIN ]
		then
			DOMAIN="[runDomain]"
		fi

		if [ -z $EMAILS ]
		then
			EMAILS="[runEmails]"
		fi
		
		
		echo "SMTP Server=$SMTP" >> $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf
		echo "Your Domain=$DOMAIN" >> $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf
		echo "Emails=$EMAILS" >> $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf

echo $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf

		FTPFOLDER=$DIR

		mkdir -p $PIPELINE_CONFIG_FOLDER/Configs/SystemConfigs 2> /dev/null
		mkdir -p $PIPELINE_CONFIG_FOLDER/Configs/ReportConfigs 2> /dev/null
		mkdir -p $PIPELINE_CONFIG_FOLDER/Configs/FtpConfigs 2> /dev/null
		mkdir -p $DIR 2> /dev/null

		OPMODEDONE=1
	elif [[ -n $OPERATIONMODE && $OPERATIONMODE -eq 2 ]]
	then
		echo "OK -- Distributed Installation"
		
		CONFIGMACHINEDONE=0
		while [ ! $CONFIGMACHINEDONE -eq 1 ]
		do
			read -p "Are you configuring: 
1- One of the pipeline analysis servers ?
2- The FTP component ?
----> " CONFIGMACHINE
			if [ $CONFIGMACHINE = "1" ]
			then
				getFolderLoc "$FTPFOLDER_TEXT
For things to be done properly, this should be a mount point that the FTP component also has access to (on a different machine).
----> "
				echo "FTP Folder=$DIR/FTP" > $PIPELINE_CONFIG_FOLDER/$RAMMSCRIPTBASE.conf
				mkdir -p $DIR/FTP 2> /dev/null
				echo "System Configs Folder=$DIR/Configs/SystemConfigs" >> $PIPELINE_CONFIG_FOLDER/$RAMMSCRIPTBASE.conf
				mkdir -p $DIR/Configs/SystemConfigs 2> /dev/null
				echo "Report Configs Folder=[pipelineBin]/$PIPELINE_CONFIG_FOLDER_BINREL/Configs/ReportConfigs" >> $PIPELINE_CONFIG_FOLDER/$RAMMSCRIPTBASE.conf
				mkdir -p "${PIPELINE_CONFIG_FOLDER}/Configs/ReportConfigs" 2> /dev/null
				# no syncScript stuff here because it's handled by the ftp component on the cron job
				if [ ! -d $DIR/FTP/$MACHINENAME ]
				then
					mkdir $DIR/FTP/$MACHINENAME 2> /dev/null
				fi
				
				chmod -R a+w $DIR 2> /dev/null
				
				CONFIGMACHINEDONE=1
			elif [ $CONFIGMACHINE = "2" ]
			then
				getFolderLoc "$FTPFOLDER_TEXT
This should be the same mounted folder you configured (will configure?) on all the Pipeline analysis machines.
----> "

				# In this case the config folder is the same folder the script is located in
				PIPELINE_CONFIG_FOLDER=$THISSCRIPTLOC
				echo "FTP Folder=$DIR/FTP" > $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf
				mkdir -p $DIR/FTP 2> /dev/null
				echo "System Configs Folder=$DIR/Configs/SystemConfigs" >> $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf
				mkdir -p $DIR/Configs/SystemConfigs 2> /dev/null
				echo "Transferred Files Folder=$DIR/Transferred" >> $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf # this should be ending as a subfolder of a run (in GERALD)
				mkdir -p $DIR/Transferred 2> /dev/null
				echo "FTP Configs Folder=$DIR/Configs/FtpConfigs" >> $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf
				mkdir -p $DIR/Configs/FtpConfigs 2> /dev/null
				chmod -R a+w $DIR/Configs/FtpConfigs 2> /dev/null

				getEmailSettings
		
				echo "SMTP Server=$SMTP" >> $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf
				echo "Your Domain=$DOMAIN" >> $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf
				echo "Emails=$EMAILS" >> $PIPELINE_CONFIG_FOLDER/$SYNCSCRIPTBASE.conf

				NUMERIC=0
				while [[ $NUMERIC = 0 && $CONFIGMACHINE -eq 2 && $OPERATIONMODE -eq 2 ]]
				do
			        	read -p "This script installs a cron job on your user account in order to run the syncFolder.bash script at a regular interval

Please specify how often (in minutes) you want the syncFolder.bash script to check-in
to Illumina's FTP Server to send your run diagnostic data (default 120 minutes): " INTERVAL

				        expr $INTERVAL + 1 2>&1 > /dev/null
				        if [ $? = 0 ]
				        then
				                NUMERIC=1
				        else
				                echo ""
				                echo "---->  Error: Please enter a numeric value"
				                NUMERIC=0
				        fi
	
				done

				if [[ -z $INTERVAL || $INTERVAL = "" ]]
				then
				        INTERVAL=120
				fi
				
				crontab -l > cron.tmp
				echo "* */$INTERVAL     *       *       *       $THISSCRIPTLOC/$SYNCSCRIPT" >> cron.tmp

				crontab cron.tmp
				rm -f cron.tmp
			
				CONFIGMACHINEDONE=1
			else
				echo "please enter one of the allowed choices"
				CONFIGMACHINEDONE=0
			fi
		done		

		OPMODEDONE=1
	else
		echo "please enter one of the allowed choices"
		OPMODEDONE=0
	fi
done
	

