#!/bin/sh

# PROJECT: RAMM
# MODULE: sendDiagnosticData.sh
# AUTHOR: Gery Vessere
# Copyright (c) 2008 Illumina

# this script synchronizes the contents of the folder specified in Pipeline/share/RAMM/sendDiagnosticData.conf with the configured 
# ftp site.

# collect the ftp server information

# a complicated way to get a log file name...
# in case the script gets renamed


[[ `basename $0` =~ "(.+).sh" ]]
SCRIPT_BASE=${BASH_REMATCH[1]}
CSU_LOG=$SCRIPT_BASE.log
MACHINENAME=`uname -n`
THISSCRIPTLOC=$(cd `dirname "$0"` && pwd)

if [ -z $DEBUG ]
then
	DEBUG=0
fi

umask a+w

DEFAULTFTPSERVER='ftp-hayward.illumina.com'
DEFAULTFTPUSER='fas'
DEFAULTFTPPASS='1fas6781'
DEFAULTFTPLOC='Reports'


function updateFTPSettings
{
# Update the ftp settings if need be
for computer in `find ${FTP_FOLDER} -maxdepth 1 -mindepth 1 -type d -printf "%P\n" | egrep -v "^(SystemConfigs|Transferred|forReview)$"`;
do
	UPDATEFTP=${SYSCONF_FOLDER}/${computer}_FtpUpdate
	UPDATEALLFTP=${SYSCONF_FOLDER}/All_FtpUpdate
	ACK_UPDATEFTP=${SYSCONF_FOLDER}/ACK/ACK_${computer}_FtpUpdate
	ACK_UPDATEALLFTP=${SYSCONF_FOLDER}/ACK/ACK_${computer}_All_FtpUpdate

	DOCHANGE=0	

	if [[ -e $UPDATEFTP || -e $UPDATEALLFTP ]]
	then
		if [[ -e $UPDATEFTP && $UPDATEFTP -nt ${ACK_UPDATEFTP} ]]
		then
			SERVER=`cat $UPDATEFTP | cut -d" " -f2`
			USER=`cat $UPDATEFTP | cut -d" " -f4`
			PASS=`cat $UPDATEFTP | cut -d" " -f6`
			FOLDER=`cat $UPDATEFTP | cut -d" " -f8`
			FOLDER=${FOLDER%/}
			DOCHANGE=1
		elif [[ -e $UPDATEALLFTP && $UPDATEALLFTP -nt ${ACK_UPDATEALLFTP} ]]
		then
			SERVER=`cat $UPDATEALLFTP | cut -d" " -f2`
			USER=`cat $UPDATEALLFTP | cut -d" " -f4`
			PASS=`cat $UPDATEALLFTP | cut -d" " -f6`
			FOLDER=`cat $UPDATEALLFTP | cut -d" " -f8`
			FOLDER=${FOLDER%/}
			DOCHANGE=1
		fi
		IFS="
"
		if [ $DOCHANGE -eq 1 ]
		then
			log "updating ftp connection information for computer $computer" 1

			# update file with newly downloaded ftp settings 

			rm -f /tmp/.ftp.tmp 2>&1 > /dev/null
			
			if [ ! -e /tmp/.ftp.tmp ]
			then
				echo "Server=$SERVER
Username=$USER
Password=$PASS
Folder=$FOLDER" > /tmp/.ftp.tmp
			
				mv -f /tmp/.ftp.tmp ${FTPCONFIGS_FOLDER}/$computer.ftp
				log "ftp connection information updated for $computer" 1
				if [ -e $UPDATEFTP ]
				then
					touch ${ACK_UPDATEFTP}
				elif [ -e $UPDATEALLFTP ]
				then
					touch ${ACK_UPDATEALLFTP}
				fi
		
				log "acknowledging this change to the main ftp server" 1
		
				callMainFtp "mirror -n -R -I ACK* ${SYSCONF_FOLDER}/ ./SystemConfigs
quit
"
			else
				log "ftp settings not updated because I could not delete the file /tmp/.ftp.tmp... probably some permission issue" 1
			fi
	
		fi
	fi
done


}





# checks folder exists and is writable
function checkFolder
{
        if [[ ! -d $1 ]]
        then
		log "$SCRIPT_BASE was not previously configured or some of the configured folders ($1) are not accessible on this machine. Please run the RAMMSetup.sh script in your Pipeline installation." 1
                exit
        fi
        if [[ ! -w $1 ]]
        then
		log "Directory $1 is not writable for this machine/user, please make the folder writable using the 'chmod a+w $1' or 'chmod g+w' (if you can restrict your pipeline users to a specific group name); also if $1 is on a mounted partition, make sure it is mounted read/writable. ${SCRIPT_BASE}.sh will stop execution" 1
                exit
        fi
}

# takes a variable name and expands its content if it contains
# bracketted generic directives
function expandDirectives
{
        variableName=$1
        # [pipelineBin] -> $THISSCRIPTLOC
        f=$(eval "echo $`echo $variableName`")
        f=${f/\[pipelineBin\]/`echo $THISSCRIPTLOC`}
        eval "$variableName=\"$f\""

        # [geraldFolder] -> $GERALD_PATH
        f=$(eval "echo $`echo $variableName`")
        f=${f/\[geraldFolder\]/`echo $GERALDPATH`}
        eval "$variableName=\"$f\""

        # [runEmailServer] ->
        #f=$(eval "echo $`echo $variableName`")
        #f=${f/\[runEmailServer\]/`echo 'emailServer'`}
        #eval "$variableName=\"$f\""

        # [runDomain] ->
        #f=$(eval "echo $`echo $variableName`")
        #f=${f/\[runDomain\]/`echo 'emailDomain'`}
        #eval "$variableName=\"$f\""

        # [runEmails] ->
        #f=$(eval "echo $`echo $variableName`")
        #f=${f/\[runEmails\]/`echo 'emails'`}
        #eval "$variableName=\"$f\""

}

function readSettings
{
	SETTINGS_FILE=$1
        FTP_FOLDER=`cat $SETTINGS_FILE 2> /dev/null | grep 'FTP Folder' | cut -d"=" -f2`
        SYSCONF_FOLDER=`cat $SETTINGS_FILE 2> /dev/null | grep 'System Configs Folder' | cut -d"=" -f2`
        TRANSFERRED_FOLDER=`cat $SETTINGS_FILE 2> /dev/null | grep 'Transferred Files Folder' | cut -d"=" -f2`

        FTPCONFIGS_FOLDER=`cat $SETTINGS_FILE 2> /dev/null | grep 'FTP Configs Folder' | cut -d"=" -f2`
        REPORTCONF_FOLDER=`cat $SETTINGS_FILE 2> /dev/null | grep 'Report Configs Folder' | cut -d"=" -f2`
        FTPSYNC_SCRIPT=`cat $SETTINGS_FILE 2> /dev/null | grep 'FTP Synchronization Script' | cut -d"=" -f2`
        SMTP_SERVER=`cat $SETTINGS_FILE 2> /dev/null | grep 'SMTP Server' | cut -d"=" -f2`
        DOMAIN=`cat $SETTINGS_FILE 2> /dev/null | grep 'Your Domain' | cut -d"=" -f2`
        EMAIL_TO=`cat $SETTINGS_FILE 2> /dev/null | grep 'Emails' | cut -d"=" -f2`

        for variable in FTP_FOLDER SYSCONF_FOLDER TRANSFERRED_FOLDER FTPCONFIGS_FOLDER REPORTCONF_FOLDER FTPSYNC_SCRIPT SMTP_SERVER DOMAIN EMAIL_TO
        do
                expandDirectives $variable
        done

        if [[ $GATHER_SCRIPT -eq 1 ]]
        then
		for folder in SYSCONF_FOLDER REPORTCONF_FOLDER
                do
                        checkFolder $(eval "echo $`echo $folder`")
                done
                mkdir -p $FTP_FOLDER 2> /dev/null
		chmod 777 $FTP_FOLDER 2> /dev/null

        else
		# for the sync script
                for folder in SYSCONF_FOLDER FTPCONFIGS_FOLDER
                do
		        checkFolder $(eval "echo $`echo $folder`")
                done
		
		# sync script only: create "Transferred" folder
		mkdir -p "${REPORTCONF_FOLDER}" 2> /dev/null
		chmod a+rw "${REPORTCONF_FOLDER}" 2> /dev/null
		mkdir -p "${TRANSFERRED_FOLDER}" 2> /dev/null
		chmod a+rw "${TRANSFERRED_FOLDER}" 2> /dev/null
		mkdir -p "${FTP_FOLDER}" 2> /dev/null
		chmod a+rw "${FTP_FOLDER}" 2> /dev/null
        fi
}

function log
{
        DATE=`date +"%F %T"`
        
	if [ ! -e "$CSU_LOG" ]
	then
		touch "$CSU_LOG"	
	fi	
	echo "$DATE:$MACHINENAME:$1" >> $CSU_LOG

        # also output to STDOUT if wanted
        if [[ -n "$2" && "$2" = "1" ]]
        then
                echo "$SCRIPT_BASE: $1"
        fi
}

function callFtp
{
	if [ $LFTP -ne 0 ]
	then
		lftp -u $FTPUSERNAME,$FTPPASSWORD $FTPSERVER -e "$1" 
	fi
}

function callMainFtp
{

	if [ $LFTP -ne 0 ]
	then
		lftp -u $DEFAULTFTPUSER,$DEFAULTFTPPASS $DEFAULTFTPSERVER -e "$1"
	fi
}

function getRunEmailSettings
{
	if [ -e Makefile ]
	then
		EMAILSETTINGS=$( cat Makefile | grep -A 4 'RR_PARAMS=' | perl -pe 's/\s\\\n/|/' | cut -d" " -f2- )
		if [ $SMTP_SERVER = "[runEmailServer]" ]
		then
			SMTP_SERVER=`echo $EMAILSETTINGS | cut -d"|" -f1`
		fi

		if [ $DOMAIN = "[runDomain]" ]
		then
			DOMAIN=`echo $EMAILSETTINGS | cut -d"|" -f2`
		fi

		if [ $EMAIL_TO = "[runEmails]" ]
		then
			EMAIL_TO=`echo $EMAILSETTINGS | cut -d"|" -f5-`
		fi
	fi
}

# function getSettings
# {
	# CONF=$1
	# DATAREPOS\_FOLDER=`cat $CONF  | egrep "Data repository all\s*=" | cut -d"=" -f2`
	# # remove trailing slash
	# DATAREPOS\_FOLDER=${DATAREPOS\_FOLDER%/}
# }

function email
{

	if [[ -n $SMTP_SERVER && -n $EMAIL_TO ]]
	then
		if [ $DEBUG -ge 1 ]
		then
			"$THISSCRIPTLOC/sendEmail.pl" "$SMTP_SERVER" "$DOMAIN" "$EMAIL_TO" "$1" "$2" 
		else
			"$THISSCRIPTLOC/sendEmail.pl" "$SMTP_SERVER" "$DOMAIN" "$EMAIL_TO" "$1" "$2" 2>&1 > /dev/null
		fi
	else
		log "email not configured in $3, no email sent" 1
	fi
	
	
}


function sendEmails
{
	getRunEmailSettings $THISSCRIPTLOC/../share/RAMM/$SCRIPT_BASE.conf
	email "$1" "$2" "RAMM"
	# getRunEmailSettings
	# email "$1" "$2" "the run folder"
}


function checkPreRequisites
{
 # check whether lftp is present on this system and abort if not

LFTP=`which lftp | wc -l`

if [[ $LFTP -eq 0 ]]
then
	log "pre-requisites check failed: I could not find the lftp program. You will have to send the zip file manually, it will be placed in ${FTP_FOLDER}" 1
	NOLFTP=1
else
	log "You have lftp. pre-requisites check was successful" 1
fi

}





if [ -e $THISSCRIPTLOC/$SCRIPT_BASE.conf ]
then
	readSettings $THISSCRIPTLOC/$SCRIPT_BASE.conf
else
	readSettings $THISSCRIPTLOC/../share/RAMM/$SCRIPT_BASE.conf
fi

checkPreRequisites

updateFTPSettings

for computer in `find ${FTP_FOLDER} -maxdepth 1 -mindepth 1 -type d -printf "%P\n" | egrep -v "^(SystemConfigs|Transferred|forReview)$"`;
do 
echo "----> Syncing data for $computer"
	FTPSERVER=$DEFAULTFTPSERVER
	FTPUSERNAME=$DEFAULTFTPUSER
	FTPPASSWORD=$DEFAULTFTPPASS
	FTPLOC=$DEFAULTFTPLOC
	
	OTHERSERVER=`cat ${FTPCONFIGS_FOLDER}/${computer}.ftp 2> /dev/null |grep "Server" | cut -d"=" -f2`
	OTHERUSER=`cat ${FTPCONFIGS_FOLDER}/${computer}.ftp 2> /dev/null |grep "Username" | cut -d"=" -f2`
	OTHERPASS=`cat ${FTPCONFIGS_FOLDER}/${computer}.ftp 2> /dev/null |grep "Password" | cut -d"=" -f2`
	OTHERFOLDER=`cat ${FTPCONFIGS_FOLDER}/${computer}.ftp 2> /dev/null |grep "Folder" | cut -d"=" -f2`
	OTHERFOLDER=${OTHERFOLDER%/}
	
	if [[ -n $OTHERSERVER ]]
	then
		FTPSERVER=$OTHERSERVER
	fi
	
	if [[ -n $OTHERUSER ]]
	then
		FTPUSERNAME=$OTHERUSER
	fi
	
	if [[ -n $OTHERPASS ]]
	then
		FTPPASSWORD=$OTHERPASS
	fi
	
	if [[ -n $OTHERFOLDER ]]
	then
		FTPLOC=$OTHERFOLDER
	fi
	

	# check that FTP works
	log "checking if FTP settings are functional" 1
	touch "${computer}_CommoCheck"
	callFtp "put ${computer}_CommoCheck
rm ${computer}_CommoCheck	
quit
" 2>&1 > /dev/null
	if [[ $? -ne 0 ]]
	then
		log "ftp not functional for $computer, notifying Illumina's main ftp server" 1
		echo "ftp credentials failed for $computer
server:$FTPSERVER
username:$FTPUSERNAME
pass:$FTPPASSWORD" > ${SYSCONF_FOLDER}/FAILFTP_$computer
		
		log "falling back to main ftp server settings" 1
		
		FTPSERVER=$DEFAULTFTPSERVER
		FTPUSERNAME=$DEFAULTFTPUSER
		FTPPASSWORD=$DEFAULTFTPPASS
		FTPLOC=$DEFAULTFTPLOC
		callFtp "mirror $VERBOSEONLY -n -R -I FAILFTP* ${SYSCONF_FOLDER}/ ./SystemConfigs
exit
" 2>&1 > /dev/null
		if [[ $? -ne 0 ]]
        	then
			NOCONNECTIVITY=1
		fi

	else 
		echo "settings ok, continuing"
	fi

	rm -f "${computer}_CommoCheck"
	# calls are a bit redundant from computer to computer however we have best chances to hit ftp updates this way	
	echo "Syncing data from FTP Server"
	
	VERBOSEONLY="--verbose=4"

	if [ $DEBUG -ge 2 ]
	then
		VERBOSEONLY="$VERBOSEONLY --dry-run"  # --dry-run causes different output than a real run
	fi	


	callFtp "mirror $VERBOSEONLY -n --delete -I ${computer}_*_DataSetUpdate SystemConfigs/ ${SYSCONF_FOLDER}
mirror $VERBOSEONLY -n --delete -I ${computer}_Exclude SystemConfigs/ ${SYSCONF_FOLDER}
mirror $VERBOSEONLY -n --delete -I All_Exclude SystemConfigs/ ${SYSCONF_FOLDER}
mirror $VERBOSEONLY -n --delete -I All_*_DataSetUpdate SystemConfigs/ ${SYSCONF_FOLDER}
mirror $VERBOSEONLY -n --delete -I ${computer}_FtpUpdate SystemConfigs/ ${SYSCONF_FOLDER}
mirror $VERBOSEONLY -n --delete -I All_FtpUpdate SystemConfigs/ ${SYSCONF_FOLDER}
mirror $VERBOSEONLY -n -R -I ACK* ${SYSCONF_FOLDER}/ ./SystemConfigs
exit
" 2>&1 > /dev/null
done


# get the zip stuff
for computer in `find ${FTP_FOLDER} -maxdepth 1 -mindepth 1 -type d -printf "%P\n" | egrep -v "^(SystemConfigs|Transferred|forReview)$"`;
do
	if [[ ! -e ${SYSCONF_FOLDER}/${computer}_Exclude && ! -e ${SYSCONF_FOLDER}/All_Exclude ]]
	then
		if [[ $NOLFTP -eq 1 || $NOCONNECTIVITY -eq 1 ]] 
		then
			export FTP_FOLDER
			files=$( echo $files `ls ${FTP_FOLDER}/$computer/*.zip` )
			#mv -f ${FTP_FOLDER}/$computer/*.zip "${TRANSFERRED_FOLDER}"
			if [[ $anyFiles -lt 1 ]]
			then
				T=$( cd ${TRANSFERRED_FOLDER} && pwd )
				subject="RAMM is having issues: file(s) to send through ftp for analysis server $MACHINENAME"
				content="RAMM (Remote Automated Mining and Monitoring) diagnostics: Either lftp is not present or there are connectivity issues with your internet connection.
zip files were placed in the ${T} for manual sending through ftp to Illumina."
				anyFiles=1
				warn="Either lftp is not present or there are connectivity issues with your internet connection, some zip files were created and deposited in ${TRANSFERRED_FOLDER} for review, sending notification email (if it works...)"
			fi
		else
			callFtp "mirror $VERBOSEONLY -n -R -I *.zip ${FTP_FOLDER}/$computer $FTPLOC
exit
" 2>&1 > /tmp/ftpOut.txt
			IFS=' 
	'
			
			files=$( echo $files `egrep "\.zip'$" /tmp/ftpOut.txt | egrep "^Transferring" | rev | cut -d" " -f1 | rev | sort | uniq` ) 
			
			if [[ $anyFiles -lt 1 ]]
			then
				anyFiles=`egrep "\.zip'$" /tmp/ftpOut.txt | egrep "^Transferring" | wc -l`
				subject="RAMM FTP activity for $MACHINENAME"
				content="Hi,
 RAMM (Illumina Inc.'s Remote Automated Mining and Monitoring System) has sent one or more files to the ftp server $FTPSERVER;

 If you have any questions about this activity please contact your Illumina Field Application Specialist. Keep in mind that the data collected by RAMM is exclusively run performance data and does not contain any proprietary information relating to your run or installation. This data is collected with the purpose of monitoring Illumina's install base and detect potential problems with your installation; also, over a longer period of time, it will help Illumina Inc. improve its DNA sequencing solutions.

 The list of files follows and are available for review under the Transferred subdirectory of your GERALD Folder or within the folder you specified at the time you installed RAMM (for distributed installations):
"
				warn="some zip files were transferred to Illumina, sending notification email"
			fi
		fi
	
		if [ $DEBUG -ge 2 ]
		then
			echo "<-- this is the content of the ftp output file"
			cat /tmp/ftpOut.txt
			echo "--> "
		fi
		rm /tmp/ftpOut.txt 2> /dev/null

		# transfer the transferred zip files
		for file in $files;
	        do
			f=${file%\'}
			f=${f#\`}
			
			echo "--> moving file $f to ${TRANSFERRED_FOLDER}"
			
			if [ ! -d "${TRANSFERRED_FOLDER}" ]
			then
				mkdir -p "${TRANSFERRED_FOLDER}" 2> /dev/null
				chmod a+rw "${TRANSFERRED_FOLDER}"
			fi
			
			mv -f "${FTP_FOLDER}/$computer/$f" "${TRANSFERRED_FOLDER}"
		done

	else
		log "Illumina turned off FTP access for this computer, no files will be transferred now." 1
	fi
	
done


if [[ $anyFiles -ne 0 ]]
then
	for file in $files;
	do
		fileList="$fileList
$file"
	done

	content="$content
$fileList"

	log "$warn" 1
	sendEmails "$subject" "$content"
	IFS="
"
fi

