#!/bin/bash

. ./bootstrap/common.sh
common_variables
SCRIPT=boost.sh
VERSION="1_35_0"
BOOST_LIB_VERSION="1_35"
BOOST_VERSION="013500"
BOOST_MAJOR=01
BOOST_MINOR=35
SOURCE_TGZ=${OPT_DIR}/boost_${VERSION}.tar.gz
SOURCE_DIR=${OPT_DIR}/boost_${VERSION}
BUILD_DIR=${SOURCE_DIR}.build
LIBRARIES=program_options,filesystem,system,date_time,iostreams,python,regex,serialization,signals

common_options $@

if [[ $CLEAN ]] ; then
    echo removing $SOURCE_DIR
    rm -rf $SOURCE_DIR $BUILD_DIR ${INCLUDE_DIR}/boost ${LIB_DIR}/libboost_*.{a,so}
    exit 0
fi

rm -f bootstrap/boost_version
if [[ ! -e bootstrap/boost_version.cpp ]] ; then
    echo Could not find bootstrap/boost_version.cpp
    exit 1
fi

CPPFLAGS="${CPPFLAGS} -I${INCLUDE_DIR}"
LDFLAGS="${LDFLAGS} -L${LIB_DIR} `echo $LIBRARIES | sed -e 's/,/ -lboost_/g' -e 's/^/-lboost_/'`"
FOUND_VERSION=
g++ $CPPFLAGS -o bootstrap/boost_version bootstrap/boost_version.cpp $LDFLAGS 2> /dev/null \
    && FOUND_VERSION=`bootstrap/boost_version`
if [[ $? == 0 && "$FOUND_VERSION" && ! "$FORCE" ]] ; then
    FOUND_MAJOR=${FOUND_VERSION:0:2}
    FOUND_MINOR=${FOUND_VERSION:2:2}
    if (( "$FOUND_MAJOR" >= "$BOOST_MAJOR" && "$FOUND_MINOR" >= "$BOOST_MINOR" )) ; then
        echo boost version \"$VERSION\" is already installed
        echo nothing to be done
        exit 0
    fi
fi

common_create_source

cd ${SOURCE_DIR} \
    && ./configure --prefix=${OPT_DIR} --with-libraries=${LIBRARIES} \
    && make -j 8 install BJAM_CONFIG='--layout=system release link=static threading=single'

if [ $? != 0 ] ; then echo "$SCRIPT: build failed: Terminating..." >&2 ; exit 1 ; fi

echo "Ckeaning up ${SOURCE_DIR}"
rm -rf ${SOURCE_DIR}

echo boost-$VERSION installed successfully
