#!/bin/bash

function usage () {
    echo Usage: ./bootstrap.sh [option] [targets]
    echo option must be one of:
    echo '    ' -h, --help '      ' This small usage guide
    echo '    ' -f, --force '     ' Build the targets unconditionally
    echo '    ' -c, --clean '     ' Clean the targets
    echo targets:
    echo '    ' cmake
    echo '    ' boost
}

TEMP=`getopt -n 'bootstrap.sh' -o hfc --long help,force,clean -- "$@"`

if [ $? != 0 ] ; then usage ; echo "Terminating..." >&2 ; exit 1 ; fi

eval set -- "$TEMP"

OPTIONS=
COUNT=0

while true ; do
    case "$1" in
        -h|--help)  usage ; exit 0           ; shift ;;
        -f|--force) OPTIONS=-f ; (( ++COUNT )) ; shift ;;
        -c|--clean) OPTIONS=-c ; (( ++COUNT )) ; shift ;;
        --)                                    shift ; break ;;
        *) echo "Internal error!" ; exit 1 ;;
    esac
done

if [[ $COUNT > 1 ]] ; then 
    echo You can''t specify several options
    exit 2
fi

if [[ ! -d bootstrap ]] ; then
    echo Could not find the bootstrap directory
    exit 1
fi

if [[ $# > 0 ]] ; then
    command_list="$@"
else 
    if [[ ! -e bootstrap/default.txt ]] ; then
        echo Could not find bootstrap/default.txt
        exit 3
    fi
    command_list="`cat bootstrap/default.txt`"
fi

script_list=
for command in $command_list ; do
    script_name=bootstrap/$command.sh
    if [[ ! -e $script_name ]] ; then
        echo Could not find $script_name
        usage
        exit 3
    fi
    script_list="$script_list $script_name"
done

for script in $script_list ; do
    /bin/bash $script $OPTIONS
done
