#!/usr/bin/env perl

# PROJECT: GERALD
# MODULE:  buildExtendedAlign.pl
# AUTHOR:  A. J. Cox
#
# Copyright (c) 2007 Solexa
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# buildExtendedAlign.pl: combine the output of ELAND --multi and the output
# of unsquash into a single file. main task is to adjust the position
# of reverse complement matches

use warnings;
use strict;

use FindBin;
use lib "$FindBin::Bin/../lib";
use Gerald::Common;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: buildExtendedAlign.pl,v 1.2 2008/07/08 15:12:10 craczy Exp $';
    my $cvs_version='$Revision: 1.2 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope

my ($readID, $read, $nhood, $pos, $align);
my @s;

my $matchPos;
my $matchStrand;
my $matchGenome;
my $outString;
my $delimiter;
my $numLeadingNs;

my $reverseStrandStartOffset;


if (@ARGV==0)
{
    $reverseStrandStartOffset=0;
} # if
elsif (@ARGV==2)
{
    $reverseStrandStartOffset=$ARGV[0]-$ARGV[1];
} # elsid
else
{
    die "Usage: paste eland_multi.txt unsquash_output | $0 [read_length ELAND_read_length]\n";
} # else


while (<STDIN>)
{
#    print ">>$_";
    chomp;
    ($readID, $read, $nhood, $pos, $align)=split("\t",$_);
    if ($pos eq "")
    {
	print "$readID\t$read\t$nhood\t-\n";
	next;
    } # if
    
    $numLeadingNs=0;
    $numLeadingNs=length($&) if ($read=~/^[Nn]*/);
    
    @s=split(":",$align);

    my $i=0;
    $outString="";

    while ($pos=~/(.+?)([:,]|$)/g)
    {
	$matchPos=$1;
	$delimiter=$2;
	if ($delimiter eq ':')
	{
	    $outString.="${matchPos}:";
	} # if
	elsif ($matchPos=~/(\d+)([FR])\d/)
	{
	    $matchPos=$1;
	    $matchStrand=$2;
	    if ($matchStrand eq 'R')
	    {
		$matchPos+=$numLeadingNs-$reverseStrandStartOffset;
	    } 
	    else
	    {
		$matchPos-=$numLeadingNs;
	    }
	    
	    $outString.="${matchPos}${matchStrand}$s[$i]${delimiter}";
	    $i++;
	} # elsif
	else 
	{
	    die "Problem parsing field $matchPos in $pos\n";
	} # else
    } # while

#    while ($pos=~/(.+?)([:,]|$)/g)
#    {
#	$matchPos=$1;
#	$delimiter=$2;

#	if ($matchPos=~/(\d+)([FR])\d/)
#	{
#	    $matchPos=$1;
#	    $matchStrand=$2;
#	    $matchPos-=$reverseStrandStartOffset if ($matchStrand eq 'R');
#	    $outString.="${matchPos}${matchStrand}$s[$i]${delimiter}";
#	    $i++;
#	} # if			
#	else 
#	{
#	    die "Problem parsing field $matchPos in $pos\n"
#	        unless ($delimiter eq ':');
#
#	    $outString.="${matchPos}:";
#
#
#	} # else

#    } # while
    print "$readID\t$read\t$nhood\t$outString\n";

} # while
       
	   
