#!/usr/bin/env perl

use warnings;
use strict;
use File::Basename;

# File: convertFromELAND.pl
# Copyright (c) Solexa Limited 2006
# Author: A. J. Cox
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Simple script to convert output of ELAND to output
# as produced by phageAlign

# Usage: 
# cat impala_output.txt | ./convertFromELAND.pl [fileName] 

# Optional parameter fileName prints only those lines from
# with sequence name >fileName - compatible with queries generated
# from Solexa-style sequence files by convertToFasta.pl

use FindBin;
use lib "$FindBin::Bin/../lib";
use Gerald::Common;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: convertFromELAND.pl,v 1.2 2008/07/08 15:12:10 craczy Exp $';
    my $cvs_version='$Revision: 1.2 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope


my @F;
my @s;
my $best;
my $nextBest=0;
my $pos;
my $matched;
my $i;
my $error;
my $score0;    
my $score1;    
my $score2;    
my $score3;    
my $fileName=undef;
my $readLength=undef;
my $readLengthELAND=32; # default value
my $found=0;
my $numNs;
my $scoreMatch=500;
my $scoreMismatch=-547;
my $pref;
my $lane=-1;
my $tile=-1;
my $laneWanted=-2;
my $tileWanted=-2;
my $numLeadingNs;

my $suffix=undef;
my $dir='.';
my $fh=undef;

#------------------------------------------------------------------------------

sub make_out_file_name($$$$)
{
    my ($dir, $lane, $tile, $suffix) = @_;

    my $file_name = sprintf("%s/s_%1d_%04d%s", $dir, $lane, $tile, $suffix);
    return $file_name;
}

#------------------------------------------------------------------------------

# sub changeToNextTile
# Open a series of file handles to send output to, or just send to stdout
sub changeToNextTile
{
    die unless (@_==2);
    my ($lane, $tile)=@_;
    print STDERR "Change to lane $lane tile $tile\n";
    if (defined($suffix))
    {
	close ($fh) if (defined($fh));
	$fh=undef;
	$fileName = make_out_file_name($dir, $lane, $tile, $suffix);
	print STDERR "About to open file $fileName\n";
	open ($fh, ">$fileName") 
	    || die "Failed to open file $fileName for writing: $0";
    } # if
    else
    {
	$fh=*STDOUT;
    } # else
} # sub changeToNextTile

#------------------------------------------------------------------------------

# FIXME : This should be just a default, overridable on the command line.
my $tile_file_path = './tiles.txt';

my %tile_freqs;

if (@ARGV>=1)
{
    $fileName=shift(@ARGV);
    $fileName=basename($fileName);
    @s=split("_",$fileName);

    if ($fileName=~/^s_(\d+)_(\d+)_/)
    {
	print STDERR "Will generate file $fileName\n";
	$laneWanted=$1;
	$tileWanted=$2;
    } # if
    else
    {
	print STDERR "Will generate files of form s_L_TTTT$fileName\n";
	$suffix=$fileName;
    } # else

    if (@ARGV>=1)
    {
	$readLength=shift(@ARGV);
	if (@ARGV>=1)
	{
	    $readLengthELAND=shift(@ARGV);
	} #if
	elsif ($readLength<$readLengthELAND)
	{
	    $readLengthELAND=$readLength;
	} # elsif
    } #if
    else
    {
	$readLength=$readLengthELAND;
    }
} #if
else
{
    print STDERR << "END";
Usage: paste eland_output.txt unsquash_output.txt | $0 
[ fileName|suffix [readLength [ readLengthELAND ] ] ]
END
    exit (0);
	
} # else


while (<STDIN>)
{
    chomp;
    @F=split("\t",$_);
    @s=split(":",$F[0]);

    ++$tile_freqs{$s[-3]};

    if (($lane ne $s[-4])||($tile ne $s[-3]))
    {
	last if (($lane==$laneWanted)
		 &&($tile==$tileWanted));
	$lane=$s[-4];
	$tile=$s[-3];
	changeToNextTile($lane, $tile);
    } # if

    next if (($laneWanted>0)
	     &&(($lane!=$laneWanted)||($tile!=$tileWanted)));

#    if ((defined($lane))&&(($lane!=$s[1])||($tile!=$s[2])))
#    {
#	last if ($found==1);
#	next;
#    } # if

    $numNs=0;

    if (($F[2] eq "QC")||($F[2] eq "NM"))
    {
	$F[1]=~s/N/\./g;
	$F[1]=substr($F[1],0,$readLength) if (defined($readLength));
	print $fh "$F[1] 0 0\n"; next;
    } # if

    while ($F[1]=~s/[Nn]/\./) { $numNs++;}

    if (defined($readLength))
    {
	$score0=$scoreMatch*($readLength-$numNs);
    } # if
    else
    {
	$score0=$scoreMatch*(length($F[1])-$numNs);
    } # else

    $score1=$score0-$scoreMatch+$scoreMismatch;
    $score2=$score1-$scoreMatch+$scoreMismatch;
    $score3=$score2-$scoreMatch+$scoreMismatch;

    if ($F[2]=~/R[012M]/)
    {
	$F[1]=substr($F[1],0,$readLength) if (defined($readLength));
	if ($F[2] eq "RM")
	{	
	    print $fh "$F[1] $score0 256\n";
	}
	if ($F[2] eq "R0")
	{
	    print $fh "$F[1] $score0 $F[3]\n";
	} # elsif
	elsif ($F[2] eq "R1")
	{
	    print $fh "$F[1] $score1 $F[4]\n";
	} # elsif
	elsif ($F[2] eq "R2")
	{
	    print $fh "$F[1] $score2 $F[5]\n";
	} # elsif
	next;
    } # if

    if ($F[2] eq "U0")
    { 
	$best=$score0; 
	if ($F[4]!=0) { $nextBest=$score1; }
	elsif ($F[5]!=0) { $nextBest=$score2; }
	else { $nextBest=$score3; }
    } # if
    elsif ($F[2] eq "U1")
    { 
	$best=$score1; 
	$nextBest=(($F[5]!=0)?$score2:$score3);
    } # elsif 
    elsif ($F[2] eq "U2")
    { 
	$best=$score2; $nextBest=$score3; 
    } # elsif

    $F[6]=~s/\.fa$//;

    if ($F[1]=~/^\.*/)
    {
	$numLeadingNs=length($&);
	if ($F[8] eq 'F')
	{
	    $F[7]-= $numLeadingNs;
	} # if
	else
	{
	    $F[7]+= $numLeadingNs;
	} # else

    } # if



    
    $pos="$F[6]:$F[7]";


    if (defined($readLength))
    {
	$F[1]=substr($F[1],0,$readLength);
    } # if

    $matched=$F[1];

#    if ($F[8] ne 'R')
#    {
#	$matched=~s/^[\.]+//; 
#    } # if
#    elsif ($matched=~s/([\.]+)$//)
#    {
#	$matched=$1.$matched;
#    } # elsif   


#    $matched=~s/[\.]+$// if ($F[8] eq 'F');

    my $thisChar;

    $best=0;
    for ($i=0;$i<length($matched);$i++)
    {
	$thisChar=uc(substr($matched,$i,1));
	if ($thisChar=~/[ACGT]/)
	{
	    if ($thisChar eq uc(substr($F[-1],$i,1)))
	    {
		$best+=$scoreMatch;
	    } # if
	    else
	    {
		$best+=$scoreMismatch;
	    } # else
	} # if   
    } # for

    $matched=$F[-1];
    if ($F[8] eq "R")
    {
	$matched=reverse($matched);
	$matched=~tr/acgtACGT/tgcaTGCA/;
	
	
	if ($pos=~s/^(.*:)//)
	{
	    $pref=$1;
	} # if
	else
	{
	    $pref=undef;
	} # else
	$pos-=($readLength-$readLengthELAND);
	$pos="${pref}${pos}" if (defined($pref));
    } # if
    
    print $fh "$F[1] $best 1 $pos $F[8] $matched $nextBest\n";
} # while


# If a suffix rather than a specific filename was specified (i.e. files 
# are written for multiple tiles), write an empty file for each of any 
# tiles that were expected but not represented in the input.
# This prevents some downstream unhappiness.
#
# Warning : This section assumes that only one lane is actually relevant.
#           The above code could handle tiles from multiple lanes in its
#           input but is used currently lane-specifically.

if (defined($suffix)) {
    my @missing_tile_nums;

    if (!determine_missing_tile_nums($tile_file_path, \%tile_freqs, $lane,
				     \@missing_tile_nums)) {
	die("Failed to determine_missing_tile_nums");
    }

    foreach my $missing_tile_num (@missing_tile_nums) {
	changeToNextTile($lane, $missing_tile_num);
    }
}

#------------------------------------------------------------------------------
