#!/usr/bin/env perl

use warnings;
use strict;
use File::Basename;

# File: convertFromSAF.pl
# Copyright (c) Solexa Limited 2007
# Author: A. J. Cox
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Simple script to convert SAF alignment format files to output
# as produced by phageAlign. Note the phageAlign file format will eventually
# be phased out.

# Usage: 
# join s_N_eland_extended.txt s_N_saf.txt | ./convertFromSAF.txt _suffix.txt [dir] 

# Output will be placed in files s_L_TTTT_suffix.txt in directory dir 
# L=lane, TTTT=tile

my $version;
my $id;
{
    my $cvs_id='@(#) $Id:';
    my $cvs_version='$Revision: 1.1 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope


my @s;
my $matched;
my $i;
my $score0;    
my $fileName=undef;
my $readLength=undef;
my $scoreMatch=500;
my $scoreMismatch=-547;
my $lane =-1;
my $tile =-1;



push @ARGV,"." if (@ARGV==1); 

die "usage: join s_N_eland_extended.txt s_N_saf.txt | $0 _suffix.txt [dir]"
    unless ((@ARGV==0)||(@ARGV==2));

my ($suffix, $dir)=@ARGV;
my $fh=undef;


# sub changeToNextTile
# Open a series of file handles to send output to, or just send to stdout
sub changeToNextTile
{
    die unless (@_==2);
    my ($lane, $tile)=@_;

    if (defined($suffix))
    {
	close ($fh) if (defined($fh));
	$fh=undef;
	$fileName=sprintf("%s/s_%1d_%04d%s",$dir,$lane,$tile,$suffix);
	print STDERR "About to open file $fileName\n";
	open ($fh, ">$fileName") 
	    || die "Failed to open file $fileName for writing: $0";
    } # if
    else
    {
	$fh=*STDOUT;
    } # else
} # sub changeToNextTile

my ($readID, $read, $nhood, $aligns, $chrom, $pos, $strand, $type);
# remaining fields are single-alignment score (single reads)
# plus paired-alignment score, pair strand and insert size for pairs
# Non-nominal possibilities are:
# readID read nhood - NM (repeat)
# readID read QC/NM - NM (QC failure or NM)

my $nbor;
my $p;
my $nextBestScore=0; # not implemented as never seems to get used

while (<STDIN>)
{
    chomp;
#    print "DOING $_\n";
    ($readID, $read, $nhood, $aligns, $chrom, $pos, $strand, $type) = split;

    @s=split(':',$readID); # should be folder:lane:tile:x:y[ab]
    if (($lane ne $s[-4])||($tile ne $s[-3]))
    {
	$lane=$s[-4];
	$tile=$s[-3];
	changeToNextTile($lane, $tile);
    } # if

    $read=~s/N/\./g;

    $score0=$scoreMatch*length($read); #*(length($read)-$numNs);

    if ($chrom eq 'NM')
    { 
	if ($nhood eq 'RM')
	{ 
	    print $fh "$read $score0 256\n";
	} # if   
	elsif ($nhood!~/:/)
	{ 
	    print $fh "$read $score0 0\n";
	} # elsif
	else
	{ 
	    @s=split(':',$nhood);
	    while (defined($nbor=shift(@s)))
	    {
		last if ($nbor>=0);
		$score0-=$scoreMatch;
		$score0+=$scoreMismatch;
	    } # while
	    print $fh "ZZ $_" if ($nbor==1);
	    print $fh "$read $score0 $nbor\n";
	} # else
	next;
    } # if

    $i=0;
    $matched=$read;

    while ($type=~/(\d*)([ACGTN])/g)
    { 
	$i+=$1 if ($1 ne "");
	substr($matched,$i,1)=$2; # TBD
	$score0-=$scoreMatch;
	$score0+=$scoreMismatch unless (substr($read,$i,1) eq '.');
	$i++;
    } # while
    if ($strand eq 'R')
    {
	$matched=reverse($matched);
	$matched=~tr/ACGTacgt/TGCAtgca/;
    } # if

    print $fh "$read $score0 1 $chrom:$pos $strand $matched $nextBestScore\n";

} # while
