#!/usr/bin/env perl

# PROJECT: GERALD
# MODULE:  convertQuality.pl
# AUTHOR:  A. J. Cox
#
# Copyright (c) 2006 Solexa
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Takes a prb file and converts each line to a quality string
# Conversion factor is chr=ASCII(64+QV) (NB that's 64 decimal)
# so smallest possible QV of -5 gets called ASCII 59=';' 
# QV = 0  gets ASCII 64 = '@'
# QV = 30 gets ASCII 94 = '^'

use warnings;
use strict;
use File::Basename;

use FindBin;
use lib "$FindBin::Bin/../lib";
use Gerald::Common;


my $version;
my $id;
{
    my $cvs_id='@(#) $Id: convertQuality.pl,v 1.2 2008/07/08 15:12:10 craczy Exp $';
    my $cvs_version='$Revision: 1.2 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope


my $usage=
"Usage: $0 [--useBases useBases]  [-read[1|2]] [--numeric|--symbolic] readLength prbFile[s]";


my $useBases=undef;
my $readPairMode=0;
# $readPairMode==0 (default) denotes no pairing - treat 'Y' and 'y' the same
# $readPairMode==1 denotes pair1 - ignore 'y' bases
# $readPairMode==2 denotes pair2 - ignore 'Y' bases
my $isNumeric=0;

die $usage unless (@ARGV>=2);

if ($ARGV[0] eq '--useBases')
{
    shift (@ARGV);
    die $usage unless (@ARGV>=3);
    $useBases=shift(@ARGV); 
} # if

if ($ARGV[0]=~/^\-\-read([012])$/)
{
    $readPairMode=$1;
    shift (@ARGV);
} # if


if (($ARGV[0] eq '--numeric')||($ARGV[0] eq '--symbolic'))
{
    $isNumeric=1 if ($ARGV[0] eq '--numeric');
    shift (@ARGV);
}




die $usage unless (@ARGV>=2);

my $readLength=shift(@ARGV);


my @baseIndex;

unless (defined($useBases))
{
    $useBases='all';
}

my $i;
my $j;
my $qv;

my $qstring;


sub max
{
    my $max=$_[0]->[0];
    for (@{$_[0]})
    {
        $max=$_ if ($_>$max);
    }
    return $max;
} # sub max


@baseIndex=parseUseBases($useBases, $readLength, $readPairMode);


#print scalar(@baseIndex), " $baseIndex[0] $baseIndex[-1]\n"; die;
my @s1;
my @s3;

for my $file (@ARGV)
{

    open (LINE, $file) 
	|| die "Failed to open $file: $!";

    while (<LINE>)
    { 
	$qstring='';
	@s1=split('\t',$_);
	for ($i=0;$i<scalar(@baseIndex);$i++)
	{
#            print "$i $baseIndex[$i] $s1[$baseIndex[$i]]\n";
	    @s3=split(' ',$s1[$baseIndex[$i]]);
	    unless (@s3==4)
	    {
		print STDERR basename($0), ": Error parsing line: $_";
		print STDERR basename($0),
		": can't find quality values for cycle ",
		$baseIndex[$i]+1, " in file ${file} "; 
		if (defined($s1[$baseIndex[$i]]))
		{
		    print STDERR
			"- could not parse field $s1[$baseIndex[$i]]\n";
		} # if
		else
		{
		    print STDERR 
			"- only ",scalar(@s1), 
			" cycles of quality values present on line\n";
		} # else			
		exit (-1);
	    }
	    $qv=max(\@s3);
	    $qstring.=(($isNumeric)?" $qv":chr(int(64+$qv)));
	} # for i
	$qstring=~s/^\ //;
	printf "%s\n", $qstring;
    } # while  
} # for file
