#!/usr/bin/env perl

# PROJECT: GERALD
# MODULE:  picassoBestAlignment.pl
# AUTHOR:  A. J. Cox

# This compares ELAND alignments of reads vs
# i. whole genome 
# ii. known ultra-abundant sequences 
# (mainly ribosomal and mtichondrial sequences, may want to add 
# haemoglobin for blood)
# iii. Curated set of known splice variants

# Method of first version based on prototyping work by Irina Khrebtukova
# Steps:
# 1. If aligns to abundant sequence then flag as 'RM' for 'repeat masked'
# 2. If aligns uniquely to genome, use that
# 3. If aligns to splice variants, use that

#
# Copyright (c) 2006 Solexa; 2007, 2008 Illumina
#
# This source file is covered by the "Illumina Genome Analyzer
# Software License Agreement" and bound by the terms therein.


use warnings; 
use strict;
use Cwd;
use XML::Simple;
use File::Basename;
use File::Spec;
use Data::Dumper;

use FindBin;
use lib "$FindBin::Bin/../";
use Gerald::Common;


my $version;
my $id;
{
    my $cvs_id='@(#) $Id: pickBestAlignmentRNA.pl,v 1.2 2008/07/22 15:17:21 tc Exp $';
    my $cvs_version='$Revision: 1.2 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
#    $id=~s/\$//g;
} #scope


die "Usage: $0 genomicAlign spliceAlign contamAlign qualityValues"
    unless (@ARGV==4);


my ($lineGenome, $lineSplice, $lineContam);

open (GENOME, $ARGV[0]) || die "Failed to open $ARGV[0]: $!";
open (SPLICE, $ARGV[1]) || die "Failed to open $ARGV[1]: $!";
open (CONTAM, $ARGV[2]) || die "Failed to open $ARGV[2]: $!";

my $i;
my (@FG);
my (@FS);
my (@FC);

# sub printAlignment($$) 
# print a line of alignment info in SAF format - hard coded score of 30 for now
sub printAlignment($$)
{
    my ($readID, $matchInfo)=@_;
    my ($chrom, $pos, $strand, $type)=getMatchDetails($matchInfo,"");
    print "$readID\t$chrom\t$pos\t$strand\t$type\t30\n";
} # sub print Alignment

while (defined($lineGenome=<GENOME>))
{
    $lineSplice=<SPLICE>;
    $lineContam=<CONTAM>;
    chomp($lineGenome);
    chomp($lineSplice);
    chomp($lineContam);
#    print " ----> $lineGenome $lineSplice $lineContam\n";
    @FC=split("\t",$lineContam);
    if ($FC[2] eq 'QC')
    { # too many Ns to do alignment
	print "$FC[0]\tQC\n"
    } # if
    elsif ($FC[2] ne 'NM')
    { # aligned to a contaminant
	print "$FC[0]\tRM\n";
    } # elsif
    else
    {
	@FG=split("\t",$lineGenome);
	if ($FG[2] ne 'NM')
	{
	    if (($FG[3]!~/,/)&&($FG[3] ne '-')&&($FG[3]!~/N$/))
	    { # hooray, it's a unique alignment
		printAlignment( $FG[0],$FG[3] );
	    } # if
	    else
	    {
		print "$FG[0]\tRM\n";
	    } # else
	}
	else
	{ # finally try the splice alignment
	    @FS=split("\t",$lineSplice);
	    if ($FS[2] ne 'NM')
	    {
		if (($FS[3]!~/,/)&&($FS[3] ne '-')&&($FS[3]!~/N$/))
		{ # hooray, it's a unique alignment
		    printAlignment( $FS[0], $FS[3]);
		} # if
		else
		{
		    print "$FS[0]\tRM\n";
		} # else
	    } # if
	    else
	    {
		print "$FS[0]\tNM\n";
	    } # else
	} # else

    } # else

#    die if ($i++==100);
} # while defined

close (GENOME);
close (SPLICE);
close (CONTAM);
