#!/usr/bin/env perl

# File: plotIntensity.pl
# Copyright (c) 2005, 2006 Solexa
# Author: A. J. Cox, L. J. Davies
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Description:
# Produces an example intensity by cycle plot from an intensity 

use warnings;
use strict;

use FindBin;
use lib "$FindBin::Bin/../lib";
use Gerald::Common;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: plotIntensity_monotemplates.pl,v 1.3 2008/07/22 11:20:20 mzerara Exp $';
    my $cvs_version='$Revision: 1.3 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
    $id=~s/\$//g;
} #scope


my $tmpName;
my $tmpNameData;
my ($i,$j,$k,$outName,$displayName,$total,$title,$deblock,$tag,$clusterNum);
my (@array,@intensities,@bases,%templates,%lanes);
my $lane;

my $max = 0;
my $setmax = 0;



my $width=3;
my ($file,$directory);
my @s;
my @total;
my $count;


sub draw_graphs {
    my ($lane,$tmpName,$outName,$title,$clusterNum) = @_;

    my $offset = $max - ($max / 20);
    print STDERR "offset = $offset\n";

    open (OUT, ">$tmpName.txt") 
	|| die "Problem opening $tmpName.txt for writing: $!";

    print OUT << "END";
#    set terminal png color;
set terminal postscript portrait color solid;
#    set nokey;
set yrange [0:$max];
set ytics 500;
  
set size 1.0, 1.0;
set origin 0.0, 0.0;
#    set multiplot;
#    set size 1.0, 0.5;
#    set origin 0.0, 0.5
set title "$title";
set xlabel "Cycle Number";
set ylabel "Intensity";
set label "CLUSTERS = $clusterNum" at 1,$offset,0
plot '$tmpName.dat' using 1:2 title 'A' with linespoints lt 9 lw $width pt 7 ps 2, '$tmpName.dat' using 1:3 title 'C' with linespoints lt 1 lw $width pt 7 ps 2,    '$tmpName.dat' using 1:4 title 'G' with linespoints lt 3 lw $width pt 7 ps 2, '$tmpName.dat' using 1:5 title 'T' with linespoints lt 2 lw $width pt 7 ps 2;  
  
#    set origin 0.0, 0.0;
#    set title "Occurrence of blank bases by cycle ";
#    set xlabel "Cycle Number";
#    plot '$tmpName.dat' using 1:3 title 'blank rate' with impulses lt 3 lw 24;
#    set nomultiplot;

END
    close (OUT);
    gnuplotImage($tmpName.".txt", $outName);
    unlink "$tmpName.dat", "$tmpName.txt";
}



sub draw_graphs_percentage5 {
    my ($lane,$tmpName,$outName,$title,$clusterNum) = @_;
    open (OUT, ">$tmpName.txt") 
	|| die "Problem opening $tmpName.txt for writing: $!";

    print OUT << "END";
#    set terminal png color;
set terminal postscript portrait color solid;
#    set nokey;
set yrange [0:100];
set ytics 10;
  
set size 1.0, 1.0;
set origin 0.0, 0.0;
#    set multiplot;
#    set size 1.0, 0.5;
#    set origin 0.0, 0.5
set title "$title";
set xlabel "Cycle Number";
set ylabel "% Error Rate";
set label "CLUSTERS = $clusterNum" at 1,95,0
plot '$tmpName.dat' using 1:2 title 'A' with linespoints lt 9 lw $width pt 7 ps 2, '$tmpName.dat' using 1:3 title 'C' with linespoints lt 1 lw $width pt 7 ps 2,    '$tmpName.dat' using 1:4 title 'G' with linespoints lt 3 lw $width pt 7 ps 2, '$tmpName.dat' using 1:5 title 'T' with linespoints lt 2 lw $width pt 7 ps 2,'$tmpName.dat' using 1:6 title '.' with linespoints lt 7 lw $width pt 7 ps 2;    
#    set origin 0.0, 0.0;
#    set title "Occurrence of blank bases by cycle ";
#    set xlabel "Cycle Number";
#    plot '$tmpName.dat' using 1:3 title 'blank rate' with impulses lt 3 lw 24;
#    set nomultiplot;

END
    close (OUT);
    gnuplotImage($tmpName.".txt", $outName);
    unlink "$tmpName.dat", "$tmpName.txt";
}




sub draw_graphs_impulses {
    my ($lane,$tmpName,$outName,$title,$clusterNum) = @_;
    open (OUT, ">$tmpName.txt") 
	|| die "Problem opening $tmpName.txt for writing: $!";

    print OUT << "END";
#    set terminal png color;
set terminal postscript portrait color solid;
#    set nokey;
set yrange [0:100];
set ytics 10;
 
set size 1.0, 1.0;
set origin 0.0, 0.0;
#    set multiplot;
#    set size 1.0, 0.5;
#    set origin 0.0, 0.5
set title "$title";
 set key left bottom below
set xlabel "Cycle Number";
set ylabel "Intensity";
  set label "CLUSTERS = $clusterNum" at 0,-5,0
plot '$tmpName.dat' using 1:5 title '3 or less errors' with impulses lt 2 lw 24,'$tmpName.dat' using 1:4 title '2 or less errors' with impulses lt 3 lw 24,'$tmpName.dat' using 1:3 title '1 error or less' with impulses lt 1 lw 24,'$tmpName.dat' using 1:2 title '0 errors' with impulses lt 9 lw 24;    
show label
END
    close (OUT);
    gnuplotImage($tmpName.".txt", $outName);
    unlink "$tmpName.dat", "$tmpName.txt";
}

die "Usage: $0 Monotemplate.txt Directory_Name\n"
    unless (@ARGV==2);

$file = shift(@ARGV);
$directory = shift(@ARGV);

$directory.='/' unless ($directory=~/\/$/);

open (IN, $file) 
    || die "Problem opening $file: $!";

while (<IN>)
{
  
    next if($_ =~ m/\#/);
    push (@intensities,[split('\t',$_)]);

}
close(IN);


for($i=0;$i<scalar(@intensities);$i++){
    $templates{$intensities[$i][2]}++;
    $lanes{$intensities[$i][0]}++;
}


### Going to start a loop for plotting graphs now


### Going to estimate highest value

$max = 0;

for($j=0;$j<@intensities;$j++){
   for($i=4;$i<25;$i++){
	if($intensities[$j][$i] > $max){
	    $max = $intensities[$j][$i];
	}
    }
}

$max = $max * 1.25;


foreach $lane (keys %lanes){


### Going to do CALL 

    foreach $tag (keys %templates){
  
	$outName = $directory."Plots/"."s_".$lane."_".$tag."_call.png";
	$title = "s_".$lane."_".$tag."_call.png";
	print "$outName\n";

	$tmpName="/tmp/tmp-plotIntensity-$>-$$";
	open (OUT, ">$tmpName.dat")
	    || die "Problem opening $tmpName.dat for writing: $!";

	for($j=0;$j<scalar(@intensities);$j++){

	    next if($intensities[$j][2] !~ m/$tag/);
	    next if($intensities[$j][0] !~ m/$lane/);
	    print  OUT "$intensities[$j][1]\t";
	    $clusterNum = $intensities[$j][3];
	    for($k=0;$k<4;$k++){
		print  OUT "$intensities[$j][$k+8]\t";
	    }
	    print OUT "\n";
	}
	close (OUT);
 
	draw_graphs($lane,$tmpName,$outName,$title,$clusterNum);
    }

    foreach $tag (keys %templates){

	$outName = $directory."Plots/"."s_".$lane."_".$tag."_all.png";
	$title = "s_".$lane."_".$tag."_all.png";
	print "$outName\n";

	$tmpName="/tmp/tmp-plotIntensity-$>-$$";
	open (OUT, ">$tmpName.dat")
	    || die "Problem opening $tmpName.dat for writing: $!";
    
	for($j=0;$j<scalar(@intensities);$j++){
	    next if($intensities[$j][2] !~ m/$tag/);		
	    next if($intensities[$j][0] !~ m/$lane/);

	    print  OUT "$intensities[$j][1]\t";

	    for($k=0;$k<4;$k++){
		print  OUT "$intensities[$j][$k+4]\t"; 
	    }
	    print OUT "\n";
	    $clusterNum = $intensities[$j][3];
	}
	close (OUT);

	draw_graphs($lane,$tmpName,$outName,$title,$clusterNum);
    }


    foreach $tag (keys %templates){

	$outName = $directory."Plots/"."s_".$lane."_".$tag."_error.png";
	$title = "s_".$lane."_".$tag."_errors.png";
	print "$outName\n";

	$tmpName="/tmp/tmp-plotIntensity-$>-$$";
	open (OUT, ">$tmpName.dat")
	    || die "Problem opening $tmpName.dat for writing: $!";

	for($j=0;$j<scalar(@intensities);$j++){
	    next if($intensities[$j][2] !~ m/$tag/);
	    next if($intensities[$j][0] !~ m/$lane/);

	    print  OUT "$intensities[$j][1]\t";
	    for($k=0;$k<5;$k++){
		print  OUT "$intensities[$j][$k+12]\t"; 
	    }
	    print OUT "\n";

	    $clusterNum = $intensities[$j][3];
	}
	close (OUT);

	draw_graphs_percentage5($lane,$tmpName,$outName,$title,$clusterNum);
    }

    foreach $tag (keys %templates){

	$outName = $directory."Plots/"."s_".$lane."_".$tag."_perfect.png";
	$title = "s_".$lane."_".$tag."_perfect.png";
	print "$outName\n";

	$tmpName="/tmp/tmp-plotIntensity-$>-$$";
	open (OUT, ">$tmpName.dat")
	    || die "Problem opening $tmpName.dat for writing: $!";

	for($j=0;$j<scalar(@intensities);$j++){
	    next if($intensities[$j][2] !~ m/$tag/);
	    next if($intensities[$j][0] !~ m/$lane/);

	    print  OUT "$intensities[$j][1]\t";
	    print  OUT "$intensities[$j][17]\t"; 
	    print  OUT "$intensities[$j][18]\t"; 
	    print  OUT "$intensities[$j][19]\t"; 
	    print  OUT "$intensities[$j][20]\t"; 
	    print OUT "\n";
	    $clusterNum = $intensities[$j][3];
	}
	close (OUT);

	draw_graphs_impulses($lane,$tmpName,$outName,$title,$clusterNum);
    }

    foreach $tag (keys %templates){
	$deblock = 0;
	$outName = $directory."Plots/"."s_".$lane."_".$tag."_deblock.png";
	$title = "s_".$lane."_".$tag."_deblock.png";
	print "$outName\n";

	$tmpName="/tmp/tmp-plotIntensity-$>-$$";
	open (OUT, ">$tmpName.dat")
	    || die "Problem opening $tmpName.dat for writing: $!";
    
	for($j=0;$j<scalar(@intensities);$j++){
	    next if($intensities[$j][2] !~ m/$tag/);
	    next if($intensities[$j][0] !~ m/$lane/);

	    if(($intensities[$j][21] == 0) && ($intensities[$j][22] == 0) && ($intensities[$j][23] == 0) && ($intensities[$j][24] == 0)){
		next;
	    }
	    $deblock++;
	    print  OUT "$intensities[$j][1]\t";
	    for($k=0;$k<4;$k++){
		print  OUT "$intensities[$j][$k+21]\t";
	    }
	    print OUT "\n";
	    $clusterNum = $intensities[$j][3];
	}
	close (OUT);

	if($deblock > 0){
	    draw_graphs($lane,$tmpName,$outName,$title,$clusterNum);
	}
    }
}

