#!/usr/bin/env perl

# File: plotScoreHist.pl
# Copyright (c) 2005, 2006 Solexa
# Author: A. J. Cox
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

# Description:
# Produces histogram of alignment score vs. num of clusters

use warnings;
use strict;

use FindBin;
use lib "$FindBin::Bin/../lib";
use Gerald::Common;

my $version;
my $id;
{
    my $cvs_id='@(#) $Id: plotScoreHist.pl,v 1.2 2008/07/08 15:12:10 craczy Exp $';
    my $cvs_version='$Revision: 1.2 $';
    # Make CVS tag into a displayable string
    $version=$cvs_version;
    $version=~s/\$//g;
    $version=~s/Revision//;
    $id=$cvs_id;
    $id=~s/\$//g;
} #scope


my $binSize=0.5;
my $readLength;
my @score;

sub readScoreFile
{
    my ($scoreFileName)=@_;
    my ($line, $i, $readAs);
    my @sp;

    open ( FILE, $scoreFileName) 
        || die "Problem opening $scoreFileName: $!";

    while (defined($line=<FILE>))
    {
        last if ($line=~/^>/);
    } # while
    return 0 unless defined (($line)&&($line=~/^>/));
    @score=();
    while (1)
    {
        $line=~ s/^>//;
        chomp($line);
        @sp=split("\t", $line);
        ($i, $readAs)=@sp;
        $i--;
        $score[$i]{$readAs."A"}=$sp[2];
        $score[$i]{$readAs."C"}=$sp[3];
        $score[$i]{$readAs."G"}=$sp[4];
        $score[$i]{$readAs."T"}=$sp[5];

	# Next line needed because in output derived from IMPALA we don't
	# know genomic base in the case where the sequenced base was a blank
	$score[$i]{'..'}=0;

        $line=<FILE>;
        last unless (defined($line)&&($line=~/^>/));
    } # while

    close (FILE);

    return 1;
} # sub readScoreFile

sub readAndScore
{
    my ($alignFileName, $histRef)=@_;
 #   my (%hist)=%{$_[1]};
    my @a;
    my $i;
    my $thisScore;

    open ( FILE, $alignFileName) 
        || die "Problem opening $alignFileName: $!";

    while (<FILE>)
    {
        chomp;
        next if (/^\#/);
        @a=split;
        next unless defined ($a[5]);
        if ($a[4] eq "R")
        {
            $a[5]=reverse($a[5]);
            $a[5]=~tr/acgtACGT/tgcaTGCA/;
        } # if
        $thisScore=0;
        for ($i=0;$i<length($a[0]);$i++)
        {
            $thisScore+=$score[$i]{substr($a[0],$i,1).substr($a[5],$i,1)};
        } # for

#    print "$a[1] ", $binSize*int($a[1]/$binSize)}, "\n";
        $thisScore/=(length($a[0])*10);
#        print "$_ $thisScore\n";
        $histRef->{$binSize*int($thisScore/$binSize)}++;
        
    } #while
    close (FILE);
    return ((scalar(@a)==0)?0:1);
} # sub readAndScore


my $name;
my $tmpName;
my @a;
#my $foundStart;
#my %hist;

#die "Usage: cat *align.txt | $0 outFile.png "
#    unless (@ARGV==1);

my %hist1;
my %hist2;
my %histPrb;


die "Usage: $0 out.png align1.txt score1.txt"
    ." [align2.txt score2.txt [prb.txt [useBases]]]"
    unless ((@ARGV==3)||(@ARGV==5)||(@ARGV==6)||(@ARGV==7));

my ($outName, $alignFile1, $scoreFile1, $alignFile2,
    $scoreFile2, $prbFile, $useBases )
    = @ARGV;


if (readScoreFile($scoreFile1))
{
    readAndScore( $alignFile1, \%hist1);
} # if
else
{
    # make a blank file
    open (FILE, ">$outName");
    exit (0);
}

if (defined($scoreFile2))
{
    if (readScoreFile($scoreFile2))
    {
        readAndScore( $alignFile2, \%hist2);
    } # if
} # if

if (defined($prbFile))
{
    unless (-e $prbFile )
    {
	die "Could not find file $prbFile";
    } # unless
    if (-z $prbFile )
    {
	warn "File $prbFile is empty, no prb info to plot";
    } # if
    else
    {
	$readLength=scalar(@score);
	print "Assuming read length of $readLength\n";
	$useBases="all" unless defined ($useBases);
	if ($useBases eq "all")
	{
	    $useBases= "Y" x $readLength;
	} # if
	elsif ($useBases eq "odd")
	{
	    $useBases= "YN" x ($readLength);
	    $useBases=substr($useBases,0,length($useBases)-1);
	} # elsif
	else
	{
	    die "Use bases string $useBases must be odd, all or YNYN..."
		unless ($useBases=~/^[YNyn]+$/);
	    $useBases = uc ($useBases);
	} # else

	my $thisScore;
	my $bestScore;
	my $i;
	my @aa;
	my @ub=split(//,$useBases);
	my @baseIndex;
	for ($i=0;$i<@ub;$i++)
	{
	    push (@baseIndex, $i) if ($ub[$i] eq "Y"); 
	} # if

	die "Need $readLength bases of base information, only ", 
	scalar(@baseIndex), " specified in $useBases"
	    unless (scalar(@baseIndex)>=$readLength);
	@baseIndex=splice(@baseIndex,0,$readLength);


	open ( FILE, $prbFile) 
	    || die "Problem opening $prbFile: $!";

	while (<FILE>)
	{
	    chomp;
	    next if (/^\#/);
	    @a=split('\t');
	    $thisScore=0;
	    for ($i=0; $i<@baseIndex; $i++)
	    {
#        for ($i=0; $i<@ub; $i++)
#        {
#            if ($ub[$i] eq 'Y')
#            {
                @aa=split(' ',$a[$baseIndex[$i]]);
                die unless (@aa==4);
                $bestScore=$aa[0];
                $bestScore=$aa[1] if ($aa[1]>$bestScore);
                $bestScore=$aa[2] if ($aa[2]>$bestScore);
                $bestScore=$aa[3] if ($aa[3]>$bestScore);
                $thisScore+=$bestScore;
#            } # if
	    } # for i
	    $thisScore/=$readLength;
	    $histPrb{$binSize*int($thisScore/$binSize)}++;
	}
    close (FILE);
    } # else
} # if defined prbFile

$tmpName="/tmp/tmp-plotHist-$>-$$";


#for (sort {$a<=>$b} keys %hist1)
#{
#    print "$_\t$hist1{$_}\n";
#}

my $plotCommand="plot '$tmpName.dat' using 1:2 title 'actual - raw' with lines lt 1 lw 24";

my @bins=keys (%hist1);

if (((keys %histPrb)!=0)&&(keys %hist2==0))
{
    $hist2{$_}=0 foreach (keys %histPrb);
}

if ((keys %hist2) != 0 )
{
    push @bins, keys (%hist2);
#    for (sort {$a<=>$b} keys %hist2)
#    {
#        print "$_\t$hist2{$_}\n";
#        
#    } # for
    $plotCommand.=", '$tmpName.dat' using 1:3 title 'actual - filtered' with lines lt 2 lw 24";        
} # if

if ((keys %histPrb) != 0 )
{
#    print "HELLO\n";
    push @bins, keys (%histPrb);
#    for (sort {$a<=>$b} keys %histPrb)
#    {
#        print "$_\t$histPrb{$_}\n";
#        
#    } # for
    $plotCommand.=", '$tmpName.dat' using 1:4 title 'estimated' with lines lt 3 lw 24";        
} # if


@bins = sort {$a<=>$b} @bins;

my $minBin=$bins[0];
my $maxBin=$bins[-1];
#print "$minBin $maxBin\n";

open (OUT, ">$tmpName.dat")
    || die "Problem opening $tmpName.dat for writing: $!";

for (my $i=$minBin; $i<=$maxBin; $i+=$binSize)
{
    $hist1{$i}=0 unless defined ($hist1{$i});
    print OUT "$i $hist1{$i}";
    if ((keys %hist2) != 0 )
    {
        $hist2{$i}=0 unless defined ($hist2{$i});
        print OUT " $hist2{$i}";        
    }
    if ((keys %histPrb) != 0 )
    {
        $histPrb{$i}=0 unless defined ($histPrb{$i});
        print OUT " $histPrb{$i}";        
    }
    print OUT "\n";
} # for

close (OUT);

open (OUT, ">${tmpName}.txt") 
    || die "Problem opening $tmpName.txt for writing: $!";

print OUT << "END";

    set terminal postscript portrait color solid;
#    set nokey;

    set xrange[-5:35];
    set xtics 5;
    set yrange [0:1200];
    set ytics 200;

    set size 1.2, 1.0;

    set title "Distribution of alignment scores for file $outName";
    set xlabel "Log-odds alignment score";
    set ylabel "Number of clusters";

    $plotCommand

END
 
    close (OUT);
    gnuplotImage($tmpName.".txt", $outName);
    unlink "$tmpName.dat", "$tmpName.txt";

