#!/usr/bin/env python
#
# Copyright (c)  2007 Solexa
# Author: Richard Shaw
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

import sys, os, os.path, string
GA_LIB_LIST=[os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), "..", "lib")]
sys.path = GA_LIB_LIST + sys.path
import runfolder, common

from xml.sax import make_parser, ContentHandler

"""Parse the first cycle image analysis XML QC files for cluster statistics and
output per-lane cluster statistics."""

#------------------------------------------------------------------------------

def usage():
    print "Usage: %s <XML file> [<XML file> ...]" % sys.argv[0]
    sys.exit(0)

#------------------------------------------------------------------------------

class QCMHandler(ContentHandler):
    def __init__(self):
        self.in_cluster_stats = 0
        self.in_cluster_fwhm = 0
        self.tag_name = ''
        self.tag_val = ''
        self.images = []

    def startElement(self, name, attrs):
        if (name == 'Image'):
            self.images.append({})
        if (name == 'ClusterStatistics'):
            self.in_cluster_stats = 1
        if ((name == 'FWHM') and (self.in_cluster_stats)):
            self.in_cluster_fwhm = 1
        if (((name == 'Count') and self.in_cluster_stats)
            or ((name in ['Mean', 'Median', 'StandardDeviation'])
                and self.in_cluster_fwhm)):
            self.tag_name = name
            self.tag_val = ''
        if name in [ 'ChannelNumber', 'TileNumber',
                     'CycleNumber', 'FilterNumber']:
            self.tag_name = name
            self.tag_val  = ''

    def endElement(self, name):
        if ((name == self.tag_name) and name):
            if (name in ['Count','ChannelNumber', 'TileNumber',
                         'CycleNumber', 'FilterNumber'] ):
                self.images[-1][name] = int(self.tag_val)
            else:
                self.images[-1][name] = float(self.tag_val)

            self.tag_name = ''

        if (name == 'FWHM'):
            self.in_cluster_fwhm = 0
        if (name == 'ClusterStatistics'):
            self.in_cluster_stats = 0

    def characters(self, text):
        if (self.tag_name):
            self.tag_val += text

#------------------------------------------------------------------------------

if __name__ == "__main__":
    if len(sys.argv) < 2:
        usage()
    files = sys.argv[1:]
    if len(files) == 1 and files[0] == '-': # read from stdin
        files = map(string.strip, sys.stdin.readlines())

    handler = QCMHandler()
    saxparser = make_parser()
    saxparser.setContentHandler(handler)

    for file in files:
        try:
            saxparser.parse(open(file))
        except:
            common.warning("Could not parse %s; skipping." % file)
            continue

    images = handler.images

    for image in images:
        print "%i %3i %2i %i %4.1f %4.1f %6i" %\
              (image['ChannelNumber'], image['TileNumber'],
               image['CycleNumber'],   image['FilterNumber']-1,
               image['Mean'], image['Median'], image['Count'])

    if (len(images) == 0):
        common.warning("No image statistics extracted so no output.")

#------------------------------------------------------------------------------
