#!/usr/bin/env python
#
# Copyright (c) 2006 Solexa
# Author: Klaus Maisinger
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

import re, sys, os, os.path, getopt
GA_LIB_LIST=[os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), "..", "lib")]
sys.path = GA_LIB_LIST + sys.path
sys.path.append(os.path.join(os.path.split(sys.argv[0])[0], ".."))
import XMLSimple, common

"""Reads in a list of phasing values given by a flat file format
where each row contains lane, tile, phasing value, prephasing value
and possbily other parameters. Selects a median phasing/prephasing
value per lane, excluding negative values, which are obviously not
reliable estimates."""

def usage():
    print "Usage: select_phasing.py <phasing summary filename>"
    
def parse_lines(lines):
    """Parse lines of a pahsing summary file. The format is assumed 
to be lane<TAB>tile<TAB>phasing<TAB>prephasing..."""
    phasinglist = {}
    for line in lines:
        elements = line.split("\t")
        if (len(elements) > 4):
            try:
                lane = int(elements[0])
            except:
                common.warning("Lane index not integer.")
                continue
            try:
                phasing = [ float(elements[2]), float(elements[3]) ]
            except:
                common.warning("Value not floats.")
                continue
            phasinglist.setdefault(lane, []).append(phasing)
        else:
            common.warning("File %s corrupted." % filename)
    return phasinglist

if __name__ == "__main__":

    try:
        (options, args) = getopt.getopt(sys.argv[1:], "", [])
    except getopt.GetoptError:
        usage()
        sys.exit(1)
    if (len(args) != 1):
        usage()
        sys.exit(2)

    filename = args[0]
    if filename == "-":
        fd = sys.stdin
    else:
        try:
            fd = open(filename)
        except:
            common.error("Failed to open %s\n" % filename)
            
    # read phasing/prephasing values and sort store them by lane
    lines = []
    try:
        lines = fd.readlines()
    except:
        common.error("Failed to read %s\n" % filename)

    lanephasing = parse_lines(lines)

    # summarise all lanes
    phasing = [ [ 0.], [0.] ]
    num = 1
    if len(lanephasing.values()) > 0:
        # make sure phasing values are positive
        # the estimator is totally unreliable for non-genomic samples...
        threshold = 0.00
        phasinglist = reduce(lambda x, y: x+y, lanephasing.values())
        phasinglist = filter(lambda x: x[0] > threshold
                             and x[1] > threshold,
                             phasinglist)
        if len(phasinglist) > 0:
            num = len(phasinglist)
            phasing = [ [ 0. ] * num, [ 0. ] * num ]
            for i in range(2):
                for j in range(len(phasinglist)):
                    phasing[i][j] = phasinglist[j][i]

    # calculate median of phasing values
    med = [ 0., 0. ]
    for i in range(2):
        phasing[i].sort()
        med[i] = phasing[i][len(phasing[i])/2]
    
    #sys.stdout.write("%6.4f %6.4f\n" % (med[0], med[1]))
    sys.stdout.write(XMLSimple.XMLout({ "Phasing": float(med[0]),
                                        "Prephasing": float(med[1]) },
                                      RootName = "Parameters"))
