#!/usr/bin/env python
#
# Copyright (c) 2006 Solexa
# Author: Klaus Maisinger
# This source file is covered by the "Solexa Public Source License"
# agreement and bound by the terms therein.

"""Simple matrix read and write operations."""

import sys

def read_matrix(filename):
    """Read a frequency response matrix in the pipeline matrix format.""" 
    bases = []
    matrix = []
    lines = []
    try:
        lines = open(filename).readlines()
    except:
        sys.stderr.write("Error reading matrix file %s\n" % filename)

    for line in lines:
        if line[0] == "#":
            continue
        if line[0] == ">":
            bases.append(line.split()[1])
        else:
            elements = map(float, line.split())
            matrix.append(elements)
    return matrix

def format_matrix(matrix):
    """Format a matrix as a string in the pipeline matrix format."""
    lines = []
    lines.append("# Auto-generated frequency response matrix\n")
    for base in [ 'A', 'C', 'G', 'T' ]:
        lines.append("> %s\n" % base)
    for i in range(4):
        line =  ""
        for j in range(4):
            line += ("%4.2f "%(matrix[i][j]))
        lines.append(line+'\n')
    return lines
    
def write_matrix(filehandle, matrix):
    """Write out matrix in pipeline matrix format."""
    lines = format_matrix(matrix)
    try:
        filehandle.writelines(lines)
    except:
        sys.stderr.write("Error: Failed to write output!\n")
    

