import unittest
import sys
sys.path.append("../")
from CycleSelector import CycleSelector
from ImagingProtocol import ImagingProtocol

class CycleSelectorTestCase(unittest.TestCase):
    def setUp(self):
        self.folderName1 = "my_folder1";
        self.folderList1 = ["C1.1", "C2.1", "C3.1", "C4.1", "C5.1", "C6.1"]
        self.folderName2 = "my_folder2";
        self.folderList2 = ["C1.1", "C2.1", "C3.1", "C4.1", "C5.1"]
        self.cycleSelectionString = "2-6,8-auto"
        self.cycleSelector = CycleSelector(self.cycleSelectionString);
        self.folderName1 = "my_folder1";
        self.folderList1 = ["C1.1", "C2.1", "C3.1", "C4.1", "C5.1", "C6.1"]
        self.folderName2 = "my_folder2";
        self.folderList2 = ["C1.1", "C2.1", "C3.1", "C4.1", "C5.1"]
        self.imagingProtocol = ImagingProtocol()
        self.imagingProtocol.addImagingFolder(self.folderName1, self.folderList1)
        self.imagingProtocol.addImagingFolder(self.folderName2, self.folderList2)
    def tearDown(self):
        self.cycleSelector = None
    def testConstructor(self):
        self.assertEqual(2, len(self.cycleSelector.cycleSelectionList))
        self.failIf    (self.cycleSelector.cycleSelectionList[0].autoAll)
        self.failIf    (self.cycleSelector.cycleSelectionList[0].autoLast)
        self.failIf    (self.cycleSelector.cycleSelectionList[1].autoAll)
        self.failUnless(self.cycleSelector.cycleSelectionList[1].autoLast)
        self.assertEqual(2, self.cycleSelector.cycleSelectionList[0].firstCycle)
        self.assertEqual(6, self.cycleSelector.cycleSelectionList[0].lastCycle)
        self.assertEqual(8, self.cycleSelector.cycleSelectionList[1].firstCycle)
    def testGetCycleList(self):
        cycleList = self.cycleSelector.getCycleList(self.imagingProtocol.getImagingReadList())
        self.assertEqual( 9, len(cycleList))
        self.assertEqual( 2, cycleList[0])
        self.assertEqual( 3, cycleList[1])
        self.assertEqual( 4, cycleList[2])
        self.assertEqual( 5, cycleList[3])
        self.assertEqual( 6, cycleList[4])
        self.assertEqual( 8, cycleList[5])
        self.assertEqual( 9, cycleList[6])
        self.assertEqual(10, cycleList[7])
        self.assertEqual(11, cycleList[8])
        
    def testGetReadList(self):
        readList = self.cycleSelector.getReadList(self.imagingProtocol.getImagingReadList())
        self.assertEqual(2, len(readList))
        self.assertEqual(1, readList[0].FirstCycle)
        self.assertEqual(5, readList[0].LastCycle)
        self.assertEqual(6, readList[1].FirstCycle)
        self.assertEqual(9, readList[1].LastCycle)
        self.assertEqual(1, readList[0].Index)
        self.assertEqual(2, readList[1].Index)
        self.assertEqual(self.folderName1, readList[0].RunFolder)
        self.assertEqual(self.folderName2, readList[1].RunFolder)

    def testGetReadListWithoutLastCycle(self):
        readList = self.cycleSelector.getReadList(self.imagingProtocol.getImagingReadList(), False)
        self.assertEqual(2, len(readList))
        self.assertEqual(1, readList[0].FirstCycle)
        self.assertEqual(5, readList[0].LastCycle)
        self.assertEqual(6, readList[1].FirstCycle)
        self.assertEqual(8, readList[1].LastCycle)
        self.assertEqual(1, readList[0].Index)
        self.assertEqual(2, readList[1].Index)
        self.assertEqual(self.folderName1, readList[0].RunFolder)
        self.assertEqual(self.folderName2, readList[1].RunFolder)

    def testInvalidParameters(self):
        self.assertRaises(ValueError, CycleSelector, "")
        self.assertRaises(ValueError, CycleSelector, "autox")
        self.assertRaises(ValueError, CycleSelector, "auto,1-4")
        self.assertRaises(ValueError, CycleSelector, "1-5,auto")
        self.assertRaises(ValueError, CycleSelector, "1-5,6-auto,")
        self.assertRaises(ValueError, CycleSelector, "1-5,6-autox")
        self.assertRaises(ValueError, CycleSelector, "0-5")
        self.assertRaises(ValueError, CycleSelector, "3-2")
        self.assertRaises(ValueError, CycleSelector, "a")
        self.assertRaises(ValueError, CycleSelector, "a-2")
        self.assertRaises(ValueError, CycleSelector, "2-a")
        self.assertRaises(ValueError, CycleSelector, "2-")
        self.assertRaises(ValueError, CycleSelector, "-2")
        self.assertRaises(ValueError, CycleSelector, "1-2-3")


#############################################
if __name__ == "__main__":
    unittest.main()
