import unittest
import sys
sys.path.append("../")
from ImagingProtocol import ImagingProtocol

class ImagingProtocolTestCase(unittest.TestCase):
    def setUp(self):
        self.folderName = "my_folder";
        self.cycleFolderList = ["C1.1", "C2.1", "C3.1", "C4.1", "C5.1", "C6.1"]
        self.folderName1 = "my_folder1";
        self.cycleFolderList1 = ["C1.1", "C2.1", "C3.1", "C4.1", "C5.1"]
        self.imagingProtocol = ImagingProtocol();
    def tearDown(self):
        self.imagingProtocol = None
    def testConstructor(self):
        self.assertEqual(0, len(self.imagingProtocol.imagingFolderList))
        self.assertEqual("", self.imagingProtocol.folderName)
    def testAddCycleRange(self):
        self.imagingProtocol.addImagingFolder(self.folderName, self.cycleFolderList)
        self.assertEqual(1, len(self.imagingProtocol.imagingFolderList))
        imagingFolder = self.imagingProtocol.imagingFolderList[0]
        self.assertEqual(self.folderName, imagingFolder.folderName)
        self.assertEqual(1, len(imagingFolder.imagingCycleList))
        imagingCycle = imagingFolder.imagingCycleList[0]
        self.assertEqual(self.cycleFolderList, imagingCycle.cycleFolderList)
    def testAddCycleRangeWithRestart(self):
        self.imagingProtocol.addImagingFolder(self.folderName, self.cycleFolderList, [3, 6])
        self.assertEqual(1, len(self.imagingProtocol.imagingFolderList))
        imagingFolder = self.imagingProtocol.imagingFolderList[0]
        self.assertEqual(3, len(imagingFolder.imagingCycleList))
        self.assertEqual(self.cycleFolderList[0:2], imagingFolder.imagingCycleList[0].cycleFolderList)
        self.assertEqual(self.cycleFolderList[2:5], imagingFolder.imagingCycleList[1].cycleFolderList)
        self.assertEqual(self.cycleFolderList[5:6], imagingFolder.imagingCycleList[2].cycleFolderList)
    def testGetImagingReadListSingle(self):
        self.imagingProtocol.addImagingFolder(self.folderName, self.cycleFolderList)
        imagingReadList = self.imagingProtocol.getImagingReadList()
        self.assertEqual(1, len(imagingReadList))
        self.assertEqual(1, imagingReadList[0].Index)
        self.assertEqual(self.folderName, imagingReadList[0].RunFolder)
        self.assertEqual(1, imagingReadList[0].FirstCycle);
        self.assertEqual(len(self.cycleFolderList), imagingReadList[0].LastCycle);
    def testGetImagingReadListDouble(self):
        self.imagingProtocol.addImagingFolder(self.folderName, self.cycleFolderList)
        self.imagingProtocol.addImagingFolder(self.folderName1, self.cycleFolderList1)
        imagingReadList = self.imagingProtocol.getImagingReadList()
        self.assertEqual(2, len(imagingReadList))
        self.assertEqual(1, imagingReadList[0].Index);
        self.assertEqual(2, imagingReadList[1].Index);
        self.assertEqual(self.folderName, imagingReadList[0].RunFolder);
        self.assertEqual(self.folderName1, imagingReadList[1].RunFolder);
        self.assertEqual(1, imagingReadList[0].FirstCycle);
        self.assertEqual(len(self.cycleFolderList), imagingReadList[0].LastCycle);
        self.assertEqual(imagingReadList[0].LastCycle + 1, imagingReadList[1].FirstCycle);
        self.assertEqual(len(self.cycleFolderList) + len(self.cycleFolderList1), imagingReadList[1].LastCycle);
    def testGetImagingReadListRestart(self):
        self.imagingProtocol.addImagingFolder(self.folderName, self.cycleFolderList, [3, 4])
        imagingReadList = self.imagingProtocol.getImagingReadList()
        self.assertEqual(3, len(imagingReadList))
        self.assertEqual(1, imagingReadList[0].Index);
        self.assertEqual(2, imagingReadList[1].Index);
        self.assertEqual(3, imagingReadList[2].Index);
        self.assertEqual(self.folderName, imagingReadList[0].RunFolder);
        self.assertEqual(self.folderName, imagingReadList[1].RunFolder);
        self.assertEqual(self.folderName, imagingReadList[2].RunFolder);
        self.assertEqual(1, imagingReadList[0].FirstCycle);
        self.assertEqual(3, imagingReadList[1].FirstCycle);
        self.assertEqual(4, imagingReadList[2].FirstCycle);
        self.assertEqual(2, imagingReadList[0].LastCycle);
        self.assertEqual(3, imagingReadList[1].LastCycle);
        self.assertEqual(len(self.cycleFolderList), imagingReadList[2].LastCycle);
    def testGetImagingReadListCombined(self):
        restartCycleList = [3, 4, 10]
        self.imagingProtocol.addImagingFolder(self.folderName, self.cycleFolderList, restartCycleList)
        self.imagingProtocol.addImagingFolder(self.folderName1, self.cycleFolderList1, restartCycleList)
        imagingReadList = self.imagingProtocol.getImagingReadList()
        self.assertEqual(5, len(imagingReadList))
        self.assertEqual(1, imagingReadList[0].Index);
        self.assertEqual(2, imagingReadList[1].Index);
        self.assertEqual(3, imagingReadList[2].Index);
        self.assertEqual(4, imagingReadList[3].Index);
        self.assertEqual(5, imagingReadList[4].Index);
        self.assertEqual(self.folderName, imagingReadList[0].RunFolder);
        self.assertEqual(self.folderName, imagingReadList[1].RunFolder);
        self.assertEqual(self.folderName, imagingReadList[2].RunFolder);
        self.assertEqual(self.folderName1, imagingReadList[3].RunFolder);
        self.assertEqual(self.folderName1, imagingReadList[4].RunFolder);
        self.assertEqual(1, imagingReadList[0].FirstCycle);
        self.assertEqual(3, imagingReadList[1].FirstCycle);
        self.assertEqual(4, imagingReadList[2].FirstCycle);
        self.assertEqual(7, imagingReadList[3].FirstCycle);
        self.assertEqual(10, imagingReadList[4].FirstCycle);
        self.assertEqual(len(self.cycleFolderList+self.cycleFolderList1), imagingReadList[-1].LastCycle);
    def testInvalidParameters(self):
        self.assertRaises(ValueError, self.imagingProtocol.addImagingFolder, "", [])
        self.assertRaises(ValueError, self.imagingProtocol.addImagingFolder, "a", [])
        self.assertRaises(ValueError, self.imagingProtocol.addImagingFolder, "", [0])
        self.assertRaises(ValueError, self.imagingProtocol.addImagingFolder, "a", ["C1"], [0])
        self.assertRaises(ValueError, self.imagingProtocol.addImagingFolder, "a", ["C1"], [1])
        #self.assertRaises(ValueError, self.imagingProtocol.addImagingFolder, "a", ["C1", "C2"], [3])
        self.assertRaises(ValueError, self.imagingProtocol.addImagingFolder, "a", ["C1", "C2", "C3"], [2, 1])

#############################################
if __name__ == "__main__":
    unittest.main()
